package gui;

import javax.swing.tree.DefaultMutableTreeNode;

import protein.Protein;
/*Class for the Komponent nodes*/
public class DefaultMutableTreeNodeProtein extends DefaultMutableTreeNode {
	public int percent = 1;
	public String file ="";
	public String pdbName ="";
	public String nodeName ="";
	public Protein protein;
	public int min_percent = 0;
	
	/**
	 * Creates a node for a protein in the membrane model tree.
	 * @param nodeName node name of the protein node.
	 * @param pdbName pdb code of the protein node.
	 * @param percent percent value of the protein in the membrane.
	 */
	public DefaultMutableTreeNodeProtein(String nodeName, String pdbName, int percent){
		this.percent = percent; 
		this.pdbName = pdbName;
		this.nodeName = nodeName;
		this.setUserObject(nodeName);
	}
	/**
	 * Returns the percent value of the protein in the membrane.
	 * @return Returns the percent.
	 */
	public int getPercent(){
		return percent;
	}
	/**
	 * Sets the percent value of the protein in the membrane.
	 * @param a The percent to set.
	 */
	public void setPercent(int a){
		this.percent = a;
	}
	
	/**
	 * Sets the filepath of the corresponding pdb file of the protein.
	 * @param file The file to set.
	 */
	public void setFile(String file){
		this.file = file;
	}
	
	/**
	 *  Returns filepath of the corresponding pdb file of the protein.
	 * 	 @return Returns the file.
	 */
	public String getFile(){
		return file;
	}
	
	/**
	 * Sets the pdbname of the corresponding protein object.
	 * @param name The name to set.
	 */
	public void setPDBName(String name){
		this.pdbName = name;
	}
	
	/**
	 * Returns the pdb code of the corresponding protein.
	 * @return Returns pdbName.
	 */
	public String getPDBName(){
		return pdbName;
	}
	
	/**
	 * Sets the protein object to the node.
	 * @param protein The protein to set.
	 */
	public void setProtein(Protein protein){
		this.protein = protein;
	}
	
	/**
	 * Returns the protein at this node.
	 * @return Returns the protein.
	 */
	public Protein getProtein(){
		return protein;
	}

	/**
	 * Returns the name of the node in the tree.
	 * @return Returns the nodeName.
	 */
	public String getNodeName() {
		return nodeName;
	}

	/**
	 * Sets the node name of the node in the tree.
	 * @param nodeName The nodeName to set. 
	 */
	public void setNodeName(String nodeName) {
		this.nodeName = nodeName;
		this.setUserObject(nodeName);
	}

	/**
	 * Returns the minimal percent value for the protein at the node in lipid membrane.
	 * @return Returns the min_percent.
	 */
	public int getMin_percent() {
		return min_percent;
	}

	/**
	 * Sets the minimal percent value for the protein at the node in lipid membrane.
	 * @param min_percent The min_percent to set.
	 */
	public void setMin_percent(int min_percent) {
		this.min_percent = min_percent;
	}

	
}
