package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.WindowConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.TreeModelListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
import gui.dragndrop.*;

public class HelpPanel extends JDialog implements HyperlinkListener,
		ActionListener, MouseListener {
	private JEditorPane editorPaneSection;

	protected JTextArea txar;

	protected JButton cancel;

	protected JScrollPane scrollPane;

	private Box topbox;

	private Box bottombox;

	private Box mainbox;

	private JTree tree;
	
	/**
	 * Creates a JDialog for the help html pages.
	 *
	 */
	public HelpPanel() {
		
		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		sp.setOneTouchExpandable(true);
		sp.setBackground(new Color(240, 240, 255));
		sp.setContinuousLayout(true);
		sp.setDoubleBuffered(true);
		sp.setOpaque(true);

		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Content");
		DefaultMutableTreeNode general = new DefaultMutableTreeNode("General");
		DefaultMutableTreeNode wizard = new DefaultMutableTreeNode("Wizard");
		DefaultMutableTreeNode overview = new DefaultMutableTreeNode("Overview");

		root.add(general);
		root.add(wizard);
		root.add(overview);

		overview.add(new DefaultMutableTreeNode("PDB database"));
		overview.add(new DefaultMutableTreeNode("Membrane tree"));
		overview.add(new DefaultMutableTreeNode("Protein Preview"));
		overview.add(new DefaultMutableTreeNode("Membrane Preview"));
		overview.add(new DefaultMutableTreeNode("Statistic"));
		overview.add(new DefaultMutableTreeNode("Properties"));

		DefaultTreeModel model = new DefaultTreeModel(root);
		tree = new JTree(model);

		tree.setRootVisible(true);
		tree.setEditable(false);
		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setShowsRootHandles(true);
		tree.addMouseListener(this);
		tree.setBackground(new Color(224,224,224));// fr default msste eigentlich sein:#F0F0FF
		DefaultTreeCellRenderer linksRenderer = new DefaultTreeCellRenderer();
        linksRenderer.setBackgroundNonSelectionColor(new Color(224,224,224));
        linksRenderer.setBackgroundSelectionColor(new Color(240, 240, 255));
        linksRenderer.setOpenIcon(new ImageIcon("help/filler.png"));
        linksRenderer.setClosedIcon(new ImageIcon("help/filler.png"));
        linksRenderer.setLeafIcon(new ImageIcon("help/filler.png"));
        
        //linksRenderer.setLeafIcon(new ImageIcon("middle.gif"));
        tree.setCellRenderer(linksRenderer);

		MyTreeModelListener myTreeModelListener = new MyTreeModelListener(tree);
		model.addTreeModelListener(myTreeModelListener);

		editorPaneSection = new JEditorPane();
		editorPaneSection.setEditable(false);
		editorPaneSection.setEditorKit(new HTMLEditorKit());
		editorPaneSection.addHyperlinkListener(this);
		editorPaneSection.setBorder(BorderFactory.createLineBorder(new Color(224,224,224),5));

		
		JScrollPane js = new JScrollPane(tree);
		JScrollPane js2 = new JScrollPane(editorPaneSection);
		sp.setDividerLocation(sp.getMinimumDividerLocation() + 175);
		sp.setLeftComponent(js);
		sp.setRightComponent(js2);

		try {
			editorPaneSection.setPage(getClass().getResource(
					"../help/introduction.html"));
		} catch (Exception ex) {
			System.out.print(ex.toString());
		}

		this.setTitle("Help");
		this.getContentPane().add(sp);
		this.setVisible(true);
		this.setSize(850, 600);
		this.setFocusable(true);
		this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		show();
	}
	
	/**
	 * Performs the action events for the JDialog.
	 * 	 
	 * @param arg0 ActionEvent
	 */
	public void actionPerformed(ActionEvent arg0) {
		dispose();

	}

	/**
	 * Performs the mouse event for the JDialog. 
	 * @param e MouseEvent
	 */
	public void mouseClicked(MouseEvent e) {
		TreePath path = tree.getPathForLocation(e.getX(), e.getY());
		tree.setSelectionPath(path);
		if ((!(e.isMetaDown()))){
			DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree
			.getLastSelectedPathComponent();
			if(selectedNode != null){
				if(selectedNode.toString().equals("General")){
					setPage("general.html");
				}
				if(selectedNode.toString().equals("Wizard")){
					setPage("wizard.html");
				}				
				if(selectedNode.toString().equals("Overview")){
					setPage("overview.html");
				}
				if(selectedNode.toString().equals("PDB database")){
					setPage("dbtree.html");
				}
				if(selectedNode.toString().equals("Membrane tree")){
					setPage("membrantree.html");
				}
				if(selectedNode.toString().equals("Protein Preview")){
					setPage("jmol.html");
				}
				if(selectedNode.toString().equals("Statistic")){
					setPage("statistic.html");
				}
				if(selectedNode.toString().equals("Properties")){
					setPage("properties.html");
				}
				if(selectedNode.toString().equals("Membrane Preview")){
					setPage("membranemodel.html");
				}
			}
		}
	}
			
				
		private void setPage(String string) {
			try 
			{
				File f = new File(".");
				f = new File(f.toString().substring(0,f.toString().length()-1));
				editorPaneSection.setPage(new URL("file:\\"+f.getAbsolutePath()+"\\help\\"+string));

			} 
			catch (Throwable t)
			{
				t.printStackTrace();
			}
		
	}
/**
 * not implemented
 * @param arg0 MouseEvent
 */
		public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}
		/**
		 * not implemented
		 * @param arg0 MouseEvent
		 */
	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}
	/**
	 * not implemented
	 * @param arg0 MouseEvent
	 */
	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}
	/**
	 * not implemented
	 * @param arg0 MouseEvent
	 */
	public void mouseReleased(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/**
	 * Updates the help pages after mouse clicked event was succesful.
	 * @param e HyperlinEvent
	 */
	public void hyperlinkUpdate(HyperlinkEvent e) {
		if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

			if (e instanceof HTMLFrameHyperlinkEvent) {
				HTMLDocument html_index = (HTMLDocument) editorPaneSection
						.getDocument();
				html_index
						.processHTMLFrameHyperlinkEvent((HTMLFrameHyperlinkEvent) e);
			} else {
				try {
					editorPaneSection.setPage(e.getURL());
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}
		}

	}

}
