 /**
 * CVS-informations
 * 
 * $Author: bsommer $
 * $Date: 2006/05/23 09:49:42 $
 * $Revision: 1.4 $
 * 
 */
package gui;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/**
 * This class contains the introduction popup which is shown, when the program is
 * started and can be opened again in the MainMenu by clicking on Help -> Introduction.
 */
public class Introduction extends JDialog implements ActionListener {

	public JTextField description, name;
	public boolean returnValue;
	
	/**
	 * Initializes and shows the Introduction screen.
	 * It gets the main in order to set its own position in relation to the main-JFrame.
	 * It contains a textarea with the Introduction-text, the "OK"-button to close the
	 * window and the "Help"-button to initialize the Help-window. 
	 * @param main Main
	 */
	public Introduction(Main main){
		this.setTitle("Introduction for membrane editor");
		
		this.requestFocus(true);
		this.setSize(320, 400);
		this.setLocationRelativeTo(main);

		JTextArea intro = new JTextArea(1,1);
		intro.setBackground(null);
		intro.setFocusable(false);
		intro.setFont(new Font("Dialog", Font.BOLD ,12));
		intro.setBorder(BorderFactory.createTitledBorder(""));
		intro.setText("In order to create your PDB database right-click\n" +
				"on the root of the tree on the right side and add\n" +
				"a PDB file.\n\n"+
				"Please drag the proteins in the PDB database \n" +
				"on the right side into the tree on the left side of\n" +
				"the window. The tree is divided into\n" +
				"ionchannels, porins and receptors. Please \n" +
				"place the PDB into the correct folder.\n"+
				"Then follow the instructions.\n\n" +
				"If you want to show, rename or edit a node\n"+
				"(Membrane Model, Component, PDB) contained\n"+
				"in the tree on the left side, just right-click\n" +
				"on a node and choose an option. \n\n\n" +
				"For further information have a look at the\n" +
				"Help/Contents in the menu or click on Help button.");

		
		Box introbox = new Box(BoxLayout.X_AXIS);
		introbox.add(Box.createHorizontalStrut(15));
		introbox.add(intro);
		introbox.add(Box.createHorizontalStrut(15));

		JButton okey = new JButton("OK");
		JButton help = new JButton("Help");
		okey.addActionListener(this);
		help.addActionListener(this);
		
		//adding the buttons to the bottonbox
		Box buttonbox = new Box(BoxLayout.X_AXIS);
		buttonbox.add(Box.createVerticalStrut(30));
		buttonbox.add(okey);
		buttonbox.add(Box.createVerticalStrut(20));
		buttonbox.add(help);
		buttonbox.add(Box.createVerticalStrut(30));

		//adding the topbox and the bottumbox to the mainbox
		Box mainbox = new Box(BoxLayout.Y_AXIS);
		mainbox.add(Box.createVerticalStrut(10));
		mainbox.add(introbox);
		mainbox.add(Box.createVerticalStrut(10));
		mainbox.add(buttonbox);
		mainbox.add(Box.createVerticalStrut(20));
		getContentPane().add(mainbox);
		show();		
	}
	
	/**
	 *  The action which is performed when the user is clicking on the 
	 *  "OK" or "Help" Buttons.
	 *  @param arg0 ActionEvent
	 */
	public void actionPerformed(ActionEvent arg0) {
		String actionCommand = arg0.getActionCommand();
		if(actionCommand.equals("OK")){
				returnValue = true;
				dispose();
			
		}
		if(actionCommand.equals("Help")){
				returnValue = false;
			    HelpPanel help =  new HelpPanel();
				dispose();
		}		
		
	}

	/**
	 * Returns the value of the actionPerformed function.
	 * @return returnValue boolean
	 */
	public boolean getReturnValue() {
		return returnValue;
	}



}


