package gui;

import gui.dragndrop.MyTreeModelListener;
import gui.messages.ErrorMessage;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.io.File;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileSystemView;
import javax.swing.filechooser.FileView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import membranModel.MembranModelSingleton;




public class JImport extends JPanel implements TextListener, MouseListener, ActionListener  {

	public JTree tree;
	public JTree original;
	private TextField searchText;
	public DefaultTreeModel model, modelOri;
	private DefaultMutableTreeNode root, rootOri;
	private JPopupMenu popUpPDB, popUpRoot;
	private JMenuItem pdbDelete, pdbAdd, renameDB;
	public NewProteinDialog npd;
	
	/**
	 * Creates the panel with the database tree in the PDB Database tab.
	 *
	 */
	public JImport(){
		BorderLayout layout = new BorderLayout();
		this.setLayout(layout);

		root = new DefaultMutableTreeNode(" ");
		model = new DefaultTreeModel(root);
		tree = new JTree(model);
		tree.setRootVisible(false);
		tree.setEditable(false);
		rootOri = new DefaultMutableTreeNode("null");
		modelOri = new DefaultTreeModel(rootOri);
		original = new JTree(modelOri);
		
		
		tree.setBackground(MembranModelSingleton.getInstance().getBackground_color());
		DefaultTreeCellRenderer linksRenderer = new DefaultTreeCellRenderer();
        linksRenderer.setBackgroundNonSelectionColor(MembranModelSingleton.getInstance().getBackground_color());
        linksRenderer.setBackgroundSelectionColor(MembranModelSingleton.getInstance().getSelection_color());
        //linksRenderer.setLeafIcon(new ImageIcon("middle.gif"));
        tree.setCellRenderer(linksRenderer);

		TreeSelectionModel tsm = new DefaultTreeSelectionModel();
		tsm.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setSelectionModel(tsm);
		
	    JScrollPane js = new JScrollPane(tree);

		this.add(js);


		MyTreeModelListener myTreeModelListener = new MyTreeModelListener(tree);
	    model.addTreeModelListener(myTreeModelListener);
		tree.addMouseListener(this);

		MyTreeModelListener myTreeModelListener2 = new MyTreeModelListener(original);
	    modelOri.addTreeModelListener(myTreeModelListener2);

		

		original.setEditable(false);
        tree.setEditable(false);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
		
		
		Box box1 = Box.createHorizontalBox();
		JLabel searchLabel = new JLabel("Search  ");
		searchText = new TextField();
		searchText.addTextListener(this);
		JLabel platzhalter = new JLabel("              ");
		box1.add(searchLabel);
		box1.add(searchText);
		box1.add(platzhalter);
		this.add(box1, BorderLayout.SOUTH);
		
		popUpPDB = new JPopupMenu();
		popUpRoot = new JPopupMenu();
		pdbDelete = new JMenuItem("Delete PDB");
		pdbDelete.addActionListener(this);
		popUpPDB.add(pdbDelete);
		pdbAdd = new JMenuItem("Add PDB");
		pdbAdd.addActionListener(this);
		popUpRoot.add(pdbAdd);
		renameDB = new JMenuItem("Rename Database");
		renameDB.addActionListener(this);
		popUpRoot.add(renameDB);
		repaint();
		
	}
	
	
	/**
	 * Returns if a tree was generated after a dialog.
	 * @return boolean (true, if a tree was generated)  
	 */
	public boolean createNewDB() {
		npd = new NewProteinDialog(((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).getMain());
		if(npd.getReturnValue()){
			String newName = npd.getName();
			while(newName.equals("")){
				JOptionPane.showMessageDialog(
						this,
						"Please enter a name for the new database! "
							);
				npd = new NewProteinDialog(((JWorkPanel) SwingUtilities.getAncestorNamed("workpanel", this)).getMain());
				 
			}
			newTree(newName);
			return true;
		}
		else{
		return false;
		}
	}
	
	/**
	 * Initialize a new PDB tree in the JPanel.
	 * @param rootName root name of the new PDB tree.
	 */
	public void newTree(String rootName){
		root.removeAllChildren();
		rootOri.removeAllChildren();
		
		model.reload();
		modelOri.reload();
		root.setUserObject(rootName);
		rootOri.setUserObject(rootName);
		
		model.nodeChanged(root);
		modelOri.nodeChanged(rootOri);
		
		tree.setRootVisible(true);

	}


	
	/**
	 * The tree to set in the panel after loading process.
	 * @param newTree tree to load in the panel.
	 */
	public void setTree(JTree newTree){
		root.removeAllChildren();
		rootOri.removeAllChildren();
		tree.setRootVisible(true);
		model.reload();
		modelOri.reload();
		TreeModel newModel = newTree.getModel();
		DefaultMutableTreeNode newRoot = (DefaultMutableTreeNode) newModel.getRoot();
		root.setUserObject(newRoot.toString());
		int count = newRoot.getChildCount();
		for(int i = 0; i<count; i++){
			DefaultMutableTreeNodeFile newChild = (DefaultMutableTreeNodeFile) newRoot.getChildAt(i);
			String name = newChild.getName();
			String file = newChild.getFile();
			model.insertNodeInto(new DefaultMutableTreeNodeFile(name, file),root,i);
			modelOri.insertNodeInto(new DefaultMutableTreeNodeFile(name, file),rootOri,i);
		}
		model.reload(); 
			
	}
	
/**
 * Actions after text was changed.
 */
	public void textValueChanged(TextEvent arg0) {
		Search search = new Search();
		if (searchText.getText().length() != 0) {
			root.removeAllChildren();
			model.reload();
			tree = search.searchresults(original, tree, searchText.getText());
			model.reload();
			tree.expandPath(new TreePath(root));

				
		} else {
			updateTree(original);
		}
		
	}
	
	
	private void updateTree(JTree showTree) {
		root.removeAllChildren();
		model.reload();
		DefaultMutableTreeNode showTreeRoot = (DefaultMutableTreeNode) showTree.getModel().getRoot();
		for(int i = 0; i<showTreeRoot.getChildCount(); i++){
			
			DefaultMutableTreeNodeFile newChild = (DefaultMutableTreeNodeFile) showTreeRoot.getChildAt(i);
			String name = newChild.getName();
			String file = newChild.getFile();
			model.insertNodeInto(new DefaultMutableTreeNodeFile(name, file),root,i);
		}
		tree.expandPath(new TreePath(root));
		
	}
	

	private void addNode(String file) {
		
		String name = file.substring((file.lastIndexOf("/"))+1,file.length()-4);
		String newFile = file;
	    for(int i = 0; i<root.getChildCount(); i++){
	    	  if(root.getChildAt(i).toString().equals(name)){
	    		  //ErrorMessage error = new ErrorMessage("There exist such a pdb file in the database "+root.toString()+"!");
	  			JOptionPane.showMessageDialog(
	  					this,
	  					"There exist such a pdb file in the database "+root.toString()+"!");
	  						
	    		  return;
	    	  }
	      }

		//DefaultMutableTreeNodeFile newPDBOri = new DefaultMutableTreeNodeFile(name, file);
		//DefaultMutableTreeNodeFile newPDB = new DefaultMutableTreeNodeFile(name, file);
		DefaultMutableTreeNodeFile newPDBOri = new DefaultMutableTreeNodeFile(name, newFile);
		DefaultMutableTreeNodeFile newPDB = new DefaultMutableTreeNodeFile(name, newFile);
	    
		model.insertNodeInto(newPDB, (DefaultMutableTreeNode)model.getRoot(), root.getChildCount());
		modelOri.insertNodeInto(newPDBOri, (DefaultMutableTreeNode)modelOri.getRoot(), rootOri.getChildCount());
		tree.expandPath(new TreePath(root));
		model.reload();
	
	}
	
	
	private void renameNode() {
		DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
		if (selectedNode == null)
			return;
		
		String newName =  (String)JOptionPane.showInputDialog("Enter a new name for "+selectedNode);
		while(newName.equals("")){
			ErrorMessage Error = new ErrorMessage("Please enter a new name for "+selectedNode);
			newName =  (String)JOptionPane.showInputDialog("Enter a new name for "+selectedNode);
		}
		
		selectedNode.setUserObject(newName);
		rootOri.setUserObject(newName);
		model.nodeChanged(selectedNode);
		modelOri.nodeChanged(rootOri);
		
	}

	
	private void removeNode(){
		
		
		DefaultMutableTreeNodeFile selectedNode = (DefaultMutableTreeNodeFile) tree.getLastSelectedPathComponent();
		if (selectedNode == null)
				return;
			MutableTreeNode parent = (MutableTreeNode)(selectedNode.getParent());
		if (parent != null) 
		    model.removeNodeFromParent(selectedNode);
		
		for(int i = 0; i<rootOri.getChildCount(); i++){
			if(selectedNode.toString().equals(rootOri.getChildAt(i).toString())){
				rootOri.remove((DefaultMutableTreeNodeFile)rootOri.getChildAt(i));
				return; 
			}
		}
	}

	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseEntered(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mouseExited(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	public void mousePressed(MouseEvent arg0) {
		// TODO Auto-generated method stub
		
	}
/**
 * Shows the popUpMenu.
 * @param e MouseEvent
 */
	public void mouseReleased(MouseEvent e) {
		if (!e.isPopupTrigger()) return;
		TreePath path = tree.getPathForLocation(e.getX(), e.getY());
		tree.setSelectionPath(path);
	    if (path!=null){
	    	switch(path.getPathCount()){
				case 1: popUpRoot.show(tree,e.getX(),e.getY()); break;
				case 2: popUpPDB.show(tree,e.getX(),e.getY()); break;
				
	    	}
		}
		}
/**
 * Performs the action events for the JPanel.
 * @param arg0 ActionEvent
 */
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource() == pdbDelete){
			removeNode();
			return;
		}
		
		if (arg0.getSource() == renameDB){
			renameNode();
			return;
		}
		if (arg0.getSource() == pdbAdd){
				
			File tmplimitedRoot = new File("./Files");
			String newFile = tmplimitedRoot.getAbsoluteFile().toString();	
			int index = newFile.lastIndexOf("Files");

			if(newFile.substring(index-3).equals("\\.\\Files")){
				index = newFile.lastIndexOf("\\.\\");	
				tmplimitedRoot = new File(newFile.substring(0,index)+newFile.substring(index+2));
			}

			FileSystemView systemView = new DatabaseSystemView(tmplimitedRoot); 
			JFileChooser fileChooser = new JFileChooser(systemView);

			
			final DatabaseSystemView nlf;
		
			fileChooser.setFileFilter(new PDBFileFilter());

			int returnValue = fileChooser.showOpenDialog(null);
			if (returnValue == JFileChooser.CANCEL_OPTION){
				return;
			}
	
			File f = fileChooser.getSelectedFile();
			String newfile = f.toString();
			String substr= f.toString().substring((f.toString().lastIndexOf("\\"))+1, f.toString().length()-4);
			String pdbfile = substr+".pdb";
			while(!substr.equals("Files")){
				newfile = newfile.substring(0,newfile.lastIndexOf("\\"));
				substr = newfile.substring(newfile.lastIndexOf("\\")+1);
				pdbfile = substr+"/"+pdbfile;
			}
			//String test = f.toString().substring((f.toString().lastIndexOf("\\"))+1, f.toString().length()-4);
			addNode(pdbfile);
			fileChooser = null;
			if (f == null)
				return;
			
			if (!f.getAbsolutePath().endsWith(".pdb")) {
				JOptionPane.showMessageDialog(this, "The file " + f.getAbsolutePath() + " seems not to be a valid one!");
				return;
			}
		}
	}



	private void printTree(JTree printer) {
		DefaultMutableTreeNode printRoot = (DefaultMutableTreeNode) printer.getModel().getRoot();
		for(int i = 0; i<printRoot.getChildCount(); i++){
			System.out.println(printRoot.getChildAt(i));
		}
		
	}
	
	/**
	 * Returns the whole pdb tree.
	 * @return Returns original.
	 */
	public JTree getTree(){
		//updateTree(original);
		return original;
	}

	/**
	 * Returns the tree with pdbs are filtered.
	 * @return Returns tree.
	 */
	public JTree getVisualTree() {

		return tree;
	}
}


