package gui;

import java.awt.BorderLayout;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.DefaultMutableTreeNode;

import protein.Protein;

public class JWorkPanel extends JPanel {

	public JImport importPDB;

	public JPreview membranePreview;

	private JPreview membraneViewer;

	public Statistic statistics;

	public JProperties properties;

	public Msg_panel msg_panel;

	public JTabbedPane tp;

	public Membran membran;

	private boolean preview = false, property = false, statistic = true,
			membrane = false, msg = false;

	public Main main;

	/**
	 * not needed at the moment
	 */
	private class SwitchedTabsListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			JTabbedPane pane = (JTabbedPane) e.getSource();

			if (pane.getSelectedIndex() == 1) {
				if (membran.getProteins().isEmpty()) {
					pane.setEnabledAt(1, false);
				} else {
					pane.setEnabledAt(1, true);
				}
				if (preview) {
					DefaultMutableTreeNode tmp = (DefaultMutableTreeNode) membran
							.getTree().getLastSelectedPathComponent();
					if (tmp == null) {
						return;
					}
					if (tmp.getPath().length == 4) {
						/*
						 * if a protein in the tree is selected, first the
						 * values of the protein-object are fetched
						 */
						membranePreview
								.fetchPDBManipulationFromProtein(((DefaultMutableTreeNodeProtein) tmp)
										.getProtein());
					}
					/*
					 * This is a workaround to force Jmol to repaint the actual
					 * view of the PDBPreview.
					 */
					membranePreview.setSize(membranePreview.getWidth() + 1,
							membranePreview.getHeight() + 1);
				} else {
					preview = true;
					initPreview();
				}
			}
			if (pane.getSelectedIndex() == 4) {
				/**
				 * if(preview) { membranePreview.computePDBManipulation(); }
				 */

				properties.setPropertiesText(membran.createPropertiesText());
			}

			if (pane.getSelectedIndex() == 2) {
				if (!msg) {
					msg = true;
					msg_panel = new Msg_panel(); // TODO:
					tp.insertTab("Membrane Preview", null, msg_panel, "", 3);
					tp.removeTabAt(2);
				}
			}
		}
	}
/**
 * Main Panel in the main frame.
 * @param main Main
 */
	public JWorkPanel(Main main) {

		this.setLayout(new BorderLayout());
		this.setName("workpanel");
		this.main = main;
		tp = new JTabbedPane();

		importPDB = new JImport();
		properties = new JProperties();
//		membraneViewer = new JPreview(this);

		membran = new Membran(main);
		membran.setJTreePDB(importPDB.getVisualTree(), main);
		tp.addTab("PDB Database", importPDB);
		tp.addTab("Protein Preview", new JPanel());
		tp.addTab("Membrane Preview", new JPanel());
		statistics = new Statistic(membran.getTree());
		tp.addTab("Statistics", statistics);
		tp.addTab("Properties", properties);
		// tp.addTab("Membrane Viewer",membraneViewer.getMembraneJPanel());
		tp.setEnabledAt(1, false);
		tp.setEnabledAt(2, false);
		tp.setEnabledAt(3, false);
		tp.setEnabledAt(4, false);
		// tp.setEnabledAt(5,false);
		tp.addChangeListener(new SwitchedTabsListener());

		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
		sp.setOneTouchExpandable(true);
		sp.setContinuousLayout(true);
		sp.setDoubleBuffered(true);
		sp.setOpaque(true);
		sp.setLeftComponent(membran);
		sp.setRightComponent(tp);
		sp.setDividerLocation(sp.getMinimumDividerLocation() + 150);

		this.add(sp, BorderLayout.CENTER);
	}

	/**
	 * Initialises JPreview and inserts the tab Protein Preview to all the tabs.
	 *
	 */
	public void initPreview() {
		membranePreview = new JPreview(this);

		tp.insertTab("Protein Preview", null, membranePreview, "", 2);
		/*
		 * JPanel tpC = (JPanel)(tp.getComponentAt(5));
		 * tpC.add(membranePreview.getMembraneJPanel()); //tp.addTab("Membrane
		 * Viewer",membranePreview.getMembraneJPanel());
		 */
		tp.removeTabAt(1);

	}

	/**
	 * Vector with Proteins to show in the Protein Preview tab.
	 * @param blubb Vector
	 */
	public void setPreviewTab(Vector blubb) {
		if (blubb.size() != 0) {
			tp.setSelectedIndex(1);
			membranePreview.setPDB(blubb);
			// JOptionPane.showMessageDialog(null, "No protein selected ");
		}

	}
/**
 * Sets the text to print in the Properties tab.
 * @param toprint String
 */
	public void setPropertiesTab(String toprint) {
		tp.setSelectedIndex(4);
		properties.setPropertiesText(toprint);
	}

	/**
	 * Updates the Preview when there is loaded a new pdb
	 */
	public void updatePreview() {
		// workaround to repaint the pdb in order to correctly resize it
		membranePreview.setSize(membranePreview.getWidth() - 1, membranePreview
				.getHeight() - 1);
		membranePreview.setSize(membranePreview.getWidth() + 1, membranePreview
				.getHeight() + 1);
	}

	/**
	 * This method gets the yTranslation and the rotationMatrix from the
	 * protein-object which is selected in the tree and assigns it to the
	 * Jmol-application. It is used when a protein-object is loaded and shown
	 * the first time in the PDBPreview.
	 */
	public void fetchPDBManipulation() {
		// works only if the preview is already initialized
		if (preview) {
			DefaultMutableTreeNode tmp = (DefaultMutableTreeNode) membran
					.getTree().getLastSelectedPathComponent();
			if (tmp == null) {
				return;
			}
			if (tmp.getPath().length == 4) {
				membranePreview
						.fetchPDBManipulationFromProtein(((DefaultMutableTreeNodeProtein) tmp)
								.getProtein());
			}
			// repaint JPreview
			membranePreview.setSize(membranePreview.getWidth() + 1,
					membranePreview.getHeight() + 1);
		}
	}

	/**
	 * This method gets the yTranslation and the rotationMatrix from the
	 * Jmol-application and assigns it to the protein-object which is selected
	 * in the tree. It is used when the user is shifting or twisting the PDB in
	 * the PDBPreview.
	 */
	public void computePDBManipulation() {
		// works only if the preview is already initialized
		if (preview) {
			DefaultMutableTreeNode tmp = (DefaultMutableTreeNode) membran
					.getTree().getLastSelectedPathComponent();
			Protein tmpProtein = (((DefaultMutableTreeNodeProtein) tmp)
					.getProtein());
			if (tmpProtein != null)
				membranePreview.computePDBManipulation(tmpProtein);
		}
	}

	/**
	 * Enables the Protein Preview, Membrane Preview and Statistic tab.
	 * @param b boolean (1 if the tabs shoult be enabled)
	 */
	public void enableMyTabs(boolean b) {
		tp.setEnabledAt(2, b);
		tp.setEnabledAt(3, b);
		tp.setEnabledAt(4, b);
	}

	/**
	 * Returns the current active tab.
	 * @return Returns current active tab.
	 */
	public int getActiveTab() {
		return tp.getSelectedIndex();

	}
/**
 * Returns the main frame
 * @return Returns the main.
 */
	public Main getMain() {
		return main;
	}

	/**
	 * 
	 * @return Returns the preview.
	 */
	public boolean isPreview() {
		return preview;
	}

	/**
	 * @param preview
	 *            The preview to set.
	 */
	public void setPreview(boolean preview) {
		this.preview = preview;
	}

	/**
	 * @return Returns the msg.
	 */
	public boolean isMsg() {
		return msg;
	}

	/**
	 * @param msg
	 *            The msg to set.
	 */
	public void setMsg(boolean msg) {
		this.msg = msg;
	}

	/**
	 * Show a complete pdb-file at the MembraneViewer
	 * 
	 * @param filepath
	 *            The path of the file to display
	 */
	public void showCompleteMembrane(String filepath) {
		membraneViewer.showViewerPdbFile(filepath, membraneViewer
				.getJmolMemViewer());
	}

	/**
	 * @return Returns the membraneViewer.
	 */
	public JPreview getMembraneViewer() {
		return membraneViewer;
	}

	/**
	 * @param membraneViewer
	 *            The membraneViewer to set.
	 */
	public void setMembraneViewer(JPreview membraneViewer) {
		this.membraneViewer = membraneViewer;
	}

	/**
	 * Initialises the msg_panel and adds it to the tabs.
	 */
	public void initmsg() {
		msg = true;
		msg_panel = new Msg_panel(); 
		tp.insertTab("Membrane Preview", null, msg_panel, "", 3);
		tp.removeTabAt(2);

	}
}
