package gui;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.File;
import java.util.Locale;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTree;

import pdbParser.PDBProgressBar;
import pdbParser.PDBwriter;
import save.SaveFile;



import membranModel.MembranModel;
import membranModel.MembranModelSingleton;

import xml.XMLReader;

import xml.XMLWriter;

/** Main is the main application which starts the membrane editor.
 * 	Please write a test-file to invoke this main-class
 */

public class Main extends JFrame implements ActionListener
{

  public JWorkPanel workPanel;
  public JMenuItem loadDB, saveDB,newDB, newMembran, saveMembran, loadMembran, generateMembran, exit;
  public Starting starting;
  
  public boolean goon1, goon2 = true;
/**
 * Creates the main JFrame of the application with all the content in it
 *
 */
  public Main()
  {
	  
	super("CELLmicrocosmos MembraneEditor");
    addWindowListener(new WindowClosingAdapter(true));
    // add icon to main-window
    Image cmLogo;
    cmLogo = Toolkit.getDefaultToolkit().getImage("images/cmIcon.gif");
    this.setIconImage(cmLogo); 
    // menu at the top
    
  //  JSplitPane mainSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    
    JMenuBar menuBar = new javax.swing.JMenuBar();
    workPanel = new JWorkPanel(this);
    this.getContentPane().add(workPanel);


    
    JMenu fileMenu = new javax.swing.JMenu("File");
    newMembran = new JMenuItem("New Membrane Model");
    loadMembran = new JMenuItem("Load Membrane Model");
    saveMembran = new JMenuItem("Save Membrane Model");

    generateMembran = new JMenuItem("Generate Membrane Model");

    exit = new JMenuItem("Exit");
    exit.addActionListener(this);
       
    JMenuItem databaseMenu = new JMenu("Database");
    newDB = new JMenuItem("New Protein Database");
    loadDB = new JMenuItem("Open Protein Database");
    saveDB = new JMenuItem("Save Protein Database");

    JMenuItem helpMenu = new JMenu("Help");
    JMenuItem contentsMenuItem = new JMenuItem("Contents");
    contentsMenuItem.addActionListener(this);
    JMenuItem introduction = new JMenuItem("Introduction");
    introduction.addActionListener(this);
    JMenuItem creditsMenuItem = new JMenuItem("Credits");
    creditsMenuItem.addActionListener(this);

    
    fileMenu.add(newMembran);
    fileMenu.add(loadMembran);
    fileMenu.add(saveMembran);
    fileMenu.addSeparator();
    fileMenu.add(generateMembran);
    fileMenu.addSeparator();
    fileMenu.add(exit);
    
    newMembran.addActionListener(this);
    loadMembran.addActionListener(this);
    saveMembran.addActionListener(this);
    generateMembran.addActionListener(this);
    generateMembran.setEnabled(false);
    
    //fileMenu.add(newMembraneModelMenuItem);
    //fileMenu.add(loadMembraneModelMenuItem);
    
    //fileMenu.add(saveMembraneModelMenuItem);
    //fileMenu.add(newComponentMenuItem);

    menuBar.add(fileMenu);
    
    databaseMenu.add(newDB);
    databaseMenu.add(loadDB);
    databaseMenu.add(saveDB);


    newDB.addActionListener(this);
    saveDB.addActionListener(this);
    loadDB.addActionListener(this);
    //databaseMenu.add(newMembraneProteineDatabaseMenuItem);
    //databaseMenu.add(loadMembraneProteineDatabaseMenuItem);
    //databaseMenu.add(saveMembraneProteineDatabaseMenuItem);
    //databaseMenu.add(addProteinToDatabaseMenuItem);
    //databaseMenu.add(removeProteinFromDatabaseMenuItem);
    menuBar.add(databaseMenu);
/*
    lipidMenu.add(loadLipidMembraneMenuItem);
    lipidMenu.add(saveLipidMembraneMenuItem);
    lipidMenu.add(editLipidMembraneMenuItem);
    menuBar.add(lipidMenu);
  */  
    helpMenu.add(contentsMenuItem);
    helpMenu.add(introduction);
    helpMenu.add(creditsMenuItem);
    menuBar.add(helpMenu);

    setJMenuBar(menuBar);
    setLocation(100, 100);
    setSize(800, 600);

    //this.minimumSize(new Dimension(800,600));
    //setPrefferedSize(800,600);

    //quickstart();
    pack();
    
    start();
    lockInMinSize(this);
    
  }

  
  private void lockInMinSize(final Main frame) {
	  final int origX = 800;
	  final int origY = 600;
      frame.addComponentListener(new ComponentAdapter(){
	    	public void componentResized(ComponentEvent e){
	    		frame.setSize(
	    				(frame.getWidth()< origX) ? origX : frame.getWidth(),
	    				(frame.getHeight()< origY) ? origY : frame.getHeight());
	    	}
      });
      
      
  }


private void start() {
	  starting = new Starting(this);
	  starting.requestFocus();
	
}
	public void setGenerate(boolean b){
		generateMembran.setEnabled(b);
	}
	
  private void quickstart() {
      File f = new File("Files/b.xml");
      DBload dbload = new DBload();
      try {
          JTree newTree = dbload.loadFromFile(f);
          workPanel.importPDB.setTree(newTree);

      } catch (Exception ex) {
          JOptionPane.showMessageDialog(
              this,
              "Error during file loading process! "
                  + ex.toString());
          return;

      }

  // TODO Auto-generated method stub
  
}
	/**
	 * Performs the action events mainly for the menu.
	 * 
	 * 
	 * @param e ActionEvent
	 */
  public void actionPerformed(ActionEvent e) {
	  String actionCommand = e.getActionCommand();
	  
  	if(e.getSource() == saveDB){
  

  		JFileChooser fileChooser = new JFileChooser(new File("./Files"));
  		fileChooser.setFileFilter(new XMLFileFilter());
  		fileChooser.showSaveDialog(null);

  		File f = fileChooser.getSelectedFile();
  		fileChooser = null;

  		if (f == null)
  			return;
  		if (!f.getAbsolutePath().endsWith(".xml")) {
  			f = new File(f.toString()+".xml");
  		}
  		boolean check = f.exists();
  		while(check){
  			SaveQuestion sqe = new SaveQuestion(this, "", "Save database", f.toString());
  			int result = sqe.getReturnValue();
  			if(result == 2){
  				fileChooser = new JFileChooser(new File("./Files"));
  				fileChooser.setFileFilter(new XMLFileFilter());
  				fileChooser.showSaveDialog(null);

  				f = fileChooser.getSelectedFile();
  				fileChooser = null;

  				if (f == null)
  					return;
  				if (!f.getAbsolutePath().endsWith(".xml")) {
  					f = new File(f.toString()+".xml");
  				}
  	  			check = f.exists();
  			}

  			else if(result == 0){
  				return;
  			}
  			else if(result == 1){
  				check = false;
  			}
  		}
  		DBSave dbSave = new DBSave();

  		try {
  			dbSave.writeDB(f, workPanel.importPDB.getTree());
  		} catch (Exception ex) {
  			JOptionPane.showMessageDialog(
  				this,
  				"Error during saving the PDB-Database!\nMessage: " + ex.getMessage());
  			return;
  		}

  		
  		return;
  	}
  	if(e.getSource() == newDB){
  		workPanel.importPDB.createNewDB();
  		
  	}
  	if(e.getSource() == loadDB){ //  || actionCommand.equals("loadPDB")){
		JFileChooser fileChooser = new JFileChooser(new File("./Files"));
		fileChooser.setFileFilter(new XMLFileFilter());
  		int returnValue = fileChooser.showOpenDialog(null);
  		if (returnValue == JFileChooser.CANCEL_OPTION){


  			return;
  		}
  		File f = fileChooser.getSelectedFile();
  		fileChooser = null;


  		if (f == null)
  			return;
  			
  		if (!f.getAbsolutePath().endsWith(".xml")) {
  			JOptionPane.showMessageDialog(this, "The file " + f.getAbsolutePath() + " seems not to be a valid one!");
  			return;
  		}

  		DBload dbload = new DBload();
  		try {
  			JTree newTree = dbload.loadFromFile(f);
  			workPanel.importPDB.setTree(newTree);

  		} catch (Exception ex) {
  			JOptionPane.showMessageDialog(
  				this,
  				"Error during file loading process! "
  					+ ex.toString());
  			return;

  		}
  		//setVisible(true);
 
  	}
  	
  	if(e.getSource() == saveMembran){
  		if (workPanel.isMsg()) {
  			workPanel.msg_panel.proteinMatrix.saveCurrentStatus();
  		}
  		else{
  			//msg_panel = new Msg_panel();
  		}
  		Vector allProteins = workPanel.membran.getProteins();
  		MembranModel mm = MembranModelSingleton.getInstance();

  		JFileChooser fileChooser = new JFileChooser(new File("./Files"));
  		fileChooser.setFileFilter(new XMLFileFilter());
  		fileChooser.showSaveDialog(null);

  	     File f = fileChooser.getSelectedFile();
  		fileChooser = null;
  		if (f == null)
  			return;
  		if (!f.getAbsolutePath().endsWith(".xml")) {
  			f = new File(f.toString()+".xml");
  			
  			
  		}
 		boolean check = f.exists();
  		while(check){
  			SaveQuestion sqe = new SaveQuestion(this, "", "Save membrane model", f.toString());
  			int result = sqe.getReturnValue();
  			if(result == 2){
  				fileChooser = new JFileChooser(new File("./Files"));
  				fileChooser.setFileFilter(new XMLFileFilter());
  				fileChooser.showSaveDialog(null);

  				f = fileChooser.getSelectedFile();
  				fileChooser = null;

  				if (f == null)
  					return;
  				if (!f.getAbsolutePath().endsWith(".xml")) {
  					f = new File(f.toString()+".xml");
  				}
  	  			check = f.exists();
  			}

  			else if(result == 0){
  				return;
  			}
  			else if(result == 1){
  				check = false;
  			}
  		}


 		XMLWriter memSave = new XMLWriter(f.toString(), mm);
 		return;
  	}
  	
  	if(e.getSource() == loadMembran){
		MembranModel mm = MembranModelSingleton.getInstance();
		mm.removeAllComponents();
		JFileChooser fileChooserMembran = new JFileChooser(new File("./Files"));
		fileChooserMembran.setFileFilter(new XMLFileFilter());
  		int returnValue = fileChooserMembran.showOpenDialog(null);
  		if (returnValue == JFileChooser.CANCEL_OPTION){
  			goon1 = false;
  			return;
  		}
  		File f = fileChooserMembran.getSelectedFile();
  		fileChooserMembran = null;
  		goon1 = true;
  		if (f == null)
  			return;
  			
  		if (!f.getAbsolutePath().endsWith(".xml")) {
  			JOptionPane.showMessageDialog(this, "The file " + f.getAbsolutePath() + " seems not to be a valid one!");
  			return;
  		}
  		
  		try {
  			XMLReader xmlreader = new XMLReader(f.toString());

  		} catch (Exception ex) {
  			JOptionPane.showMessageDialog(
  				this,
  				"Error during file loading process! "
  					+ ex.toString());
  			return;

  		}
  		workPanel.membran.initTree(MembranModelSingleton.getInstance());
	
  		if(!workPanel.isMsg()){
  			workPanel.initmsg();
  		}
  		workPanel.msg_panel.proteinMatrix.generateLoaded();
  		return;
  	}
  	
  	if(e.getSource() == newMembran){
  		workPanel.membran.newMembranModel();
  		return;
  	}
  	if(e.getSource() == generateMembran){
  		workPanel.msg_panel.proteinMatrix.saveCurrentStatus();
  		//TODO: Abfrage ob der writer auch wirklich zuende luft!!! Dann Tab switch!!! Add Status-Window!!

  		new PDBProgressBar(this);
  		
  		
  		//  		PDBwriter peter = new PDBwriter("1pfe");
  	//	String membraneToView = peter.getFilename()+".pdb";
  		
  		//System.out.println(membraneToView);
  		//System.out.println("*********** File geschrieben nun switchen!");
  
//  		workPanel.showCompleteMembrane(membraneToView);
//  		workPanel.tp.setEnabledAt(5,true);
//  		workPanel.tp.setSelectedIndex(5);
  	}
	
  	if(actionCommand.equals("newMembrane")){
 //   	workPanel.membran.newMembranModel();
 //   	starting.changeWindow();
    	return;
  	}
	
	
//	if(actionCommand.equals("defaultlipid")){
//  		MembranModel membranemodel = MembranModelSingleton.getInstance();
//  		membranemodel.setBiLayerType("pc2_exp");
//  		membranemodel.getBiLayerType().startup();
//  		setVisible(true);
//		return;
//	}

	
//	if(actionCommand.equals("loadlipid")){
//		setVisible(true);
//		return;
//	}
	
	if(e.getSource() ==exit){
		Runtime.getRuntime().exit(0);
	}
	else if(actionCommand.equals("Contents")){
	    HelpPanel help =  new HelpPanel();
	}
	else if(actionCommand.equals("Introduction")){
	    Introduction intro = new Introduction(this);
	}
	else if(actionCommand.equals("Credits")){
		Credits credits = new Credits(this);
	}

  }





  
}