package gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;

public class NewMembraneDialog extends JDialog implements ActionListener{
	public JTextField description, name, species, celltype;
	public boolean returnValue;
	public JSpinner ratioNumber;
	/**
	 * Builds up a JDialog for creating a new membrane model.
	 * @param main Main
	 */
	public NewMembraneDialog(Main main){
		this.setTitle("New membrane model");
		this.requestFocus(true);
		this.setSize(330, 370);
		this.setLocationRelativeTo(main);
		this.setModal(true);

		JTextArea descriptionOrder = new JTextArea("     Please enter a description (optional)");
		JTextArea nameOrder = new JTextArea("     Please enter a name for the new membrane model");
		JTextArea speciesOrder = new JTextArea("     Please enter species name (optional)");
		JTextArea celltypeOrder = new JTextArea("     Please enter a celltype name (optional)");
		
		
		descriptionOrder.setBackground(null);
		descriptionOrder.setFocusable(false);
		descriptionOrder.setFont(new Font("Dialog", Font.BOLD ,12));
		
		celltypeOrder.setBackground(null);
		celltypeOrder.setFocusable(false);
		celltypeOrder.setFont(new Font("Dialog", Font.BOLD ,12));
		
		nameOrder.setBackground(null);
		nameOrder.setFocusable(false);
		nameOrder.setFont(new Font("Dialog", Font.BOLD ,12));
		
		speciesOrder.setBackground(null);
		speciesOrder.setFocusable(false);
		speciesOrder.setFont(new Font("Dialog", Font.BOLD ,12));

		
		description = new JTextField(1);
		name = new JTextField(1);
		species =new JTextField(1);
		celltype =new JTextField(1);
		name.setBorder(BorderFactory.createTitledBorder(""));
		description.setBorder(BorderFactory.createTitledBorder(""));
		species.setBorder(BorderFactory.createTitledBorder(""));
		celltype.setBorder(BorderFactory.createTitledBorder(""));
		
		Box descriptionbox = new Box(BoxLayout.X_AXIS);
		descriptionbox.add(Box.createHorizontalStrut(15));
		descriptionbox.add(description);
		descriptionbox.add(Box.createHorizontalStrut(15));
		

		Box namebox = new Box(BoxLayout.X_AXIS);
		namebox.add(Box.createHorizontalStrut(15));
		namebox.add(name);
		namebox.add(Box.createHorizontalStrut(15));
		
		Box speciesbox = new Box(BoxLayout.X_AXIS);
		speciesbox.add(Box.createHorizontalStrut(15));
		speciesbox.add(species);
		speciesbox.add(Box.createHorizontalStrut(15));
		
		Box celltypebox = new Box(BoxLayout.X_AXIS);
		celltypebox.add(Box.createHorizontalStrut(15));
		celltypebox.add(celltype);
		celltypebox.add(Box.createHorizontalStrut(15));
		
		
		Box areabox = new Box(BoxLayout.Y_AXIS);
		areabox.add(Box.createVerticalStrut(20));
		areabox.add(nameOrder);
		areabox.add(Box.createVerticalStrut(10));
		areabox.add(namebox);

		areabox.add(Box.createVerticalStrut(10));
		areabox.add(descriptionOrder);
		areabox.add(Box.createVerticalStrut(10));
		areabox.add(descriptionbox);
		
		areabox.add(Box.createVerticalStrut(20));
		areabox.add(speciesOrder);
		areabox.add(Box.createVerticalStrut(10));
		areabox.add(speciesbox);
		
		areabox.add(Box.createVerticalStrut(20));
		areabox.add(celltypeOrder);
		areabox.add(Box.createVerticalStrut(10));
		areabox.add(celltypebox);
		areabox.add(Box.createVerticalStrut(20));
		


		JButton okey = new JButton("OK");
		JButton cancel = new JButton("Cancel");
		okey.addActionListener(this);
		cancel.addActionListener(this);
		
		//adding the buttons to the bottonbox
		Box buttonbox = new Box(BoxLayout.X_AXIS);
		buttonbox.add(Box.createVerticalStrut(30));
		buttonbox.add(okey);
		buttonbox.add(Box.createVerticalStrut(10));
		buttonbox.add(cancel);
		buttonbox.add(Box.createVerticalStrut(30));

		//adding the topbox and the bottumbox to the mainbox
		Box mainbox = new Box(BoxLayout.Y_AXIS);
		mainbox.add(areabox);
		mainbox.add(buttonbox);
		mainbox.add(Box.createVerticalStrut(20));
		name.requestFocus(true);
		getContentPane().add(mainbox);


		show();

	
		
	}
	/**
	 	 * Performs the action events for the JDialog (mainly the ok and cancel button).
	 * @param arg0 Action Event
	 */
	public void actionPerformed(ActionEvent arg0) {
		String actionCommand = arg0.getActionCommand();
		if(actionCommand.equals("OK")){
			if(!(name.getText().equals(""))){
				returnValue = true;
				dispose();
				
			}
			else{
				JOptionPane.showMessageDialog(null,"Please enter a name for the new Membrane Model!" );
				name.setFocusable(true);
			}
		}
		if(actionCommand.equals("Cancel")){
			returnValue = false;
			dispose();
		}		
		
	}
	/**
	 * Returns the description of the membrane model.
	 * @return Returns the description.
	 */
	public String getDescription() {
		return description.getText();
	}

	/**
	 * Returns the name of the membrane model.
	 * @return Returns the name.
	 */
	public String getName() {
		return name.getText();
	}

	/**
	 * Returns the return value of the JDialog.
	 * @return Returns the return value.
	 */
	public boolean getReturnValue() {
		return returnValue;
	}


	/**
	 * Returns the celltype of the membrane model.
	 * @return Returns the celltype.
	 */
	public String getCelltype() {
		return celltype.getText();
	}
	/**
	 * The celltype of the membrane model to set.
	 * @param celltype The celltype to set.
	 */
	public void setCelltype(JTextField celltype) {
		this.celltype = celltype;
	}
	/**
	 * Returns the species of the membrane model.
	 * @return Returns the species.
	 */
	public String[] getSpecies() {
		String[] tmp = species.getText().split(",");
		return tmp;
	}
	/**
	 * The species of the membrane model to set.
	 * @param species The species to set.
	 */
	public void setSpecies(JTextField species) {
		this.species = species;
	}
}
