package gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.SwingUtilities;

import membranModel.MembranModel;
import membranModel.MembranModelSingleton;
import xml.XMLReader;

public class Starting extends JDialog implements ActionListener {

	private JButton load, exit;

	private JButton newModel;

	private boolean sec = false;

	private String loadText, loadAction, newModelAction, content = "";

	private JTextArea area;

	private JTextArea description;

	private JLabel pro, mem, lip, firstStep, secStep, thirdStep;

	private Main main;

	private int step = 1;

	Color one, two;

	private boolean introduction = false;
	/**
	 * Creates the Wizard as an JDialog with all contents.
	 * @param main Main
	 */
	public Starting(Main main) {
		this.main = main;
		this.setTitle("Membrane Editor");
		this.setFocusable(true);
		this.setUndecorated(true);
		this.getRootPane().setWindowDecorationStyle(JRootPane.PLAIN_DIALOG);

		loadText = " Membrane";
		loadAction = "loadMembran";
		newModelAction = "newMembrane";
		content = "                                  1) Select a membrane model";
		one = Color.BLACK;
		two = Color.LIGHT_GRAY;

		// creating the buttons
		load = new JButton("Load" + loadText);
		load.addActionListener(main);
		load.setToolTipText("Load " + loadText + " model");
		load.setActionCommand(loadAction);
		load.addActionListener(this);

		newModel = new JButton("New" + loadText);
		newModel.setToolTipText("Create new" + loadText);
		newModel.addActionListener(main);
		newModel.addActionListener(this);

		exit = new JButton("Exit Application");
		exit.addActionListener(this);
		exit.setActionCommand("exit");
		exit.setToolTipText("Quit the Membrane Editor");

		area = new JTextArea(content, 1, 1);
		area.setFont(new Font("Dialog", Font.BOLD, 14));
		area.setBackground(null);

		description = new JTextArea(
				"The Membrane Model is an XML-file which contains all informations concerning the\n"
						+ "membrane:\n"
						+ "The identifier of the included PDBs, the translations, the different ratios, the \n"
						+ "descriptions, etc..", 1, 1);
		description.setFont(new Font("Dialog", Font.BOLD, 12));
		description.setBackground(null);
		description.setAutoscrolls(false);
		description.setFocusable(false);
		area.setFocusable(false);

		Box buttonbox = new Box(BoxLayout.X_AXIS);
		buttonbox.add(Box.createVerticalStrut(20));
		buttonbox.add(load);
		buttonbox.add(Box.createVerticalStrut(20));
		buttonbox.add(newModel);
		buttonbox.add(Box.createVerticalStrut(20));
		buttonbox.add(exit);
		buttonbox.add(Box.createVerticalStrut(20));

		// adding the textarea and the bottonbox to the bottombox
		Box bottombox = new Box(BoxLayout.Y_AXIS);
		bottombox.add(Box.createVerticalStrut(20));
		bottombox.add(area);
		bottombox.add(Box.createVerticalStrut(5));
		bottombox.add(description);
		bottombox.add(Box.createVerticalStrut(20));
		bottombox.add(buttonbox);
		bottombox.add(Box.createVerticalStrut(20));

		Box stepsbox = new Box(BoxLayout.X_AXIS);
		stepsbox.add(Box.createHorizontalStrut(20));
		stepsbox.add(bottombox);
		stepsbox.add(Box.createHorizontalStrut(30));
		// mainWizardbox.setBorder(BorderFactory.createTitledBorder(""));

		JLabel introdoction = new JLabel("Steps to start the Membrane Editor");
		introdoction.setFont(new Font("Dialog", Font.BOLD, 14));
		firstStep = new JLabel("  Step 1");
		firstStep.setFont(new Font("Dialog", Font.BOLD, 12));
		firstStep.setForeground(one);
		thirdStep = new JLabel("  Step 3");
		thirdStep.setFont(new Font("Dialog", Font.BOLD, 12));
		thirdStep.setForeground(two);
		secStep = new JLabel("  Step 2");
		secStep.setFont(new Font("Dialog", Font.BOLD, 12));
		secStep.setForeground(two);

		mem = new JLabel("    Membrane model  ");
		mem.setForeground(one);
		pro = new JLabel("    Protein database  ");
		pro.setForeground(two);
		lip = new JLabel("    Lipid type   ");
		lip.setForeground(two);

		ImageIcon proteinIcon = new ImageIcon("gui/pics/protein.gif");
		JLabel label = new JLabel(proteinIcon);
		Box iconbox = new Box(BoxLayout.Y_AXIS);
		iconbox.add(label);

		Box wizardbox = new Box(BoxLayout.Y_AXIS);
		wizardbox.add(Box.createHorizontalStrut(20));
		wizardbox.add(introdoction);
		wizardbox.add(Box.createHorizontalStrut(40));
		wizardbox.add(firstStep);
		wizardbox.add(Box.createHorizontalStrut(10));
		wizardbox.add(mem);
		wizardbox.add(Box.createHorizontalStrut(20));
		wizardbox.add(secStep);
		wizardbox.add(Box.createHorizontalStrut(10));
		wizardbox.add(lip);
		wizardbox.add(Box.createHorizontalStrut(20));
		wizardbox.add(thirdStep);
		wizardbox.add(Box.createHorizontalStrut(10));
		wizardbox.add(pro);
		wizardbox.add(Box.createHorizontalStrut(20));

		Box mainWizardbox = new Box(BoxLayout.X_AXIS);
		mainWizardbox.add(Box.createHorizontalStrut(20));
		mainWizardbox.add(wizardbox);
		mainWizardbox.add(Box.createHorizontalStrut(30));
		mainWizardbox.setBorder(BorderFactory.createTitledBorder(""));
		// adding the topbox and the bottumbox to the mainbox
		Box mainbox = new Box(BoxLayout.X_AXIS);
		mainbox.add(mainWizardbox);
		mainbox.add(stepsbox);

		getContentPane().add(mainbox);

		setSize(850, 200);
		this.setResizable(false);
		this.setModal(true);
		this.setLocationRelativeTo(main);
		this.requestFocus(true);

		setVisible(true);

	}
	/**
	 * Performs the action events for the JDialog.
	 * 	 
	 * @param arg0 ActionEvent
	 */
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource() == load) {
			//load pdb
			if (step == 3) {
				JFileChooser fileChooser = new JFileChooser(new File("./Files"));
				fileChooser.setFileFilter(new XMLFileFilter());
				int returnValue = fileChooser.showOpenDialog(null);
				if (returnValue == JFileChooser.CANCEL_OPTION) {
					return;
				}
				File f = fileChooser.getSelectedFile();
				fileChooser = null;
				if (f == null)
					return;

				if (!f.getAbsolutePath().endsWith(".xml")) {
					JOptionPane.showMessageDialog(this, "The file "
							+ f
							+ " seems not to be a valid one!");
					return;
				}

				DBload dbload = new DBload();
				try {
					JTree newTree = dbload.loadFromFile(f);
					main.workPanel.importPDB.setTree(newTree);

				} catch (Exception ex) {
					JOptionPane.showMessageDialog(this,
							"Error during file loading process!\n" +
							"The file "+f+"" +
									" seems not to be a valid one!");
									
					return;

				}
		  		main.setVisible(true);
				dispose();
				if(introduction){
					Introduction intro = new Introduction(main);
				}



			} else if (step == 1) {
				JFileChooser fileChooserMembran = new JFileChooser(new File(
						"./Files"));

				fileChooserMembran.setFileFilter(new XMLFileFilter());
				int returnValue = fileChooserMembran.showOpenDialog(null);
				if (returnValue == JFileChooser.CANCEL_OPTION) {

					return;
				}
				File f = fileChooserMembran.getSelectedFile();
				fileChooserMembran = null;

				if (f == null)
					return;
				boolean exceptionhappened = false;
				if (!f.getAbsolutePath().endsWith(".xml")) {
					JOptionPane.showMessageDialog(this, "The file "
							+ f.getAbsolutePath()
							+ " seems not to be a valid one!");
					return;
				}

				try{
					XMLReader xmlreader = new XMLReader(f.toString());
		 		} catch (Exception ex) {
					if(ex.toString().contains("Files") && ex.toString().contains(".pdb")){
						String message = ex.toString(); 
						message = message.substring(message.indexOf("Files"), message.indexOf("pdb")+3);
						JOptionPane
								.showMessageDialog(main, "File " +
										message+" not found in the default file directory! \nPlease choose a new membrane model from the file directory or place "+message.substring(6)+" into the default file directory.");

		 			}
		 			else{
					JOptionPane.showMessageDialog(this,
							"Error during file loading process!\n" +
							"The file "+f+"" +
									" seems not to be a valid one!");

		 			}
		  			return;

		  		}
				main.workPanel.membran.initTree(MembranModelSingleton
						.getInstance());
		  		
		  		
		  		if(!main.workPanel.isMsg()){
		  			main.workPanel.initmsg();
		  		}
		  		main.workPanel.msg_panel.proteinMatrix.generateLoaded();
				
				

		  		changeWindow();
				
				step = 3;
				//JOptionPane.showMessageDialog
				//(this, "Step 2 will be skipped. \n The lipid type is set from the loaded membrane model.", "Step 2", JOptionPane.INFORMATION_MESSAGE);
				//showInputDialog("Step two will be skipped. \n The lipid type is saved in the loaded membrane model.");
			} else if (step == 2) {
				JFileChooser fileChooserLipid = new JFileChooser(new File(
						"./Files"));
				fileChooserLipid.setFileFilter(new PDBFileFilter());
				int returnValue = fileChooserLipid.showOpenDialog(null);
				if (returnValue == JFileChooser.CANCEL_OPTION) {

					return;
				}
				File f = fileChooserLipid.getSelectedFile();

				fileChooserLipid = null;

				if (f == null)

					return;

				String name = f.toString().substring((f.toString().lastIndexOf("\\"))+1,f.toString().length()-4);
		  		MembranModel membranemodel = MembranModelSingleton.getInstance();
		  		try {
					membranemodel.setBiLayerType(name);
				} catch (IOException e) {
					String message = e.toString(); 
					message = message.substring(message.indexOf("Files"), message.indexOf("pdb")+3);
					JOptionPane
							.showMessageDialog(main, "File " +
									message+" not found in the default file directory!");
				}
		  		membranemodel.getBiLayerType().startup();

				changeWindow();
				step = 3;
			}
		}
		if (arg0.getSource() == newModel) {
			if (step == 1) {
				main.workPanel.membran.newMembranModel();
				if (main.workPanel.membran.nmd.getReturnValue()) {
					changeWindow2();
					step = 2;
				}
				introduction = true;
			} else if (step == 3) {
				main.workPanel.importPDB.createNewDB();
				if (main.workPanel.importPDB.npd.getReturnValue()) {
			  		main.setVisible(true);

			  		dispose();
					if(introduction){
						Introduction intro = new Introduction(main);
					}
				}
			} else if (step == 2) {	
			  		MembranModel membranemodel = MembranModelSingleton.getInstance();
			  		try {
						membranemodel.setBiLayerType("pc2_exp");
					} catch (IOException e) {
						// TODO Auto-generated catch block
						String message = e.toString(); 
						message = message.substring(message.indexOf("Files"), message.indexOf("pdb")+3);
						JOptionPane
								.showMessageDialog(main, "File " +
										message+" not found in the default file directory!");
					}
			  		membranemodel.getBiLayerType().startup();
					changeWindow();
					step = 3;

			}
		}
		if(arg0.getSource() == exit ){

				Runtime.getRuntime().exit(0);

		}

	}

	private void changeWindow() {
		loadText = " PDB Database";
		load.setText("Load " + loadText);

		loadAction = "loadPDB";
		load.setActionCommand(loadAction);

		newModelAction = "newPDB";
		newModel.setText("New " + loadText);
		newModel.setActionCommand(newModelAction);
		content = "                                        3) Select a PDB database";
		area.setText(content);
		one = Color.LIGHT_GRAY;
		two = Color.BLACK;
		description
				.setText("The PDB Database is an XML-file which contains informations about the database\n"
						+ "used by the membrane model. \n"
						+ "The databases organise the PDB-files located in the PDB-rootdirectory into different\n"
						+ "categories.");
		firstStep.setForeground(one);
		thirdStep.setForeground(two);
		secStep.setForeground(one);
		mem.setForeground(one);
		pro.setForeground(two);
		lip.setForeground(one);

	}

	private void changeWindow2() {
		loadText = " lipid type";
		load.setText("Load " + loadText);

		loadAction = "loadlipid";
		load.setActionCommand(loadAction);

		newModelAction = "defaultlipid";
		newModel.setText("Use default " + loadText);
		newModel.setActionCommand(newModelAction);
		content = "                                      2) Select a lipid type";
		area.setText(content);
		one = Color.LIGHT_GRAY;
		two = Color.BLACK;
		load.setToolTipText("Load new lipid type");
		newModel.setToolTipText("Use the default lipid type");
		description
				.setText("The Lipid Type defines, which PDB file is used to be destributed on the whole\n"
						+ "membrane surface.");

		firstStep.setForeground(one);
		thirdStep.setForeground(one);
		secStep.setForeground(two);
		mem.setForeground(one);
		pro.setForeground(one);
		lip.setForeground(two);

	}

	/**
	 * Returns the current step of the wizard.
	 * @return Returns the step.
	 */
	public int getStep() {
		return step;
	}

	/**
	 * Sets the current step of the wizard.
	 * @param step
	 *            The step to set.
	 */
	public void setStep(int step) {
		this.step = step;
	}
	
	/**
	 * 
	 * @return Returns the indrodutction
	 */
	public boolean getIntroduction(){
		return introduction;
	}

}
