package gui.dragndrop;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.swing.tree.DefaultMutableTreeNode;




public class TransferableTreeNode extends DefaultMutableTreeNode implements Transferable {

	final static int TREE = 0;

	final static int STRING = 1;

	final static int PLAIN_TEXT = 1;

	final public static DataFlavor DEFAULT_MUTABLE_TREENODE_FLAVOR = new DataFlavor(
			DefaultMutableTreeNode.class, "Default Mutable Tree Node");

	static DataFlavor flavors[] = { DEFAULT_MUTABLE_TREENODE_FLAVOR,
		DataFlavor.stringFlavor, DataFlavor.plainTextFlavor };

	private DefaultMutableTreeNode data;
/**
 * Creates transferable nodes.
 * @param data DefaultMutableTreeNode
 */
	public TransferableTreeNode(DefaultMutableTreeNode data) {
		this.data = data;
	}
/**
 * @return Return the flavors.
 */
	public DataFlavor[] getTransferDataFlavors() {
		return flavors;
	}

	/**
	 * @param flavor DataFlavor
	 * @return Returns transferData
	 */
	public Object getTransferData(DataFlavor flavor)
		throws UnsupportedFlavorException, IOException {
		Object returnObject;
		if (flavor.equals(flavors[TREE])) {
			Object userObject = data.getUserObject();
			if (userObject == null) {
				returnObject = data;
			} else {
				returnObject = userObject;
			}
		} else if (flavor.equals(flavors[STRING])) {
			Object userObject = data.getUserObject();
			if (userObject == null) {
				returnObject = data.toString();
			} else {
				returnObject = userObject.toString();
			}
		} else if (flavor.equals(flavors[PLAIN_TEXT])) {
			Object userObject = data.getUserObject();
			String string;
			if (userObject == null) {
				string = data.toString();
			} else {	
				string = userObject.toString();
			}
			returnObject = new ByteArrayInputStream(string.getBytes("Unicode"));
		} else {
			throw new UnsupportedFlavorException(flavor);
		}
		return returnObject;
	}

	/**
	 * @param flavor DataFlavor
	 * @return boolean (true: if flavor is supported)
	 */
	public boolean isDataFlavorSupported(DataFlavor flavor) {
		boolean returnValue = false;
		for (int i = 0, n = flavors.length; i < n; i++) {
			if (flavor.equals(flavors[i])) {
				returnValue = true;
				break;
			}
		}
		return returnValue;
	}
}


	



