package gui.dragndrop;

//Color colorBackground = MembranModelSingleton.getInstance().getBackground_color();
import gui.ChangePercent;
import gui.CreateKomponent;
import gui.DefaultMutableTreeNodeFile;
import gui.DefaultMutableTreeNodeKatagory;
import gui.DefaultMutableTreeNodeProtein;
import gui.Main;
import gui.messages.ErrorMessage;

import java.awt.Color;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.jar.JarException;

import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.MutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.vecmath.Matrix3f;

import membranModel.MembranModel;
import membranModel.MembranModelSingleton;

import pdbParser.PDBreader;
import protein.Protein;

public class TreeDropTargetListener implements DropTargetListener {

	public JTree tree, pdbs;

	public int klickedNode;

	private int komponentsCounter = 1;

	public Main main;

	private boolean newKategory = false;

	private boolean rejectdrop = false;

	private boolean created = false;

	private TreePath path;
/**
 * Extended the DropTargetListener
 * @param treeGiven Membrane Model Tree
 * @param mainGiven Main frame
 * @param pdbs JTree with pdb files
 * @param componenteNumb int (number of components in treeGiven)
 */
	public TreeDropTargetListener(JTree treeGiven, Main mainGiven, JTree pdbs,
			int componenteNumb) {
		this.tree = treeGiven;
		this.main = mainGiven;
		this.pdbs = pdbs;
		this.komponentsCounter = componenteNumb;
	}

	public void dragEnter(DropTargetDragEvent dropTargetDragEvent) {

	}

	public void dragExit(DropTargetEvent dropTargetEvent) {
	}

	public void dragOver(DropTargetDragEvent dropTargetDragEvent) {
	}

	public void dropActionChanged(DropTargetDragEvent dropTargetDragEvent) {
	}
/**
 * Checks if the drop is allowed at this place in the tree.
 * @param dropTargetDropEvent DropTargetDropEvent
 */
	public synchronized void drop(DropTargetDropEvent dropTargetDropEvent) {
		Point location = dropTargetDropEvent.getLocation();
		path = tree.getPathForLocation(location.x, location.y);
		if (path == null) {
			return;
		}
		DefaultMutableTreeNode parent = (DefaultMutableTreeNode) path
				.getLastPathComponent();
		tree.setSelectionPath(path);
		switch (path.getPathCount()) {
		// drop on root
		case 1:
			dropTargetDropEvent.rejectDrop();
			ErrorMessage error = new ErrorMessage(
					"Sorry! Can't drop a pdb-File to the root :(");
			dropTargetDropEvent.rejectDrop();
			break;
		// drop as new komponete
		case 2:
			created = false;
			newKategory = true;
			checkTree(parent, dropTargetDropEvent);
			path = path.pathByAddingChild(parent);
			tree.expandPath(path);

			if (rejectdrop) {
				dropTargetDropEvent.rejectDrop();
				rejectdrop = false;
			} else {
				main.workPanel.enableMyTabs(true);
			}
			newKategory = false;

			break;
		// drop into kompontne
		case 3:

			checkTree(parent, dropTargetDropEvent);
			if (rejectdrop) {
				dropTargetDropEvent.rejectDrop();
				rejectdrop = false;
			} else {
				main.workPanel.enableMyTabs(true);
			}

			break;
		// drop new leaf
		case 4:
			checkTree((DefaultMutableTreeNode) path.getParentPath()
					.getLastPathComponent(), dropTargetDropEvent);
			if (rejectdrop) {
				dropTargetDropEvent.rejectDrop();
				rejectdrop = false;
			} else {
				main.workPanel.enableMyTabs(true);
			}
			break;
		}
	}
/**
 * Checks if the dropping is allowed in the tree.
 * @param parent DefaultMutableTreeNode, at which the new node should be added
 * @param dropTargetDropEvent DropTargetDropEvent
 */
	private void checkTree(DefaultMutableTreeNode parent,
			DropTargetDropEvent dropTargetDropEvent) {

		try {
			Transferable tr = dropTargetDropEvent.getTransferable();
			if (tr
					.isDataFlavorSupported(TransferableTreeNode.DEFAULT_MUTABLE_TREENODE_FLAVOR)) {
				dropTargetDropEvent
						.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				Object userObject = tr
						.getTransferData(TransferableTreeNode.DEFAULT_MUTABLE_TREENODE_FLAVOR);
				addElement(parent, userObject);
				dropTargetDropEvent.dropComplete(true);
			} else if (tr.isDataFlavorSupported(DataFlavor.stringFlavor)) {
				dropTargetDropEvent
						.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				String string = (String) tr
						.getTransferData(DataFlavor.stringFlavor);
				addElement(parent, string);
				dropTargetDropEvent.dropComplete(true);
			} else if (tr.isDataFlavorSupported(DataFlavor.plainTextFlavor)) {
				dropTargetDropEvent
						.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				Object stream = tr.getTransferData(DataFlavor.plainTextFlavor);
				if (stream instanceof InputStream) {
					InputStreamReader isr = new InputStreamReader(
							(InputStream) stream);
					BufferedReader reader = new BufferedReader(isr);
					String line;
					while ((line = reader.readLine()) != null) {
						addElement(parent, line);
					}
					dropTargetDropEvent.dropComplete(true);
				} else if (stream instanceof Reader) {
					BufferedReader reader = new BufferedReader((Reader) stream);
					String line;
					while ((line = reader.readLine()) != null) {
						addElement(parent, line);
					}
					dropTargetDropEvent.dropComplete(true);
				} else {
					System.err.println("Unknown type: " + stream.getClass());
					dropTargetDropEvent.rejectDrop();
				}
			} else if (tr.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
				dropTargetDropEvent
						.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);
				List fileList = (List) tr
						.getTransferData(DataFlavor.javaFileListFlavor);
				Iterator iterator = fileList.iterator();
				while (iterator.hasNext()) {
					File file = (File) iterator.next();
					addElement(parent, file.toURL());
				}
				dropTargetDropEvent.dropComplete(true);
			} else {
				System.err.println("Rejected");
				dropTargetDropEvent.rejectDrop();
			}
		} catch (IOException io) {
			io.printStackTrace();
			dropTargetDropEvent.rejectDrop();
		} catch (UnsupportedFlavorException ufe) {
			ufe.printStackTrace();
			dropTargetDropEvent.rejectDrop();
		}

	}
/**
 * Adds the new protein to the membrane model tree.
 * @param parentNode DefaultMutableTreeNode 
 * @param element Object to add to parentNode
 */
	private void addElement(DefaultMutableTreeNode parentNode, Object element) {
		DefaultMutableTreeNodeKatagory parent = null;
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) pdbs.getModel()
				.getRoot();
		// search the corresponded pdb file in order to get the main
		// information(file, name) of the pdb file and add it
		for (int i = 0; i < root.getChildCount(); i++) {
			if (root.getChildAt(i).toString().equals(element)) {
				String pdbName = ((DefaultMutableTreeNodeFile) root
						.getChildAt(i)).getName();
				String file = ((DefaultMutableTreeNodeFile) root.getChildAt(i))
						.getFile();
				// checks if the protein was added in the model before
				String nodeName = "";
				boolean moven = false;
				boolean notoccur = true;
				int nodeRatio = 0;
				DefaultMutableTreeNodeKatagory rootMembrane = (DefaultMutableTreeNodeKatagory) tree
						.getModel().getRoot();
				int max = rootMembrane.getIdeal_value()
						- rootMembrane.getIs_value();
				if (newKategory) {
					CreateKomponent newKomp = new CreateKomponent(main,
							pdbName, 0, komponentsCounter, max);
					if (newKomp.getMoven()) {
						moven = newKomp.getMoven();
						notoccur = checkOccurence(file,
								(DefaultMutableTreeNode) tree.getModel()
										.getRoot(), newKomp.getKatName(),
								parentNode.getUserObject().toString());
						if (notoccur) {
							parent = addKomponenteAsNode(parentNode, newKomp
									.getKatName(), 0);

							path = path.pathByAddingChild(parent);
							tree.expandPath(path);
							// path = path.pathByAddingChild(parent);
							// tree.expandPath(path);
						} else {
							rejectdrop = true;
							return;
						}
						nodeName = newKomp.getName();
						nodeRatio = newKomp.getRatio();
						komponentsCounter++;
					} else {
						rejectdrop = true;
					}

					// add pdb file to existing kategory
				} else {
					CreateKomponent pdbInfos = new CreateKomponent(main,
							pdbName, 1, 0, max);
					if (pdbInfos.getMoven()) { // parent) {
						nodeName = pdbInfos.getName();
						nodeRatio = pdbInfos.getRatio();
						moven = pdbInfos.getMoven();
					} else {
						rejectdrop = true;
					}
					parent = (DefaultMutableTreeNodeKatagory) parentNode;
					notoccur = checkOccurence(file,
							(DefaultMutableTreeNode) tree.getModel().getRoot(),
							parent.getName(), parent.getParent().toString());

				}
				// if(moven){
				// notoccur = checkOccurence(file,
				// (DefaultMutableTreeNode) tree.getModel()
				// .getRoot(), parent.getName(), parent.getParent().toString());
				// }
				if (moven && notoccur) {



					try {
						MembranModel mm = MembranModelSingleton.getInstance();
						mm.addComponent(pdbName, parent.getName());

						DefaultMutableTreeNodeProtein node = new DefaultMutableTreeNodeProtein(
								nodeName, pdbName, nodeRatio);
						node.setFile(file);					
						
						Protein protein = mm.getProtein(pdbName + "_"
								+ parent.getName());

						node.setProtein(protein);

						float tmp = ((float) nodeRatio / 100);
						protein.setRatio(tmp);
						protein.setName(pdbName);
						protein.setSubcomponent(parent.getName());
						protein.setComponent(parent.getParent().toString());
						node.getProtein().setRatio(tmp);
						protein.setFilepath(node.getFile());
						protein.setYTranslation(0);
						protein.startup();
						protein.setNodeName(nodeName);
						protein.setProteinColor(mm.getColor(mm.getAllProteins()
								.size() - 1));
						protein.setQuantity(nodeRatio);
						Matrix3f pdbRotationMatrixEmpty = new Matrix3f();
						pdbRotationMatrixEmpty.setElement(0, 0, (float) 1.0);
						pdbRotationMatrixEmpty.setElement(1, 1, (float) 1.0);
						pdbRotationMatrixEmpty.setElement(2, 2, (float) 1.0);
						protein.setRotationMatrix(pdbRotationMatrixEmpty);
						DefaultTreeModel model = (DefaultTreeModel) (tree
								.getModel());
						if (checkSize(protein, parent,
								((DefaultMutableTreeNodeKatagory) tree
										.getModel().getRoot()), nodeRatio, node)) {
							model.insertNodeInto(node, parent, parent
									.getChildCount());
							node.setPercent(protein.getQuantity());


						} else {
							mm.removeComponent(pdbName);
							if (newKategory) {
								model.removeNodeFromParent(parent);
								komponentsCounter--;
							}
						}
					} catch (IOException e) {
						// TODO Auto-generated catch block
						DefaultTreeModel model = (DefaultTreeModel) (tree
								.getModel());
						String message = e.toString(); 
						message = message.substring(message.indexOf("Files"), message.indexOf("pdb")+3);
						JOptionPane
								.showMessageDialog(main, "File " +
										message+" not found in the default file directory!");
					}

				} else {
					rejectdrop = true;
				}
				newKategory = false;
			}
		}
	}

	/**
	 * Checks the size of the new constructed protein object.
	 * @param protein Protein
	 * @param katagory DefaultMutableTreeNode
	 * @param parent DefaultMutableTreeNode
	 * @param int (current Protein ratio)
	 * @param node DefaultMutableTreeNodeProtein
	 *
	 */
	private boolean checkSize(Protein protein,
			DefaultMutableTreeNodeKatagory parent,
			DefaultMutableTreeNodeKatagory katagory, int nodeRatio,
			DefaultMutableTreeNodeProtein node) {
		int min_value = protein.get_xz_percentage();
		node.setMin_percent(min_value);
		if (min_value > nodeRatio) { // value is small

			int max_value = (((DefaultMutableTreeNodeKatagory) tree.getModel()
					.getRoot()).getIdeal_value() - ((DefaultMutableTreeNodeKatagory) tree
					.getModel().getRoot()).getIs_value());
			ChangePercent cp = new ChangePercent(main, protein.getPDBcode(),
					"Protein", max_value, min_value, min_value, 1);
			nodeRatio = Integer.parseInt(cp.getRatio());
			parent.update_is_value(nodeRatio);
			katagory.update_is_value(nodeRatio);

			float tmp = ((float) nodeRatio / 100);
			protein.setRatio(tmp);
			protein.setQuantity(nodeRatio);
			return cp.getChangen();
		} else {
			parent.update_is_value(nodeRatio);
			katagory.update_is_value(nodeRatio);
			return true;
		}

	}

	private boolean checkOccurence(String file, DefaultMutableTreeNode root,
			String parent, String parentparent) {
		// children of root: Ioenenkanal

		for (int i = 0; i < 3; i++) {
			DefaultMutableTreeNode kategorie = (DefaultMutableTreeNode) root
					.getChildAt(i);
			// children von Kategorien
			for (int j = 0; j < kategorie.getChildCount(); j++) {
				DefaultMutableTreeNode subkategorie = (DefaultMutableTreeNode) kategorie
						.getChildAt(j);
				// pdb

				for (int k = 0; k < subkategorie.getChildCount(); k++) {

					if ((!(parentparent.equals(kategorie.getUserObject())))// falls
																			// in
																			// verschieden
																			// kategorien
							&& (((DefaultMutableTreeNodeProtein) subkategorie
									.getChildAt(k)).getFile().equals(file))) {
						JOptionPane.showMessageDialog(main,
								"There exists the same pdb file "
										+ (subkategorie.getChildAt(k)
												.toString()) + " in "
										+ kategorie.getUserObject());
						rejectdrop = true;
						newKategory = false;
						created = false;
						return false;
					}
					if (// falls in verschieden kategorien

					((parent.equals(subkategorie.getUserObject())) // falls
																	// nicht in
																	// verschiedenn
																	// subKategorien
					&&

					(((DefaultMutableTreeNodeProtein) subkategorie
							.getChildAt(k)).getFile().equals(file)))
					// falls pdb name gleich => im selben parent
					// node

					) {
						JOptionPane.showMessageDialog(main,
								"There exists the same pdb file "
										+ (subkategorie.getChildAt(k)
												.toString()) + " in "
										+ kategorie.getUserObject());
						rejectdrop = true;
						newKategory = false;
						created = false;
						return false;
					}
				}
			}
		}
		return true;
	}

	private DefaultMutableTreeNodeKatagory addKomponenteAsNode(
			DefaultMutableTreeNode parent, String element, int percent) {
		DefaultMutableTreeNodeKatagory node = new DefaultMutableTreeNodeKatagory(
				element, percent);
		DefaultTreeModel model = (DefaultTreeModel) (tree.getModel());
		model.insertNodeInto(node, parent, parent.getChildCount());
		return node;
	}

	/**
	 * @return Returns the komponentsCounter.
	 */
	public int getKomponentsCounter() {
		return komponentsCounter;
	}

	/**
	 * @param komponentsCounter
	 *            The komponentsCounter to set.
	 */
	public void setKomponentsCounter(int komponentsCounter) {
		this.komponentsCounter = komponentsCounter;
	}
}
