package msg;

import gui.Msg_panel;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;
import java.util.Vector;
import javax.swing.JPanel;

public class ProteinElement extends JPanel implements MouseListener,MouseMotionListener,ActionListener {
	
	String id, component, subcomponent, key, pdbcode;
	String rotateProteinText = "Rotate protein";
	
	boolean pressed;
	
	int oldMouseX, oldMouseY, startMouseX, startMouseY;
	int pX, pY, pXnew, pYnew, sizeX, sizeY, ratio;
	int xa, ya, vWidth, vHeight;
	int mOffsetX, mOffsetY;
	int borderSize    = 1;
	int borderSizeHl  = 3;
	int rotationAngle = 0;
	
	Color colorBackground;	
	Color colorBorder            = Color.BLACK;
	Color colorText              = new Color(180,240,250); //new Color(18,4,50);	
	Color colorBackgroundHl      = this.colorBackground;
	Color colorBorderHl          = Color.WHITE;
	Color colorBackgroundPressed = new Color(29,3,111);
	Color colorTextPressed       = new Color(180,240,250);	
	
	JLabel proteinLabel;
	JPopupMenu popupRotate = new JPopupMenu();	
	
	Msg_panel alles;
	ProteinMatrix proteinMatrix;
	Vector lipids;
		
	//######################################################################################
	//##                                 MAIN STUFF                                       ##
	//######################################################################################
	
	/**
	 * Initializes a new ProteinElement.
	 * 
	 * @param alles Msg_panel
	 * @param panel ProteinMatrix
	 * @param pdbcode String
	 * @param id String
	 * @param component String
	 * @param subcomponent String
	 * @param x int
	 * @param y int
	 * @param vWidth int
	 * @param vHeight int
	 * @param colorBackground Color
	 * @param pX int
	 * @param pY int
	 * @param sizeX int
	 * @param sizeY int
	 * @param ratio int
	 * @param mOffsetX int
	 * @param mOffsetY int
	 * @param rotation int
	 * @param lipids Vector
	 */
	public ProteinElement(Msg_panel alles,
				   ProteinMatrix panel,
				   String pdbcode,
				   String id,
				   String component,
				   String subcomponent,
				   int x,
				   int y,
				   int vWidth,
				   int vHeight,
				   Color colorBackground,
				   int pX,
				   int pY,
				   int sizeX,
				   int sizeY,
				   int ratio,
				   int mOffsetX,
				   int mOffsetY,
				   int rotation,
				   Vector lipids) {
		
		this.alles = alles;
		this.proteinMatrix = panel;
		this.pdbcode = pdbcode;
		this.id = id;
		this.component = component;
		this.subcomponent = subcomponent;
		this.key = pdbcode+"_"+subcomponent;
		this.xa = x;
		this.ya = y;		
		this.vWidth = vWidth;
		this.vHeight = vHeight;
		this.colorBackground = colorBackground;
		this.pX = pX;
		this.pY = pY;
		this.sizeX = sizeX;
		this.sizeY = sizeY;
		this.ratio = ratio;
		this.mOffsetX = mOffsetX;
		this.mOffsetY = mOffsetY;
		this.rotationAngle = rotation;
		this.lipids = lipids;
		
		createProteinElement(this.id, this.xa, this.ya, this.vWidth, this.vHeight);
		addMouseListener(this);
		addMouseMotionListener(this);
		
		JMenuItem menuitemRotate = new JMenuItem(this.rotateProteinText);
		menuitemRotate.addActionListener(this);
		popupRotate.add(menuitemRotate);
		
		String toolText = generateToolTip();
		setToolTipText(toolText);
	}
	
	/**
	 * Creates a new ProteinElement.
	 * 
	 * @param id String
	 * @param x int
	 * @param y int
	 * @param sizeX int
	 * @param sizeY int
	 */
	public void createProteinElement(String id, int x, int y, int sizeX, int sizeY) {
		setSize(sizeX, sizeY);
		setLocation(x,y);
		setBorder(BorderFactory.createLineBorder(this.colorBorder, 1));		
		setBackground(this.colorBackground);
		String labelText = "<HTML><CENTER>";
		labelText += this.pdbcode;
		labelText += "<CENTER></HTML>";
		this.proteinLabel = new JLabel();
		this.proteinLabel.setFont(new Font("",Font.PLAIN,10));
		this.proteinLabel.setSize(sizeX, sizeY);
		this.proteinLabel.setText(labelText);
		this.proteinLabel.setForeground(this.colorText);
		//add(this.proteinLabel);
		this.revalidate();
	}

	/**
	 * Rotates an ProteinElement with 90.
	 * 
	 * @return boolean (rotations finished/possible)
	 */
	public boolean rotateProteinElement() {
		if (!testProteinMatrixPosition()) {
			return false;
		} else {
			this.proteinMatrix.removeBusyCords(this.pX, this.pY, this.sizeX, this.sizeY);
			int newX = this.pX + (int)((this.sizeX / 2) - (this.sizeY / 2));
			int newY = this.pY + (int)((this.sizeY / 2) - (this.sizeX / 2));
			if (this.proteinMatrix.testCords(newX, newY, this.sizeY, this.sizeX)) {	
				this.proteinMatrix.printCords();
				this.proteinMatrix.setBusyCords(this.pX, this.pY, this.sizeX, this.sizeY);
				return false;
			} else {
				changeLipidCubeRotation(this.sizeX, this.sizeY, this.rotationAngle);
				int tmp = this.vHeight;
				this.vHeight = this.vWidth;
				this.vWidth = tmp;
				setSize(this.vWidth, this.vHeight);
				this.revalidate();
				tmp = this.sizeX;
				this.sizeX = this.sizeY;
				this.sizeY = tmp;
				this.pX = newX;
				this.pY = newY;
				setLocation(this.pX*this.ratio+this.mOffsetX, this.pY*this.ratio+this.mOffsetY);
				this.proteinMatrix.setBusyCords(this.pX, this.pY, this.sizeX, this.sizeY);
				this.proteinMatrix.printCords();
				this.rotationAngle = (this.rotationAngle%360) + 90;
			}	
			this.repaint();
			return true;
		}
	}
	
	/**
	 * Tests if the element could be moved to the new position in the given matrix.
	 * 
	 * @return boolean (test the new position for the element)
	 */
	public boolean testProteinMatrixPosition() {		
		int newXminus = this.pX + (int)((this.sizeX / 2) - (this.sizeY / 2));
		int newYminus = this.pY + (int)((this.sizeY / 2) - (this.sizeX / 2));
		int newXplus  = newXminus + this.sizeY;
		int newYplus  = newYminus + this.sizeX;
		if ((newXplus > this.alles.getMWidth())  ||
			(newYplus > this.alles.getMHeight()) ||
			(newXminus < 0)                      ||
			(newYminus < 0)) {
			return false;			
		} else {
			return true;
		}
	}
	
	/**
	 * Paints the component.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.setColor(Color.WHITE);		
		for (int i=0; i<this.lipids.size(); i++) {
			g.drawOval(this.ratio*((int[])this.lipids.get(i))[0],
					   this.ratio*((int[])this.lipids.get(i))[1],
					   this.ratio,
					   this.ratio);			
		}
	}
	
	/**
	 * Changes the values in the matrix of the lipids for a rotation of 90 of the matrix.
	 * 
	 * @param sizeX int
	 * @param sizeY int
	 * @param rotation int
	 */
	public void changeLipidCubeRotation(int sizeX, int sizeY, int rotation) {
		Vector tmp = new Vector();
		int[] newV;
		for (int i=0; i<this.lipids.size(); i++) {
			int tmpX = ((int[])this.lipids.get(i))[0];
			int tmpY = ((int[])this.lipids.get(i))[1];
			newV = new int[2];
			newV[0] = sizeY - tmpY - 1;
			newV[1] = tmpX;
			//System.out.println(tmpX+"_"+tmpY);
			tmp.add(newV);
		}		
		this.lipids = tmp;
	}
	
	/**
	 * Generates the tooltip for this component.
	 * 
	 * @return String (contains the tooltip content)
	 */
	public String generateToolTip() {
		String tooltip = "<html>";
		tooltip += "<table border=0><tr><td>";
		tooltip += "<b>Component:</b></td><td>"+this.getComponent()+"</td></tr><tr><td>";
		tooltip += "<b>Subcomponent:</b></td><td>"+this.getSubcomponent()+"</td></tr><tr><td>";
		tooltip += "<b>PDB Code:</b></td><td>"+this.getPdbcode()+"</td></tr>";
		//tooltip += "<tr><td><b>ID:</b></td><td>"+this.getLabeltext()+"</td></tr>";		
		tooltip += "</table>";
		tooltip += "</html>";
		return tooltip; 
	}
	
	//######################################################################################
	//##                                   MOUSE EVENTS                                   ##
	//######################################################################################
	
	/**
	 * Features when the mouse enteres this component. 
	 */
	public void mouseEntered(MouseEvent e) {
		if (this.pressed) {
			setBackground(this.colorBackgroundPressed);	
		}
		activateHighlight();
	}
	
	/**
	 * Features when the mouse lefts this component.
	 */
	public void mouseExited(MouseEvent e) {
		if (!this.pressed) {
			deactivateHighlight();
		}
	}	
	
	/**
	 * Features when the mouse button is pressed.
	 */
	public void mousePressed(MouseEvent e) {
		if (!e.isMetaDown()) {
		this.pressed = true;
		setBackground(this.colorBackgroundPressed);
		this.proteinLabel.setForeground(this.colorTextPressed);
		this.proteinMatrix.removeBusyCords(this.pX, this.pY, this.sizeX, this.sizeY);
		this.proteinMatrix.printCords();
		this.startMouseX = this.pX*this.proteinMatrix.getRatioWidth();
		this.startMouseY = this.pY*this.proteinMatrix.getRatioHeight();
		this.oldMouseX = this.startMouseX;
		this.oldMouseY = this.startMouseY;
		}
	}
	
	/**
	 * Features when the mouse button is released.
	 */
	public void mouseReleased(MouseEvent e) {
		if (e.isMetaDown()) {
			popupRotate.show(this, (e.getX() + 5), (e.getY() + 5));
		} else {
		this.pressed = false;
		this.alles.setClickedProtein(this);
		this.alles.setClickedProteinSet(true);
		setBackground(this.colorBackground);
		this.proteinLabel.setForeground(this.colorText);
		this.pXnew = this.pX+(int)((this.oldMouseX-this.startMouseX)/this.proteinMatrix.getRatioWidth());
		this.pYnew = this.pY+(int)((this.oldMouseY-this.startMouseY)/this.proteinMatrix.getRatioHeight());;
		this.pXnew += (Math.abs(this.oldMouseX-this.startMouseX)%this.proteinMatrix.getRatioWidth())>(this.proteinMatrix.getRatioWidth()/2) ? (this.oldMouseX>this.startMouseX ? 1 : -1) : 0; 
		this.pYnew += (Math.abs(this.oldMouseY-this.startMouseY)%this.proteinMatrix.getRatioHeight())>(this.proteinMatrix.getRatioHeight()/2) ? (this.oldMouseY>this.startMouseY ? 1 : -1) : 0;
		if (this.proteinMatrix.testCords(this.pXnew, this.pYnew, this.sizeX, this.sizeY)) {
			this.pXnew = this.pX;
			this.pYnew = this.pY;
		}
		if (this.proteinMatrix.isDebugModus()) System.out.print("X: "+this.pXnew+" - "+this.pX+" = ");
		if (this.proteinMatrix.isDebugModus()) System.out.println((int)((this.oldMouseX-this.startMouseX)/this.proteinMatrix.getRatioWidth()));
		if (this.proteinMatrix.isDebugModus()) System.out.print("Y: "+this.pYnew+" - "+this.pY+" = ");
		if (this.proteinMatrix.isDebugModus()) System.out.println((int)((this.oldMouseY-this.startMouseY)/this.proteinMatrix.getRatioHeight()));
		this.proteinMatrix.setBusyCords(this.pXnew, this.pYnew, this.sizeX, this.sizeY);
		this.proteinMatrix.printCords();
		setLocation(this.pXnew*this.proteinMatrix.getRatioWidth()+this.mOffsetX,this.pYnew*this.proteinMatrix.getRatioHeight()+this.mOffsetY);
		this.pX = this.pXnew;
		this.pY = this.pYnew;
		this.startMouseX = this.pXnew*this.proteinMatrix.getRatioWidth();
		this.startMouseY = this.pYnew*this.proteinMatrix.getRatioHeight();
		}
	}

	/**
	 * Features when the mouse is dragged with pressed left button.
	 */
	public void mouseDragged(MouseEvent e) {
		if (!e.isMetaDown()) {
		this.xa += e.getX()-(int)(this.getWidth()/2);
		this.ya += e.getY()-(int)(this.getHeight()/2);
		if ((this.xa-this.mOffsetX) <= 0 && (this.ya-this.mOffsetY) <= 0) {
			setLocation(1+this.mOffsetX,1+this.mOffsetY);
			this.oldMouseX = 1;
			this.oldMouseY = 1;
		} else if ((this.xa+this.vWidth) >= (this.proteinMatrix.getMatrixWidth()+this.mOffsetX) && (this.ya-this.mOffsetY) <= 0) {
			setLocation((this.proteinMatrix.getMatrixWidth()-this.vWidth-1+this.mOffsetX),1+this.mOffsetY);
			this.oldMouseX = (this.proteinMatrix.getMatrixWidth()-this.vWidth-1);
			this.oldMouseY = 1;
		} else if ((this.xa-this.mOffsetX) <= 0 && (this.ya+this.vHeight) >= (this.proteinMatrix.getMatrixHeight()+this.mOffsetY)) {
			setLocation(1+this.mOffsetX,(this.proteinMatrix.getMatrixHeight()-this.vHeight-1+this.mOffsetY));
			this.oldMouseX = 1;
			this.oldMouseY = (this.proteinMatrix.getMatrixHeight()-this.vHeight-1);
		} else if ((this.xa+this.vWidth) >= (this.proteinMatrix.getMatrixWidth()+this.mOffsetX) && (this.ya+this.vHeight) >= (this.proteinMatrix.getMatrixHeight()+this.mOffsetY)) {
			setLocation((this.proteinMatrix.getMatrixWidth()-this.vWidth-1+this.mOffsetX),(this.proteinMatrix.getMatrixHeight()-this.vHeight-1+this.mOffsetY));
			this.oldMouseX = (this.proteinMatrix.getMatrixWidth()-this.vWidth-1);
			this.oldMouseY = (this.proteinMatrix.getMatrixHeight()-this.vHeight-1);
		} else if ((this.xa+this.vWidth) >= (this.proteinMatrix.getMatrixWidth()+this.mOffsetX)) {
			setLocation((this.proteinMatrix.getMatrixWidth()-this.vWidth-1+this.mOffsetX),ya);
			this.oldMouseX = (this.proteinMatrix.getMatrixWidth()-this.vWidth-1);
			this.oldMouseY = ya-this.mOffsetY;
		} else if ((this.ya+this.vHeight) >= (this.proteinMatrix.getMatrixHeight()+this.mOffsetY)) {
			setLocation(xa,(this.proteinMatrix.getMatrixHeight()-this.vHeight-1+this.mOffsetY));
			this.oldMouseX = xa-this.mOffsetX;
			this.oldMouseY = (this.proteinMatrix.getMatrixHeight()-this.vHeight-1);
		} else if ((this.xa) <= (0+this.mOffsetX)) {
			setLocation(1+this.mOffsetX,ya);
			this.oldMouseX = 1;
			this.oldMouseY = ya-this.mOffsetY;
		} else if ((this.ya) <= (0+this.mOffsetY)) {
			setLocation(xa,1+this.mOffsetY);
			this.oldMouseX = xa-this.mOffsetX;
			this.oldMouseY = 1;
		} else {
			setLocation(xa,ya);
			this.oldMouseX = xa-this.mOffsetX;
			this.oldMouseY = ya-this.mOffsetY;
		}
		getParent().repaint();
		}
	}
	
	/**
	 * Features when the mouse is moved.
	 */
	public void mouseMoved(MouseEvent e) {
		//if (this.proteinMatrix.isDebugModus()) System.out.println("moved");
	}
	
	/**
	 * Features when the mouse button is clicked.
	 */
	public void mouseClicked(MouseEvent e) {
		//if (this.proteinMatrix.isDebugModus()) System.out.println("clicked");		
	}
	
	/**
	 * Highlights all components with the same id.
	 *
	 */
	public void activateHighlight() {
		for (int i=0; i<(this.alles.getVectorE().size()); i++) {
			if (this.id.equals((String)this.alles.getProteinFromVector(i).getId())) {
				this.alles.getProteinFromVector(i).setBorder(BorderFactory.createLineBorder(this.colorBorderHl,this.borderSizeHl));				
			}
		}
	}
	
	/**
	 * Remove highlight of all components with the same id.
	 *
	 */
	public void deactivateHighlight() {
		for (int i=0; i<(this.alles.getVectorE().size()); i++) {
			if (this.id.equals((String)this.alles.getProteinFromVector(i).getId())) {
				this.alles.getProteinFromVector(i).setBorder(BorderFactory.createLineBorder(this.colorBorder,this.borderSize));
				this.alles.getProteinFromVector(i).setForeground(this.colorText);
			}
		}
	}
	
	/**
	 * Performs the action of the action command.
	 */
	public void actionPerformed(ActionEvent event) {
		String cmd = event.getActionCommand();

		if (cmd.equals(rotateProteinText)) {
			if (!this.rotateProteinElement()) {
				//System.out.println("ERROR: rotate not possible !");
			} else {
				//System.out.println("rotate done");
			}
		}
	}
	
	/**
	 * @return Returns the lipids.
	 */
	public Vector getLipids2() {
		Vector lipids2 = new Vector();
		Vector tmp = getLipids();
		for (int j=0; j<tmp.size(); j++) {
			int[] tmpInt = new int[2];
			tmpInt[0] = ((int[])tmp.get(j))[0]+getPX();
			tmpInt[1] = ((int[])tmp.get(j))[1]+getPY();
			lipids2.add(tmpInt);
		}
		return lipids2;
	}

	//######################################################################################
	//##                              GETTER AND SETTER                                   ##
	//######################################################################################
		
	/**
	 * @return Returns the alles.
	 */
	public Msg_panel getAlles() {
		return alles;
	}

	/**
	 * @param alles The alles to set.
	 */
	public void setAlles(Msg_panel alles) {
		this.alles = alles;
	}

	/**
	 * @return Returns the borderSize.
	 */
	public int getBorderSize() {
		return borderSize;
	}

	/**
	 * @param borderSize The borderSize to set.
	 */
	public void setBorderSize(int borderSize) {
		this.borderSize = borderSize;
	}

	/**
	 * @return Returns the borderSizeHl.
	 */
	public int getBorderSizeHl() {
		return borderSizeHl;
	}

	/**
	 * @param borderSizeHl The borderSizeHl to set.
	 */
	public void setBorderSizeHl(int borderSizeHl) {
		this.borderSizeHl = borderSizeHl;
	}

	/**
	 * @return Returns the colorBackground.
	 */
	public Color getColorBackground() {
		return colorBackground;
	}

	/**
	 * @param colorBackground The colorBackground to set.
	 */
	public void setColorBackground(Color colorBackground) {
		this.colorBackground = colorBackground;
	}

	/**
	 * @return Returns the colorBackgroundHl.
	 */
	public Color getColorBackgroundHl() {
		return colorBackgroundHl;
	}

	/**
	 * @param colorBackgroundHl The colorBackgroundHl to set.
	 */
	public void setColorBackgroundHl(Color colorBackgroundHl) {
		this.colorBackgroundHl = colorBackgroundHl;
	}

	/**
	 * @return Returns the colorBackgroundPressed.
	 */
	public Color getColorBackgroundPressed() {
		return colorBackgroundPressed;
	}

	/**
	 * @param colorBackgroundPressed The colorBackgroundPressed to set.
	 */
	public void setColorBackgroundPressed(Color colorBackgroundPressed) {
		this.colorBackgroundPressed = colorBackgroundPressed;
	}

	/**
	 * @return Returns the colorBorder.
	 */
	public Color getColorBorder() {
		return colorBorder;
	}

	/**
	 * @param colorBorder The colorBorder to set.
	 */
	public void setColorBorder(Color colorBorder) {
		this.colorBorder = colorBorder;
	}

	/**
	 * @return Returns the colorBorderHl.
	 */
	public Color getColorBorderHl() {
		return colorBorderHl;
	}

	/**
	 * @param colorBorderHl The colorBorderHl to set.
	 */
	public void setColorBorderHl(Color colorBorderHl) {
		this.colorBorderHl = colorBorderHl;
	}

	/**
	 * @return Returns the colorText.
	 */
	public Color getColorText() {
		return colorText;
	}

	/**
	 * @param colorText The colorText to set.
	 */
	public void setColorText(Color colorText) {
		this.colorText = colorText;
	}

	/**
	 * @return Returns the colorTextPressed.
	 */
	public Color getColorTextPressed() {
		return colorTextPressed;
	}

	/**
	 * @param colorTextPressed The colorTextPressed to set.
	 */
	public void setColorTextPressed(Color colorTextPressed) {
		this.colorTextPressed = colorTextPressed;
	}

	/**
	 * @return Returns the component.
	 */
	public String getComponent() {
		return component;
	}

	/**
	 * @param component The component to set.
	 */
	public void setComponent(String component) {
		this.component = component;
	}

	/**
	 * @return Returns the id.
	 */
	public String getId() {
		return id;
	}

	/**
	 * @param id The id to set.
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @return Returns the key.
	 */
	public String getKey() {
		return key;
	}

	/**
	 * @param key The key to set.
	 */
	public void setKey(String key) {
		this.key = key;
	}

	/**
	 * @return Returns the lipids.
	 */
	public Vector getLipids() {
		return lipids;
	}	

	/**
	 * @param lipids The lipids to set.
	 */
	public void setLipids(Vector lipids) {
		this.lipids = lipids;
	}

	/**
	 * @return Returns the mOffsetX.
	 */
	public int getMOffsetX() {
		return mOffsetX;
	}

	/**
	 * @param offsetX The mOffsetX to set.
	 */
	public void setMOffsetX(int offsetX) {
		mOffsetX = offsetX;
	}

	/**
	 * @return Returns the mOffsetY.
	 */
	public int getMOffsetY() {
		return mOffsetY;
	}

	/**
	 * @param offsetY The mOffsetY to set.
	 */
	public void setMOffsetY(int offsetY) {
		mOffsetY = offsetY;
	}

	/**
	 * @return Returns the oldMouseX.
	 */
	public int getOldMouseX() {
		return oldMouseX;
	}

	/**
	 * @param oldMouseX The oldMouseX to set.
	 */
	public void setOldMouseX(int oldMouseX) {
		this.oldMouseX = oldMouseX;
	}

	/**
	 * @return Returns the oldMouseY.
	 */
	public int getOldMouseY() {
		return oldMouseY;
	}

	/**
	 * @param oldMouseY The oldMouseY to set.
	 */
	public void setOldMouseY(int oldMouseY) {
		this.oldMouseY = oldMouseY;
	}

	/**
	 * @return Returns the pdbcode.
	 */
	public String getPdbcode() {
		return pdbcode;
	}

	/**
	 * @param pdbcode The pdbcode to set.
	 */
	public void setPdbcode(String pdbcode) {
		this.pdbcode = pdbcode;
	}

	/**
	 * @return Returns the popupRotate.
	 */
	public JPopupMenu getPopupRotate() {
		return popupRotate;
	}

	/**
	 * @param popupRotate The popupRotate to set.
	 */
	public void setPopupRotate(JPopupMenu popupRotate) {
		this.popupRotate = popupRotate;
	}

	/**
	 * @return Returns the pressed.
	 */
	public boolean isPressed() {
		return pressed;
	}

	/**
	 * @param pressed The pressed to set.
	 */
	public void setPressed(boolean pressed) {
		this.pressed = pressed;
	}

	/**
	 * @return Returns the proteinLabel.
	 */
	public JLabel getProteinLabel() {
		return proteinLabel;
	}

	/**
	 * @param proteinLabel The proteinLabel to set.
	 */
	public void setProteinLabel(JLabel proteinLabel) {
		this.proteinLabel = proteinLabel;
	}

	/**
	 * @return Returns the proteinMatrix.
	 */
	public ProteinMatrix getProteinMatrix() {
		return proteinMatrix;
	}

	/**
	 * @param proteinMatrix The proteinMatrix to set.
	 */
	public void setProteinMatrix(ProteinMatrix proteinMatrix) {
		this.proteinMatrix = proteinMatrix;
	}

	/**
	 * @return Returns the pX.
	 */
	public int getPX() {
		return pX;
	}

	/**
	 * @param px The pX to set.
	 */
	public void setPX(int px) {
		pX = px;
	}

	/**
	 * @return Returns the pXnew.
	 */
	public int getPXnew() {
		return pXnew;
	}

	/**
	 * @param xnew The pXnew to set.
	 */
	public void setPXnew(int xnew) {
		pXnew = xnew;
	}

	/**
	 * @return Returns the pY.
	 */
	public int getPY() {
		return pY;
	}

	/**
	 * @param py The pY to set.
	 */
	public void setPY(int py) {
		pY = py;
	}

	/**
	 * @return Returns the pYnew.
	 */
	public int getPYnew() {
		return pYnew;
	}

	/**
	 * @param ynew The pYnew to set.
	 */
	public void setPYnew(int ynew) {
		pYnew = ynew;
	}

	/**
	 * @return Returns the ratio.
	 */
	public int getRatio() {
		return ratio;
	}

	/**
	 * @param ratio The ratio to set.
	 */
	public void setRatio(int ratio) {
		this.ratio = ratio;
	}

	/**
	 * @return Returns the rotateProteinText.
	 */
	public String getRotateProteinText() {
		return rotateProteinText;
	}

	/**
	 * @param rotateProteinText The rotateProteinText to set.
	 */
	public void setRotateProteinText(String rotateProteinText) {
		this.rotateProteinText = rotateProteinText;
	}

	/**
	 * @return Returns the rotationAngle.
	 */
	public int getRotationAngle() {
		return rotationAngle;
	}

	/**
	 * @param rotationAngle The rotationAngle to set.
	 */
	public void setRotationAngle(int rotationAngle) {
		this.rotationAngle = rotationAngle;
	}

	/**
	 * @return Returns the sizeX.
	 */
	public int getSizeX() {
		return sizeX;
	}

	/**
	 * @param sizeX The sizeX to set.
	 */
	public void setSizeX(int sizeX) {
		this.sizeX = sizeX;
	}

	/**
	 * @return Returns the sizeY.
	 */
	public int getSizeY() {
		return sizeY;
	}

	/**
	 * @param sizeY The sizeY to set.
	 */
	public void setSizeY(int sizeY) {
		this.sizeY = sizeY;
	}

	/**
	 * @return Returns the startMouseX.
	 */
	public int getStartMouseX() {
		return startMouseX;
	}

	/**
	 * @param startMouseX The startMouseX to set.
	 */
	public void setStartMouseX(int startMouseX) {
		this.startMouseX = startMouseX;
	}

	/**
	 * @return Returns the startMouseY.
	 */
	public int getStartMouseY() {
		return startMouseY;
	}

	/**
	 * @param startMouseY The startMouseY to set.
	 */
	public void setStartMouseY(int startMouseY) {
		this.startMouseY = startMouseY;
	}

	/**
	 * @return Returns the subcomponent.
	 */
	public String getSubcomponent() {
		return subcomponent;
	}

	/**
	 * @param subcomponent The subcomponent to set.
	 */
	public void setSubcomponent(String subcomponent) {
		this.subcomponent = subcomponent;
	}

	/**
	 * @return Returns the vHeight.
	 */
	public int getVHeight() {
		return vHeight;
	}

	/**
	 * @param height The vHeight to set.
	 */
	public void setVHeight(int height) {
		vHeight = height;
	}

	/**
	 * @return Returns the vWidth.
	 */
	public int getVWidth() {
		return vWidth;
	}

	/**
	 * @param width The vWidth to set.
	 */
	public void setVWidth(int width) {
		vWidth = width;
	}

	/**
	 * @return Returns the xa.
	 */
	public int getXa() {
		return xa;
	}

	/**
	 * @param xa The xa to set.
	 */
	public void setXa(int xa) {
		this.xa = xa;
	}

	/**
	 * @return Returns the ya.
	 */
	public int getYa() {
		return ya;
	}

	/**
	 * @param ya The ya to set.
	 */
	public void setYa(int ya) {
		this.ya = ya;
	}
}