package msg;

import gui.Msg_panel;
import javax.swing.*;
import membranModel.MembranModel;
import membranModel.MembranModelSingleton;
import protein.Protein;
import java.awt.*;
import java.util.Hashtable;
import java.util.Random;
import java.util.Vector;

public class ProteinMatrix extends JPanel {
	
	Msg_panel alles;
	
	int mHeight, mWidth, wHeight, wWidth;
	int ratioWidth, ratioHeight;
	int resizeValue = 0;
	int maxTry = 5000;
	int mZoom = 0;
	int mZoomFactor = 10; // in percent
	boolean debugModus = false;
	//Color colorBackground = new Color(253,255,231);
	Color colorBackground = MembranModelSingleton.getInstance().getBackground_color();
	Vector elementList = new Vector();
	MembranModel membranModel;
		
	Hashtable busy = new Hashtable();
	
	MatrixProgressBar progbar;		
	
	//######################################################################################
	//##                                 MAIN STUFF                                       ##
	//######################################################################################
	
	/**
	 * Initializes a new ProteinMatrix.
	 * Execute the calculateMatrixStartSize() and the generateMatrix() functions. 
	 * 
	 * @param alles Msg_panel
	 * @param wWidth int (panel width)
	 * @param wHeight int (panel height)
	 */
	public ProteinMatrix(Msg_panel alles, int wWidth, int wHeight) {
		this.alles        = alles;
		this.mHeight      = alles.getMHeight();
		this.mWidth       = alles.getMWidth();
		this.membranModel = MembranModelSingleton.getInstance();
		this.elementList  = this.membranModel.getAllProteins();
		calculateMatrixStartSize(wWidth, wHeight, this.mWidth, this.mHeight);
		
		if (this.debugModus) System.out.println("Different Proteins: "+this.elementList.size());
		setBackground(this.colorBackground);
		setLayout(null);
		generateMatrix(this.wWidth, this.wHeight);		
	}
	
	/**
	 * Generate the matrix of the lipids and executes the calculateRatio() function.
	 * 
	 * @param wWidth int (panel width)
	 * @param wHeight int (panel height)
	 */
	public void generateMatrix(int wWidth, int wHeight) {
		setPreferredSize(new Dimension(wWidth+1, wHeight+1));
		if (this.debugModus) System.out.println("wWidth: "+wWidth+" - wHeight: "+wHeight);				
		calculateRatio(wHeight, wWidth, this.mHeight, this.mWidth);
		this.wWidth = wWidth;
		this.wHeight = wHeight;
	}
	
	/**
	 * Change the properties of the component.
	 * These properties are the size, location and ratio.
	 *
	 */
	public void changeElements() {
		for (int i=0; i<this.getComponents().length; i++) {
			ProteinElement tmp = (ProteinElement)this.getComponent(i);
			tmp.setSize(((tmp.getWidth()-1)/tmp.getRatio()*this.ratioWidth)+1,
						((tmp.getHeight()-1)/tmp.getRatio()*this.ratioHeight)+1);
			tmp.setLocation(tmp.getPX()*this.ratioWidth+this.alles.getMOffsetX(),
							tmp.getPY()*this.ratioHeight+this.alles.getMOffsetY());
			tmp.setRatio(this.ratioWidth);
			tmp.setVWidth(tmp.getWidth());
			tmp.setVHeight(tmp.getHeight());
			tmp.setMOffsetX(this.alles.getMOffsetX());
			tmp.setMOffsetY(this.alles.getMOffsetY());
		}
	}
	
	/**
	 * Changes the component if the window is resized.
	 *
	 */
	public void changeResize() {
		if (this.debugModus) System.out.println("tw"+this.getWidth());
		if (this.debugModus) System.out.println("th"+this.getHeight());
		if (this.debugModus) System.out.println("ww"+this.wWidth);
		if (this.debugModus) System.out.println("wh"+this.wHeight);
		if ((this.wWidth < (this.getWidth()-1)) && (this.wHeight < (this.getHeight()-1))) {
			if (this.debugModus) System.out.println("resize");
			if (this.debugModus) System.out.println("resizeValue: "+this.resizeValue);
			this.alles.setMOffsetX((int) (this.getWidth() - this.wWidth) / 2);
			this.alles.setMOffsetY((int) (this.getHeight() - this.wHeight) / 2);
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			this.repaint();
			this.revalidate(); 
		} else if (this.wWidth < (this.getWidth()-1))  {
			if (this.debugModus) System.out.println("resize w <");
			this.alles.setMOffsetX((int)((this.getWidth()-1-this.wWidth)/2));
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			this.repaint();
			this.revalidate();
		} else if (this.wHeight < (this.getHeight()-1))  {
			if (this.debugModus) System.out.println("resize h <");
			this.alles.setMOffsetY((int)((this.getHeight()-1-this.wHeight)/2));
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			this.repaint();
			this.revalidate();
		} else if (((this.wWidth+this.alles.getMOffsetX()) > (this.getWidth()-1)) && ((this.wHeight+this.alles.getMOffsetY()) > (this.getHeight()-1))) {
			if (this.debugModus) System.out.println("resize w > & h >");
			this.alles.setMOffsetX(0);
			this.alles.setMOffsetY(0);
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			this.repaint();
			this.revalidate();
		} else if ((this.wWidth+this.alles.getMOffsetX()) > (this.getWidth()-1))  {
			if (this.debugModus) System.out.println("resize w >");
			this.alles.setMOffsetX(0);
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			this.repaint();
			this.revalidate();
		} else if ((this.wHeight+this.alles.getMOffsetY()) > (this.getHeight()-1))  {
			if (this.debugModus) System.out.println("resize h >");
			this.alles.setMOffsetY(0);
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			this.repaint();
			this.revalidate();
		}
	}
	
	/**
	 * Change the size of the component to zoom in.
	 *
	 */
	public void changeZoomIn() {
		int wWidthOld  = this.wWidth;
		//int wHeightOld = this.wHeight;
		this.wWidth  += (int)(this.wWidth*mZoomFactor/100);
		this.wWidth   = (int)(this.wWidth/mWidth)*mWidth;
		this.wHeight += (int)(this.wHeight*mZoomFactor/100);
		this.wHeight  = (int)(this.wHeight/mHeight)*mHeight;
		if ((wWidthOld/mWidth) == this.wWidth/mWidth) {
			this.wWidth  = ((int)this.wWidth/mWidth+1)*mWidth;
			this.wHeight = ((int)this.wHeight/mHeight+1)*mHeight;
		}
		mZoom += (int)(this.wWidth/mWidth)-(int)(wWidthOld/mWidth);
		if (this.debugModus) System.out.println("ZOOM IN: "+mZoom);		
		this.alles.setMOffsetX((this.getWidth() < this.wWidth) ? 0 : (int) (this.getWidth() - this.wWidth) / 2);
		this.alles.setMOffsetY((this.getHeight() < this.wHeight) ? 0 : (int) (this.getHeight() - this.wHeight) / 2);
		generateMatrix(this.wWidth,this.wHeight);
		changeElements();
		repaint();
		revalidate();
	}
	
	/**
	 * Change the size of the component to zoom out.
	 *
	 */
	public void changeZoomOut() {
		if (this.ratioHeight > 1) {
			int wWidthOld  = this.wWidth;
			//int wHeightOld = this.wHeight;
			this.wWidth  -= (int)(this.wWidth*mZoomFactor/100);
			this.wWidth   = (int)(this.wWidth/mWidth)*mWidth;
			this.wHeight -= (int)(this.wHeight*mZoomFactor/100);
			this.wHeight  = (int)(this.wHeight/mHeight)*mHeight;
			if ((wWidthOld/mWidth) == this.wWidth/mWidth) {
				this.wWidth  = ((int)this.wWidth/mWidth-1)*mWidth;
				this.wHeight = ((int)this.wHeight/mHeight-1)*mHeight;
			}
			mZoom += (int)(this.wWidth/mWidth)-(int)(wWidthOld/mWidth);
			if (this.debugModus) System.out.println("ZOOM IN: "+mZoom);		
			this.alles.setMOffsetX((getParent().getWidth() < this.wWidth) ? 0 : (int) (getParent().getWidth() - this.wWidth) / 2);			
			this.alles.setMOffsetY((getParent().getHeight() < this.wHeight) ? 0 : (int) (getParent().getHeight() - this.wHeight) / 2);
			generateMatrix(this.wWidth,this.wHeight);
			changeElements();
			repaint();
			revalidate();
		}
	}
	
	/**
	 * Change the size of the component to fit the size of the panel.
	 *
	 */
	public void changeZoomDefault() {
		calculateMatrixStartSize(getParent().getWidth(), getParent().getHeight(), this.mWidth, this.mHeight);
		calculateRatio(this.wHeight, this.wWidth, this.mHeight, this.mWidth);
		mZoom = 0;
		if (this.debugModus) System.out.println("ZOOM DEFAULT: "+mZoom);
		generateMatrix(this.wWidth,this.wHeight);
		changeElements();
		repaint();
		revalidate();
	}
	
	/**
	 * Save the current status with all its components and their properties.
	 *
	 */
	public void saveCurrentStatus() {
		if (this.membranModel.getAllProteins().size() > 0) {
			ProteinElement tmp;
			int[] coordinates;
			if (this.debugModus) System.out.println("PN:"+this.membranModel.getProteinNames().size());
			//remove all entries
			for (int i = 0; i < this.getComponents().length; i++) {
				tmp = (ProteinElement)this.getComponent(i);
				try{
				((Vector)(this.membranModel.getProtein(tmp.getKey()).getProteinPositions())).removeAllElements();
				} catch (Exception e) {
					;
				}
			}
			//add new protein entries and lipidCube entries
			Vector lipidCube = new Vector();
			for (int i = 0; i < this.getComponents().length; i++) {
				tmp = (ProteinElement)this.getComponent(i);
				coordinates = new int[3];
				coordinates[0] = tmp.getPX();
				coordinates[1] = tmp.getPY();
				coordinates[2] = tmp.getRotationAngle();
				try{
				(this.membranModel.getProtein(tmp.getKey())).addProteinPositions(coordinates);
				} catch (Exception e) {
					;
				}
				for (int j=0; j<tmp.getLipids2().size(); j++) {
					lipidCube.add(tmp.getLipids2().get(j)); //lipidCube entries
				}
			}
			//add new lipid entries
			if ((this.membranModel.getBiLayerType()) != null) {
				((this.membranModel.getBiLayerType()).getProteinPositions()).removeAllElements();
				for (int h=0; h<lipidCube.size(); h++) {
					(this.membranModel.getBiLayerType()).addProteinPositions((int[])lipidCube.get(h));
				}
				for (int j=0; j<this.mHeight; j++) {
					for (int i=0; i<this.mWidth; i++) {	
						coordinates = new int[2];
						coordinates[0] = i;
						coordinates[1] = j;
						if (this.busy.containsKey(i+"_"+j)) { 
							if (!this.busy.get(i+"_"+j).equals("1")) {
								(this.membranModel.getBiLayerType()).addProteinPositions(coordinates);
							}
						} else {
							(this.membranModel.getBiLayerType()).addProteinPositions(coordinates); 
						}
					}
				}		
				if (this.debugModus) {
					System.out.println("entries:\n");
					for (int i=0; i<(this.membranModel.getBiLayerType()).getProteinPositions().size(); i++) {
						System.out.println(this.membranModel.getBiLayerType().getProteinPositions().size());
						System.out.println(((int[])(this.membranModel.getBiLayerType().getProteinPositions().get(i)))[0]+" - "+((int[])(this.membranModel.getBiLayerType().getProteinPositions().get(i)))[1]);
					}
				}
			}
			//printout entries
			if (this.debugModus) {
				Vector testing = this.membranModel.getAllProteins();
				for (int i = 0; i < testing.size(); i++) {
					Protein test = (Protein)testing.get(i);
					for (int j = 0; j < test.getProteinPositions().size(); j++) {
						System.out.println("PP: "+((int[])test.getProteinPositions().get(j))[0]+" - "+((int[])test.getProteinPositions().get(j))[1]);
					}
				}
			}
		} else {
			if (this.debugModus) System.out.println("no proteins !");
		}

	}
	
	/**
	 * Change the key of all elements that belong to the old key.
	 * 
	 * @param oldKey String
	 * @param newKey String
	 */
	public void changeKey(String oldKey, String newKey) {
		for (int i=0; i<this.getComponents().length; i++) {
			ProteinElement tmp = (ProteinElement)this.getComponent(i);
			if (tmp.getKey().equals(oldKey)) {
				tmp.setKey(newKey);
			}
		}
	}
	
	/**
	 * Delete the key of an elements that had been deleted.
	 * 
	 * @param key String
	 */
	public void deleteKey(String key) {
		for (int i=0; i<this.getComponents().length; i++) {
			ProteinElement tmp = (ProteinElement)this.getComponent(i);
			if (tmp.getKey().equals(key)) {
				this.remove(i);
			}
		}
	}
	
	/**
	 * Calculates the start size of the lipid matrix.
	 * 
	 * @param panelWidth int (panel width)
	 * @param panelHeight int (panel height)
	 * @param matrixWidth int (lipid matrix width)
	 * @param matrixHeight int (lipid matrix height)
	 */
	public void calculateMatrixStartSize(int panelWidth, int panelHeight, int matrixWidth, int matrixHeight) {
		int[] matrixStartSize = new int[2];

		double panelRatio = (double) panelWidth / (double) panelHeight;
		double matrixRatio = (double) matrixWidth / (double) matrixHeight;

		if (panelRatio < 1) {
			if ((matrixRatio < 1) && (matrixRatio < panelRatio)) {
				// hhe
				matrixStartSize[1] = panelHeight;
				matrixStartSize[0] = (int) ((double) panelHeight * matrixRatio);
				// System.out.println("1");
			} else {
				// breite
				matrixStartSize[0] = panelWidth;
				matrixStartSize[1] = (int) ((double) panelWidth / matrixRatio);
				// System.out.println("2");
			}
		} else {
			if ((matrixRatio > 1) && (matrixRatio > panelRatio)) {
				// breite
				matrixStartSize[0] = panelWidth;
				matrixStartSize[1] = (int) ((double) panelWidth / matrixRatio);
				// System.out.println("3");
			} else {
				// hhe
				matrixStartSize[1] = panelHeight;
				matrixStartSize[0] = (int) ((double) panelHeight * matrixRatio);
				// System.out.println("4");
			}
		}
		if (this.debugModus) System.out.println("old W: " + panelWidth + " - old H: " + panelHeight);
		if (this.debugModus) System.out.println("new W: " + matrixStartSize[0] + " - new H: " + matrixStartSize[1]);
		matrixStartSize[0] = (matrixStartSize[0] / this.mWidth) * this.mWidth;
		matrixStartSize[1] = (matrixStartSize[1] / this.mHeight) * this.mHeight;
		this.wWidth  = matrixStartSize[0];
		this.wHeight = matrixStartSize[1];
		this.alles.setMOffsetX((int) (panelWidth - this.wWidth) / 2);
		this.alles.setMOffsetY((int) (panelHeight - this.wHeight) / 2);
		this.alles.setPWidth(matrixStartSize[0]);
		this.alles.setPHeight(matrixStartSize[1]);
		if (this.debugModus) System.out.println("set: "+this.alles.getPWidth());
	}	
	
	/**
	 * Generates new ProteinElement objects and add them to this panel.
	 *
	 */
	public void generateNew() {
		this.membranModel = MembranModelSingleton.getInstance();
		this.elementList  = this.membranModel.getAllProteins();
		if (testSpace()) {
			createProgBar();	
			new Thread( new Runnable() { public void run() { if (!createElements()) { alles.getGenerateNew().setEnabled(true); return; } } } ).start();			
		} else {
			JOptionPane.showMessageDialog(null,	"ERROR: could not calculate protein positions !\n (Not enough space: too many proteins.)");
			if (this.debugModus) System.out.println("Not enough space: too many proteins !!!");
		}
	}
	
	/**
	 * Generates the ProteinElements loaded with an XML file.
	 *
	 */
	public void generateLoaded() {
		this.membranModel = MembranModelSingleton.getInstance();
		this.elementList  = this.membranModel.getAllProteins();
		if (testSpace()) {
			createProgBar();	
			new Thread( new Runnable() { public void run() { if (!loadElements()) { alles.getGenerateNew().setEnabled(true); return; } } } ).start();			
		} else {
			JOptionPane.showMessageDialog(null,	"ERROR: could not calculate protein positions !\n (Not enough space: too many proteins.)");
			if (this.debugModus) System.out.println("Not enough space: too many proteins !!!");
		}
	}
	
	/**
	 * Calculates the number of Proteins.
	 * 
	 * @return int (number of proteins)
	 */
	public int calculateProteinNumber() {
		int number = 0;
		Protein protein;
		for (int i=0; i<this.elementList.size(); i++) {
			protein = (Protein)this.elementList.get(i);
			number += calculateQuantityIntValue(protein.getQuantity(), (protein.getSizeByLipids()[0]), (int)(protein.getSizeByLipids()[1]));		
		}
		return number; 
	}
		
	/**
	 * Creates the new ProteinElements components.
	 * 	 
	 * @return boolean (true, if no errors occur)
	 */
	public boolean createElements() {
		this.alles.getGenerateNew().setEnabled(false);
		ProteinElement proteinElement;
		Protein protein;
		int[] tmpArray = new int[2];
		tmpArray[0] = 0;
		for (int i=0; i<this.elementList.size(); i++) {
			protein = (Protein)this.elementList.get(i);
			int tmpSizeX = protein.getSizeByLipids()[0];
			int tmpSizeY = protein.getSizeByLipids()[1];
			int tmpQ = calculateQuantityIntValue(protein.getQuantity(), tmpSizeX, tmpSizeY);
			for (int j=0; j<tmpQ; j++) {
				tmpArray = generateCoordinates(tmpSizeX, tmpSizeY);
				if (tmpArray[0] == -1) {
					JOptionPane.showMessageDialog(null,	"WARNING:\n" +
														"Could not find a position for the current protein element in the matrix !\n\n" + 
														"Note:\n" +
														"The current generating process will be canceled and the remaining proteins are still missing!\n" +
														"Please try again (using \"Generate new\" button).");
					deleteProgBar();
					this.revalidate();
					return false;
				} else {
					proteinElement = new ProteinElement(this.alles, 
													this,
													protein.getPDBcode(),
													protein.getPDBcode()+"_"+protein.getSubcomponent(),
													protein.getComponent(),
													protein.getSubcomponent(),
													tmpArray[0]*this.ratioWidth+this.alles.getMOffsetX(),
													tmpArray[1]*this.ratioHeight+this.alles.getMOffsetY(),
													this.ratioWidth*(tmpSizeX)+1,
													this.ratioHeight*(tmpSizeY)+1,
													protein.getProteinColor(),
													tmpArray[0],
													tmpArray[1],
													tmpSizeX,
													tmpSizeY,
													this.ratioWidth,
													this.alles.getMOffsetX(),
													this.alles.getMOffsetY(),
													0,
													protein.lipidFillingInCube());		
					this.add(proteinElement);
					this.repaint();				
					this.alles.addProteinToVector(proteinElement);
					this.progbar.getBar().setValue(this.progbar.getBar().getValue()+1);
				}
			}
		}
		printCords();
		deleteProgBar();
		this.revalidate();	
		this.alles.getGenerateNew().setEnabled(true);
		return true;
	}
	
	/**
	 * Creates the new ProteinElements components when an XML file is loaded.
	 * 	 
	 * @return boolean (true, if no errors occur)
	 */
	public boolean loadElements() {
		this.alles.getGenerateNew().setEnabled(false);
		this.removeAll();
		this.elementList = this.membranModel.getAllProteins();
		ProteinElement proteinElement;
		Protein protein;
		int[] tmpArray = new int[3];
		for (int i=0; i<this.elementList.size(); i++) {
			protein = (Protein)this.elementList.get(i);
			int tmpSizeX = protein.getSizeByLipids()[0];
			int tmpSizeY = protein.getSizeByLipids()[1];
			Vector positions = protein.getProteinPositions();
			for (int j=0; j<positions.size(); j++) {
				tmpArray = (int[])positions.get(j);
				if (checkRotate(((int[])positions.get(j))[2])) {
					proteinElement = new ProteinElement(this.alles, 
														this,
														protein.getPDBcode(),
														protein.getPDBcode()+"_"+protein.getSubcomponent(),
														protein.getComponent(),
														protein.getSubcomponent(),
														tmpArray[0]*this.ratioWidth+this.alles.getMOffsetX(),
														tmpArray[1]*this.ratioHeight+this.alles.getMOffsetY(),
														this.ratioWidth*(tmpSizeY)+1,
														this.ratioHeight*(tmpSizeX)+1,
														protein.getProteinColor(),
														tmpArray[0],
														tmpArray[1],
														tmpSizeY,
														tmpSizeX,
														this.ratioWidth,
														this.alles.getMOffsetX(),
														this.alles.getMOffsetY(),
														tmpArray[2],
														protein.lipidFillingInCube());	
				} else {
					proteinElement = new ProteinElement(this.alles, 
														this,
														protein.getPDBcode(),
														protein.getPDBcode()+"_"+protein.getSubcomponent(),
														protein.getComponent(),
														protein.getSubcomponent(),
														tmpArray[0]*this.ratioWidth+this.alles.getMOffsetX(),
														tmpArray[1]*this.ratioHeight+this.alles.getMOffsetY(),
														this.ratioWidth*(tmpSizeX)+1,
														this.ratioHeight*(tmpSizeY)+1,
														protein.getProteinColor(),
														tmpArray[0],
														tmpArray[1],
														tmpSizeX,
														tmpSizeY,
														this.ratioWidth,
														this.alles.getMOffsetX(),
														this.alles.getMOffsetY(),
														tmpArray[2],
														protein.lipidFillingInCube());	
				}
					this.add(proteinElement);
					this.repaint();				
					this.alles.addProteinToVector(proteinElement);
					this.progbar.getBar().setValue(this.progbar.getBar().getValue()+1);
			}
		}
		printCords();
		deleteProgBar();
		this.revalidate();	
		this.alles.getGenerateNew().setEnabled(true);
		return true;
	}
	
	/**
	 * Checks if the ProteinElement is rotated with 90 or 270.
	 * 
	 * @param angle int
	 * @return boolean
	 */
	public boolean checkRotate(int angle) {
		if ((angle == 90) || (angle == 270)) {
			return true;
		}		
		return false;
	}
	
	/**
	 * Calculates the quantity value of a ProteinElement in the lipid matrix. 
	 * 
	 * @param quantity int
	 * @param sizeX int (width of the protein in lipids)
	 * @param sizeY int (height of the protein in lipids)
	 * 
	 * @return int
	 */
	public int calculateQuantityIntValue(int quantity, int sizeX, int sizeY) {
		int quantityInt;
		quantityInt = (int)(this.mHeight * this.mWidth * quantity / 100 / sizeX / sizeY);	
		//System.out.println(quantity+" - "+quantityInt);
		return quantityInt;
	}
	
	/**
	 * Creates and initializes a MatrixProgressBar.
	 *
	 */
	public void createProgBar() {		
		progbar = new MatrixProgressBar(this.alles, this, calculateProteinNumber());
		this.add(progbar);
		this.repaint();
		this.revalidate();
	}
	
	/**
	 * Deletes a MatrixProgressBar.
	 *
	 */
	public void deleteProgBar() {
		this.remove(progbar);
		this.repaint();
		this.revalidate();
	}
	
	/**
	 * Paints the component.
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);		
		
		//Graphics2D g2 = (Graphics2D) g;
	    //g2.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
	     //                    RenderingHints.VALUE_ANTIALIAS_ON);
	   
		for (int i=0; i<this.mWidth; i++) {
			for (int j=0; j<this.mHeight; j++) {
				g.drawOval(this.alles.getMOffsetX()+this.ratioWidth*i,
						this.alles.getMOffsetY()+this.ratioHeight*j,
					       this.ratioWidth,
					       this.ratioHeight);
			}
		}
	}	

	//######################################################################################
	//##                                   MATRIX STUFF                                   ##
	//######################################################################################
	
	/**
	 * Checks if the proteins could be placed all on the lipid matrix.
	 * 
	 * @return boolean (true, if the proteins use less than 70% of the matrix)
	 */
	public boolean testSpace() {
		int matrixSize = this.mWidth * this.mHeight;
		matrixSize = (int)(matrixSize * 0.7); // 30% free room for empty space
		for (int i=0; i<this.elementList.size(); i++) {
			if (this.debugModus) System.out.println(((Protein)this.elementList.get(i)).getSizeByLipids()[0]);
			if (this.debugModus) System.out.println(((Protein)this.elementList.get(i)).getSizeByLipids()[1]);
			if (this.debugModus) System.out.println(((Protein)this.elementList.get(i)).getQuantity());
			matrixSize -= ((Protein)this.elementList.get(i)).getSizeByLipids()[0]*
						  ((Protein)this.elementList.get(i)).getSizeByLipids()[1]*
						  calculateQuantityIntValue(((Protein)this.elementList.get(i)).getQuantity(), (((Protein)this.elementList.get(i)).getSizeByLipids()[0]), (int)(((Protein)this.elementList.get(i)).getSizeByLipids()[1]));
			if (matrixSize < 0) {
				return false;
			}
		}
		int percentFree = (int)((matrixSize+this.mWidth * this.mHeight * 0.3) / (this.mWidth * this.mHeight) * 100);
		if (this.debugModus) System.out.println("Empty Matrix Space: "+(int)(matrixSize+this.mWidth*this.mHeight*0.3)+" fields = "+percentFree+"%");
		return true;
	}
	
	/**
	 * Generate the coordinates for the ProteinElement on the lipid matrix.
	 * 
	 * @param sizeX int (width of the protein in lipids)
	 * @param sizeY int (height of the protein in lipids)
	 * 
	 * @return int[]
	 */
	public int[] generateCoordinates(int sizeX, int sizeY) {
		int[] cords = new int[2];
		int width   = this.mWidth - sizeX;
		int height  = this.mHeight - sizeY;
		int counter = this.maxTry;
		Random r    = new Random();
		do {
			cords[0] = r.nextInt(width);
			cords[1] = r.nextInt(height);
			if (counter-- < 0) {
				if (this.debugModus) System.out.println("ERROR: could not find position for protein element in matrix !");				
				cords[0] = -1;
				cords[1] = -1;
				return cords;
			}
		} while (testCords(cords[0], cords[1], sizeX, sizeY));		
		setBusyCords(cords[0], cords[1], sizeX, sizeY);		
		return cords;
	}
	
	/**
	 * Set the busy coordinates in the lipid matrix.
	 * 
	 * @param x int (x position in lipid matrix)
	 * @param y int (y position in lipid matrix)
	 * @param sizeX int (width of the protein in lipids)
	 * @param sizeY int (height of the protein in lipids)
	 */
	public void setBusyCords(int x, int y, int sizeX, int sizeY) {
		for (int i=0; i<sizeX; i++) {
			for (int j=0; j<sizeY; j++) {
				this.busy.put((x+i)+"_"+(y+j), "1");
				if (this.debugModus) System.out.print((x+i)+"_"+(y+j));
			}
		}		
	}
	
	/**
	 * Removes the busy coordinates in the lipid matrix.
	 * 
	 * @param x int (x position in lipid matrix)
	 * @param y int (y position in lipid matrix)
	 * @param sizeX int (width of the protein in lipids)
	 * @param sizeY int (height of the protein in lipids)
	 */
	public void removeBusyCords(int x, int y, int sizeX, int sizeY) {
		for (int i=0; i<sizeX; i++) {
			for (int j=0; j<sizeY; j++) {
				this.busy.put((x+i)+"_"+(y+j), "0");
				if (this.debugModus) System.out.print((x+i)+"_"+(y+j));
			}
		}		
	}
	
	/**
	 *  Changes the busy coordinates in the lipid matrix.
	 * 
	 * @param x1 int (old x position in lipid matrix)
	 * @param y1 int (old y position in lipid matrix)
	 * @param x2 int (new x position in lipid matrix)
	 * @param y2 int (new y position in lipid matrix)
	 * @param sizeX int (width of the protein in lipids)
	 * @param sizeY int (height of the protein in lipids)
	 */
	public void changeBusyCords(int x1, int y1, int x2, int y2, int sizeX, int sizeY) {
		removeBusyCords(x1, y1, sizeX, sizeY);
		setBusyCords(x2, y2, sizeX, sizeY);
	}
	
	/**
	 * Prints the lipid matrix to the STDOUT.
	 *
	 */
	public void printCords() {
		if (this.debugModus) System.out.println("");
		for (int j=0; j<this.mHeight; j++) {
			for (int i=0; i<this.mWidth; i++) {			
				if (this.debugModus) System.out.print(this.busy.containsKey(i+"_"+j) ? ((this.busy.get(i+"_"+j).equals("1")) ? "# " : "+ ") : "- "); 
			}
			if (this.debugModus) System.out.println("");
		}
	}
	
	/**
	 * Test if the protein do not overlap with the busy coordinates in the lipid matrix.
	 * 
	 * @param x int (x position in lipid matrix)
	 * @param y int (y position in lipid matrix)
	 * @param sizeX int (width of the protein in lipids)
	 * @param sizeY int (height of the protein in lipids)
	 * 
	 * @return boolean
	 */
	public boolean testCords(int x, int y, int sizeX, int sizeY) {
		for (int i=0; i<sizeX; i++) {
			for (int j=0; j<sizeY; j++) {
				if (this.busy.containsKey((x+i)+"_"+(y+j))) {
					if (this.busy.get((x+i)+"_"+(y+j)).equals("1")) { 
						if (this.debugModus) System.out.println("WARNING: overlap");
					return true;
					}
				}				
			}
		}		
		return false; 
	}
	
	/**
	 * Clear the busy coordinates in the lipid matrix.
	 *
	 */
	public void clearCords() {
		this.busy.clear();
	}
	
	/**
	 * Calculates the ratio of a singe lipid according to the panel size. 
	 * 
	 * @param wH int (window height)
	 * @param wW int (window width)
	 * @param mH int (matrix height)
	 * @param mW int (matrix width)
	 */
	public void calculateRatio(int wH, int wW, int mH, int mW) {		
		this.ratioWidth  = (int)(wW / mW);
		this.ratioHeight = this.ratioWidth;
		if (this.debugModus) System.out.println("ratio: w "+this.ratioWidth+"px - h "+this.ratioHeight+"px");
	}
	
	//######################################################################################
	//##                              GETTER AND SETTER                                   ##
	//######################################################################################

	/**
	 *  @return Returns the matrix height.
	 */
	public int getMatrixHeight() {
		return (this.mHeight * this.ratioHeight);
	}
	
	/**
	 *  @return Returns the matrix width.
	 */
	public int getMatrixWidth() {
		return (this.mWidth * this.ratioWidth);
	}

	/**
	 * Adds a ProteinElement.
	 * 
	 * @param newElement ProteinElement
	 */
	public void addElement(ProteinElement newElement) {
		this.elementList.addElement(newElement);
	}
	
	/**
	 * Returns the element at index i.
	 * 
	 * @param i int
	 * @return String[]
	 */
	public String[] getElement(int i) {
		return (String[])this.elementList.get(i);
	}
	
	/**
	 * @return Returns the ratioWidth.
	 */
	public int getRatioWidth() {
		return ratioWidth;
	}	
	
	/**
	 * @return Returns the alles.
	 */
	public Msg_panel getAlles() {
		return alles;
	}

	/**
	 * @param alles The alles to set.
	 */
	public void setAlles(Msg_panel alles) {
		this.alles = alles;
	}

	/**
	 * @return Returns the busy.
	 */
	public Hashtable getBusy() {
		return busy;
	}

	/**
	 * @param busy The busy to set.
	 */
	public void setBusy(Hashtable busy) {
		this.busy = busy;
	}

	/**
	 * @return Returns the colorBackground.
	 */
	public Color getColorBackground() {
		return colorBackground;
	}

	/**
	 * @param colorBackground The colorBackground to set.
	 */
	public void setColorBackground(Color colorBackground) {
		this.colorBackground = colorBackground;
	}

	/**
	 * @return Returns the debugModus.
	 */
	public boolean isDebugModus() {
		return debugModus;
	}

	/**
	 * @param debugModus The debugModus to set.
	 */
	public void setDebugModus(boolean debugModus) {
		this.debugModus = debugModus;
	}

	/**
	 * @return Returns the elementList.
	 */
	public Vector getElementList() {
		return elementList;
	}

	/**
	 * @param elementList The elementList to set.
	 */
	public void setElementList(Vector elementList) {
		this.elementList = elementList;
	}

	/**
	 * @return Returns the maxTry.
	 */
	public int getMaxTry() {
		return maxTry;
	}

	/**
	 * @param maxTry The maxTry to set.
	 */
	public void setMaxTry(int maxTry) {
		this.maxTry = maxTry;
	}

	/**
	 * @return Returns the membranModel.
	 */
	public MembranModel getMembranModel() {
		return membranModel;
	}

	/**
	 * @param membranModel The membranModel to set.
	 */
	public void setMembranModel(MembranModel membranModel) {
		this.membranModel = membranModel;
	}

	/**
	 * @return Returns the mHeight.
	 */
	public int getMHeight() {
		return mHeight;
	}

	/**
	 * @param height The mHeight to set.
	 */
	public void setMHeight(int height) {
		mHeight = height;
	}

	/**
	 * @return Returns the mWidth.
	 */
	public int getMWidth() {
		return mWidth;
	}

	/**
	 * @param width The mWidth to set.
	 */
	public void setMWidth(int width) {
		mWidth = width;
	}

	/**
	 * @return Returns the mZoom.
	 */
	public int getMZoom() {
		return mZoom;
	}

	/**
	 * @param zoom The mZoom to set.
	 */
	public void setMZoom(int zoom) {
		mZoom = zoom;
	}

	/**
	 * @return Returns the mZoomFactor.
	 */
	public int getMZoomFactor() {
		return mZoomFactor;
	}

	/**
	 * @param zoomFactor The mZoomFactor to set.
	 */
	public void setMZoomFactor(int zoomFactor) {
		mZoomFactor = zoomFactor;
	}

	/**
	 * @return Returns the progbar.
	 */
	public MatrixProgressBar getProgbar() {
		return progbar;
	}

	/**
	 * @param progbar The progbar to set.
	 */
	public void setProgbar(MatrixProgressBar progbar) {
		this.progbar = progbar;
	}

	/**
	 * @return Returns the ratioHeight.
	 */
	public int getRatioHeight() {
		return ratioHeight;
	}

	/**
	 * @param ratioHeight The ratioHeight to set.
	 */
	public void setRatioHeight(int ratioHeight) {
		this.ratioHeight = ratioHeight;
	}	

	/**
	 * @param ratioWidth The ratioWidth to set.
	 */
	public void setRatioWidth(int ratioWidth) {
		this.ratioWidth = ratioWidth;
	}

	/**
	 * @return Returns the resizeValue.
	 */
	public int getResizeValue() {
		return resizeValue;
	}

	/**
	 * @param resizeValue The resizeValue to set.
	 */
	public void setResizeValue(int resizeValue) {
		this.resizeValue = resizeValue;
	}

	/**
	 * @return Returns the wHeight.
	 */
	public int getWHeight() {
		return wHeight;
	}

	/**
	 * @param height The wHeight to set.
	 */
	public void setWHeight(int height) {
		wHeight = height;
	}

	/**
	 * @return Returns the wWidth.
	 */
	public int getWWidth() {
		return wWidth;
	}

	/**
	 * @param width The wWidth to set.
	 */
	public void setWWidth(int width) {
		wWidth = width;
	}
}