/* $RCSfile: FrameRenderer.java,v $
 * $Author: jan $
 * $Date: 2006/03/16 11:20:22 $
 * $Revision: 1.8 $
 *
 * Copyright (C) 2003-2005  The Jmol Development Team
 *
 * Contact: jmol-developers@lists.sf.net
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package org.jmol.viewer;

import org.jmol.g3d.*;

import javax.swing.JPanel;
import javax.vecmath.Point3i;

import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Toolkit;

import java.awt.FontMetrics;

public class FrameRenderer {

	Viewer viewer;
	JPanel frame;

	//////////////////////// Transparent Lipids ///////////////////////////////
	boolean showTransparentLipids;
	Image image = Toolkit.getDefaultToolkit().getImage("images/lipide__1_1.gif");
	// Only create once to give a speed boost
	int thisH;
	int thisW;
	int picH;
	int picW;
	int picM;
	int quantity;
	int pixelResidue;

	ShapeRenderer[] renderers = new ShapeRenderer[JmolConstants.SHAPE_MAX];

	// new: public
	public FrameRenderer(Viewer viewer) {
		this.viewer = viewer;
	}

	public FrameRenderer(Viewer viewer, JPanel frame) {
		this.viewer = viewer;
		this.frame = frame;
		this.showTransparentLipids = true;
	}
	
	/**
	 * This method paints the tranparent lipids and calculates
	 * the size of the painted images.
	 * 
	 * @param g3d
	 */
	void paintLipids(Graphics3D g3d) {
		
		thisH = frame.getHeight();
		thisW = frame.getWidth();
		picH = thisH / 5;
		picW = picH / 4;
		picM = (thisH / 2) - (picH / 2);
		quantity = thisW / picW;
		pixelResidue = thisW % picW;

		// We use Scale_FAST, to give a good speed performance
		Image scaledInstance = image.getScaledInstance(picW, picH,Image.SCALE_FAST);

		// lipidimage start point
		int tmp = 0;
		System.out.println("Erster PaintPunkt: "+tmp+"\nBilderQuantity: "+quantity);
		for (int i = 0; i < quantity; i++) {
			g3d.drawImage(scaledInstance, tmp, picM, 0);
			if (pixelResidue > 0) {
				tmp += (picW + 1);
				pixelResidue--;
			} else {
				tmp += picW;
			}
		}
	}

	void render(Graphics3D g3d, Rectangle rectClip, Frame frame,
			int displayModelIndex) {

		if (frame == null || frame.atomCount <= 0)
			return;

		viewer.calcTransformMatrices();

		for (int i = 0; i < JmolConstants.SHAPE_MAX; ++i) {
			Shape shape = frame.shapes[i];
			if (shape == null)
				continue;
			getRenderer(i, g3d).render(g3d, rectClip, frame, displayModelIndex,
					shape);

		}
		if (showTransparentLipids == true) {
			//paintLipids(g3d);
		}
	}

	ShapeRenderer getRenderer(int refShape, Graphics3D g3d) {
		if (renderers[refShape] == null)
			renderers[refShape] = allocateRenderer(refShape, g3d);
		return renderers[refShape];
	}

	ShapeRenderer allocateRenderer(int refShape, Graphics3D g3d) {
		String classBase = JmolConstants.shapeClassBases[refShape] + "Renderer";
		String className = "org.jmol.viewer." + classBase;

		try {
			Class shapeClass = Class.forName(className);
			ShapeRenderer renderer = (ShapeRenderer) shapeClass.newInstance();
			renderer.setViewerFrameRenderer(viewer, this, g3d);
			return renderer;
		} catch (Exception e) {
			System.out.println("Could not instantiate renderer:" + classBase
					+ "\n" + e);
			e.printStackTrace();
		}
		return null;
	}

	void renderStringOutside(String str, short colix, Font3D font3d,
			Point3i screen, Graphics3D g3d) {
		renderStringOutside(str, colix, font3d, screen.x, screen.y, screen.z,
				g3d);
	}

	void renderStringOutside(String str, short colix, Font3D font3d, int x,
			int y, int z, Graphics3D g3d) {
		FontMetrics fontMetrics = font3d.fontMetrics;
		int strAscent = fontMetrics.getAscent();
		int strWidth = fontMetrics.stringWidth(str);
		int xStrCenter, yStrCenter;
		int xCenter = viewer.getBoundBoxCenterX();
		int yCenter = viewer.getBoundBoxCenterY();
		int dx = x - xCenter;
		int dy = y - yCenter;
		if (dx == 0 && dy == 0) {
			xStrCenter = x;
			yStrCenter = y;
		} else {
			int dist = (int) Math.sqrt(dx * dx + dy * dy);
			xStrCenter = xCenter
					+ ((dist + 2 + (strWidth + 1) / 2) * dx / dist);
			yStrCenter = yCenter
					+ ((dist + 3 + (strAscent + 1) / 2) * dy / dist);
		}
		int xStrBaseline = xStrCenter - strWidth / 2;
		int yStrBaseline = yStrCenter + strAscent / 2;
		g3d.drawString(str, font3d, colix, xStrBaseline, yStrBaseline, z);
	}

	public boolean getShowTransparentLipids() {
		return showTransparentLipids;
	}

	public void setShowTransparentLipids(boolean showLipids) {
		showTransparentLipids = showLipids;
	}

}
