/* $RCSfile: PreferencesDialog.java,v $
 * $Author: jan $
 * $Date: 2006/02/16 10:53:25 $
 * $Revision: 1.1 $
 *
 * Copyright (C) 2002-2005  The Jmol Development Team
 *
 * Contact: jmol-developers@lists.sf.net
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 */
package org.openscience.jmol.app;

import org.jmol.api.*;
import org.jmol.i18n.GT;
import java.awt.GridLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Properties;
import java.util.Hashtable;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import javax.swing.JRadioButton;
import javax.swing.BoxLayout;
import javax.swing.JSlider;
import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JCheckBox;
import javax.swing.Box;
import javax.swing.JTabbedPane;
import javax.swing.ButtonGroup;
import javax.swing.JColorChooser;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.Action;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.border.TitledBorder;

public class PreferencesDialog extends JDialog implements ActionListener {

  private boolean autoBond;
  boolean showHydrogens;
  //private boolean showVectors;
  boolean showMeasurements;
  boolean wireframeRotation;
  boolean perspectiveDepth;
  boolean showAxes;
  boolean showBoundingBox;
  boolean axesOrientationRasmol;
  boolean openFilePreview;
  boolean isLabelAtomColor;
  boolean isBondAtomColor;
  Color colorBackground;
  Color colorSelection;
  Color colorText;
  Color colorBond;
  Color colorVector;
  Color colorMeasurement;
  //private byte modeAtomColorProfile;
  float minBondDistance;
  float bondTolerance;
  short marBond;
  int percentVdwAtom;
  //  private double VibrateAmplitudeScale;
  //  private double VibrateVectorScale;
  //  private int VibrationFrames;
  JButton bButton, pButton, tButton, eButton, vButton;
  JButton measurementColorButton;
  private JRadioButton /*pYes, pNo, */abYes, abNo;
  //private JComboBox aProps, cRender;
  private JSlider vdwPercentSlider;
  private JSlider bdSlider, bwSlider, btSlider;
  //private JSlider vasSlider;
  //private JSlider vvsSlider;
  //private JSlider vfSlider;
  private JCheckBox cH, cM;
  private JCheckBox cbWireframeRotation, cbPerspectiveDepth;
  private JCheckBox cbShowAxes, cbShowBoundingBox;
  private JCheckBox cbAxesOrientationRasmol;
  private JCheckBox cbOpenFilePreview;
  private JCheckBox cbIsLabelAtomColor, cbIsBondAtomColor;
  private Properties originalSystemProperties;
  private Properties jmolDefaultProperties;
  Properties currentProperties;

  // The actions:

  private PrefsAction prefsAction = new PrefsAction();
  private Hashtable commands;

  final static String[] jmolDefaults  = {
    "showHydrogens",                  "true",
    "showVectors",                    "true",
    "showMeasurements",               "true",
    "wireframeRotation",              "false",
    "perspectiveDepth",               "true",
    "showAxes",                       "false",
    "showBoundingBox",                "false",
    "axesOrientationRasmol",          "false",
	"openFilePreview",                "true",
    "percentVdwAtom",                 "20",
    "autoBond",                       "true",
    "marBond",                        "150",
    "minBondDistance",                "0.40",
    "bondTolerance",                  "0.45",
    "colorSelection",                 "16762880",
    "colorBackground",                "0",
    "isLabelAtomColor",               "false",
    "colorVector",                    "16777215",
    "isBondAtomColor",                "true",
    "colorBond",                      "0",
    "colorVector",                    "16777215",
    "colorMeasurement",               "16777215",
    //    "VibrateAmplitudeScale",          "0.7",
    //    "VibrateVectorScale",             "1.0",
    //    "VibrationFrames",                "20",
  };

  final static String[] rasmolOverrides = {
    "colorBackground",                "0",
    "isLabelAtomColor",               "true",
    "percentVdwAtom",                 "0",
    "marBond",                        "1",
    "axesOrientationRasmol",          "true",
  };

  JmolViewer viewer;
  GuiMap guimap;

  public PreferencesDialog(JFrame f, GuiMap guimap,
                           JmolViewer viewer) {

    super(f, false);
    this.guimap = guimap;
    this.viewer = viewer;

    initializeProperties();

    this.setTitle(GT._("Preferences"));

    initVariables();
    commands = new Hashtable();
    Action[] actions = getActions();
    for (int i = 0; i < actions.length; i++) {
      Action a = actions[i];
      commands.put(a.getValue(Action.NAME), a);
    }
    JPanel container = new JPanel();
    container.setLayout(new BorderLayout());

    JTabbedPane tabs = new JTabbedPane();
    JPanel disp = buildDispPanel();
    JPanel atoms = buildAtomsPanel();
    JPanel bonds = buildBondPanel();
    JPanel colors = buildColorsPanel();
    //    JPanel vibrate = buildVibratePanel();
    tabs.addTab(GT._("Display"), null, disp);
    tabs.addTab(GT._("Atoms"), null, atoms);
    tabs.addTab(GT._("Bonds"), null, bonds);
    tabs.addTab(GT._("Colors"), null, colors);
    //    tabs.addTab(GT._("Vibrate"), null, vibrate);

    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

    jmolDefaultsButton = new JButton(GT._("Jmol Defaults"));
    jmolDefaultsButton.addActionListener(this);
    buttonPanel.add(jmolDefaultsButton);

    rasmolDefaultsButton = new JButton(GT._("RasMol Defaults"));
    rasmolDefaultsButton.addActionListener(this);
    buttonPanel.add(rasmolDefaultsButton);

    cancelButton = new JButton(GT._("Cancel"));
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);

    applyButton = new JButton(GT._("Apply"));
    applyButton.addActionListener(this);
    buttonPanel.add(applyButton);

    okButton = new JButton(GT._("OK"));
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    getRootPane().setDefaultButton(okButton);

    container.add(tabs, BorderLayout.CENTER);
    container.add(buttonPanel, BorderLayout.SOUTH);
    getContentPane().add(container);

    updateComponents();

    pack();
    centerDialog();
  }

  public JPanel buildDispPanel() {

    JPanel disp = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    disp.setLayout(gridbag);
    GridBagConstraints constraints;

    JPanel showPanel = new JPanel();
    showPanel.setLayout(new GridLayout(1, 3));
    showPanel.setBorder(new TitledBorder(GT._("Show All")));
    cH = guimap.newJCheckBox("Prefs.showHydrogens",
                             viewer.getShowHydrogens());
    cH.addItemListener(checkBoxListener);
    cM = guimap.newJCheckBox("Prefs.showMeasurements",
                             viewer.getShowMeasurements());
    cM.addItemListener(checkBoxListener);
    showPanel.add(cH);
    showPanel.add(cM);

    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    disp.add(showPanel, constraints);

    JPanel fooPanel = new JPanel();
    fooPanel.setBorder(new TitledBorder(""));
    fooPanel.setLayout(new GridLayout(2, 1));

    cbWireframeRotation =
      guimap.newJCheckBox("Prefs.wireframeRotation",
                          viewer.getWireframeRotation());
    cbWireframeRotation.addItemListener(checkBoxListener);
    fooPanel.add(cbWireframeRotation);

    cbPerspectiveDepth =
      guimap.newJCheckBox("Prefs.perspectiveDepth",
                          viewer.getPerspectiveDepth());
    cbPerspectiveDepth.addItemListener(checkBoxListener);
    fooPanel.add(cbPerspectiveDepth);

    cbShowAxes =
      guimap.newJCheckBox("Prefs.showAxes", viewer.getShowAxes());
    cbShowAxes.addItemListener(checkBoxListener);
    fooPanel.add(cbShowAxes);

    cbShowBoundingBox =
      guimap.newJCheckBox("Prefs.showBoundingBox", viewer.getShowBbcage());
    cbShowBoundingBox.addItemListener(checkBoxListener);
    fooPanel.add(cbShowBoundingBox);

    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    disp.add(fooPanel, constraints);

    JPanel axesPanel = new JPanel();
    axesPanel.setBorder(new TitledBorder(""));
    axesPanel.setLayout(new GridLayout(1, 1));

    cbAxesOrientationRasmol =
        guimap.newJCheckBox("Prefs.axesOrientationRasmol",
                            viewer.getAxesOrientationRasmol());
    cbAxesOrientationRasmol.addItemListener(checkBoxListener);
    axesPanel.add(cbAxesOrientationRasmol);

    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    disp.add(axesPanel, constraints);

    JPanel otherPanel = new JPanel();
    otherPanel.setBorder(new TitledBorder(""));
    otherPanel.setLayout(new GridLayout(1, 1));

    cbOpenFilePreview =
        guimap.newJCheckBox("Prefs.openFilePreview",
                            openFilePreview);
    cbOpenFilePreview.addItemListener(checkBoxListener);
    otherPanel.add(cbOpenFilePreview);
    
    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    disp.add(otherPanel, constraints);


    JLabel filler = new JLabel();
    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.gridheight = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    disp.add(filler, constraints);

    return disp;
  }

  public JPanel buildAtomsPanel() {

    JPanel atomPanel = new JPanel(new GridBagLayout());
    GridBagConstraints constraints;

    JPanel sfPanel = new JPanel();
    sfPanel.setLayout(new BorderLayout());
    sfPanel.setBorder(new TitledBorder(GT._("Default atom size")));
    JLabel sfLabel = new JLabel(GT._("(percentage of vanDerWaals radius)"),
                                SwingConstants.CENTER);
    sfPanel.add(sfLabel, BorderLayout.NORTH);
    vdwPercentSlider =
      new JSlider(SwingConstants.HORIZONTAL, 0, 100, viewer.getPercentVdwAtom());
    vdwPercentSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    vdwPercentSlider.setPaintTicks(true);
    vdwPercentSlider.setMajorTickSpacing(20);
    vdwPercentSlider.setMinorTickSpacing(10);
    vdwPercentSlider.setPaintLabels(true);
    vdwPercentSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        percentVdwAtom = source.getValue();
        viewer.setPercentVdwAtom(percentVdwAtom);
        currentProperties.put("percentVdwAtom", "" + percentVdwAtom);
      }
    });
    sfPanel.add(vdwPercentSlider, BorderLayout.CENTER);
    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.HORIZONTAL;
    constraints.weightx = 1.0;
    atomPanel.add(sfPanel, constraints);


    JLabel filler = new JLabel();
    constraints = new GridBagConstraints();
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    constraints.gridheight = GridBagConstraints.REMAINDER;
    constraints.fill = GridBagConstraints.BOTH;
    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    atomPanel.add(filler, constraints);

    return atomPanel;
  }

  public JPanel buildBondPanel() {

    JPanel bondPanel = new JPanel();
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    bondPanel.setLayout(gridbag);
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 1.0;
    c.weighty = 1.0;

    JPanel autobondPanel = new JPanel();
    autobondPanel.setLayout(new BoxLayout(autobondPanel, BoxLayout.Y_AXIS));
    autobondPanel.setBorder(new TitledBorder(GT._("Compute Bonds")));
    ButtonGroup abGroup = new ButtonGroup();
    abYes = new JRadioButton(GT._("Automatically"));
    abNo = new JRadioButton(GT._("Don't Compute Bonds"));
    abGroup.add(abYes);
    abGroup.add(abNo);
    autobondPanel.add(abYes);
    autobondPanel.add(abNo);
    autobondPanel.add(Box.createVerticalGlue());
    abYes.setSelected(viewer.getAutoBond());
    abYes.addActionListener(new ActionListener() {

      public void actionPerformed(ActionEvent e) {
        viewer.setAutoBond(true);
      }
    });

    abNo.addActionListener(new ActionListener() {

      public void actionPerformed(ActionEvent e) {
        viewer.setAutoBond(false);
      }
    });

    c.gridwidth = GridBagConstraints.REMAINDER;
    gridbag.setConstraints(autobondPanel, c);
    bondPanel.add(autobondPanel);

    JPanel bwPanel = new JPanel();
    bwPanel.setLayout(new BorderLayout());
    bwPanel.setBorder(new TitledBorder(GT._("Default Bond Radius")));
    JLabel bwLabel = new JLabel(GT._("(Angstroms)"), SwingConstants.CENTER);
    bwPanel.add(bwLabel, BorderLayout.NORTH);

    bwSlider = new JSlider(0, 250,viewer.getMadBond()/2);
    bwSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    bwSlider.setPaintTicks(true);
    bwSlider.setMajorTickSpacing(50);
    bwSlider.setMinorTickSpacing(25);
    bwSlider.setPaintLabels(true);
    for (int i = 0; i <= 250; i += 50) {
      String label = "" + (1000 + i);
      label = "0." + label.substring(1);
      bwSlider.getLabelTable().put(new Integer(i),
                                   new JLabel(label, SwingConstants.CENTER));
      bwSlider.setLabelTable(bwSlider.getLabelTable());
    }
    bwSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        marBond = (short)source.getValue();
        viewer.setMarBond(marBond);
        currentProperties.put("marBond", "" + marBond);
      }
    });

    bwPanel.add(bwSlider, BorderLayout.SOUTH);

    c.weightx = 0.0;
    gridbag.setConstraints(bwPanel, c);
    bondPanel.add(bwPanel);

    // Bond Tolerance Slider
    JPanel btPanel = new JPanel();
    btPanel.setLayout(new BorderLayout());
    btPanel.setBorder(new TitledBorder(
      GT._("Bond Tolerance - sum of two covalent radii + this value")));
    JLabel btLabel = new JLabel(GT._("(Angstroms)"), SwingConstants.CENTER);
    btPanel.add(btLabel, BorderLayout.NORTH);

    btSlider = new JSlider(SwingConstants.HORIZONTAL, 0, 100,
        (int) (100 * viewer.getBondTolerance()));
    btSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    btSlider.setPaintTicks(true);
    btSlider.setMajorTickSpacing(20);
    btSlider.setMinorTickSpacing(10);
    btSlider.setPaintLabels(true);
    btSlider.getLabelTable().put(new Integer(0),
        new JLabel("0.0", SwingConstants.CENTER));
    btSlider.setLabelTable(btSlider.getLabelTable());
    btSlider.getLabelTable().put(new Integer(20),
        new JLabel("0.2", SwingConstants.CENTER));
    btSlider.setLabelTable(btSlider.getLabelTable());
    btSlider.getLabelTable().put(new Integer(40),
        new JLabel("0.4", SwingConstants.CENTER));
    btSlider.setLabelTable(btSlider.getLabelTable());
    btSlider.getLabelTable().put(new Integer(60),
        new JLabel("0.6", SwingConstants.CENTER));
    btSlider.setLabelTable(btSlider.getLabelTable());
    btSlider.getLabelTable().put(new Integer(80),
        new JLabel("0.8", SwingConstants.CENTER));
    btSlider.setLabelTable(btSlider.getLabelTable());
    btSlider.getLabelTable().put(new Integer(100),
        new JLabel("1.0", SwingConstants.CENTER));
    btSlider.setLabelTable(btSlider.getLabelTable());

    btSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        bondTolerance = source.getValue() / 100f;
        viewer.setBondTolerance(bondTolerance);
        currentProperties.put("bondTolerance", "" + bondTolerance);
        viewer.rebond();
      }
    });
    btPanel.add(btSlider);


    c.weightx = 0.0;
    gridbag.setConstraints(btPanel, c);
    bondPanel.add(btPanel);

    // minimum bond distance slider
    JPanel bdPanel = new JPanel();
    bdPanel.setLayout(new BorderLayout());
    bdPanel.setBorder(new TitledBorder(GT._("Minimum Bonding Distance")));
    JLabel bdLabel = new JLabel(GT._("(Angstroms)"), SwingConstants.CENTER);
    bdPanel.add(bdLabel, BorderLayout.NORTH);

    bdSlider = new JSlider(SwingConstants.HORIZONTAL, 0, 100,
        (int) (100 * viewer.getMinBondDistance()));
    bdSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    bdSlider.setPaintTicks(true);
    bdSlider.setMajorTickSpacing(20);
    bdSlider.setMinorTickSpacing(10);
    bdSlider.setPaintLabels(true);
    bdSlider.getLabelTable().put(new Integer(0),
        new JLabel("0.0", SwingConstants.CENTER));
    bdSlider.setLabelTable(bdSlider.getLabelTable());
    bdSlider.getLabelTable().put(new Integer(20),
        new JLabel("0.2", SwingConstants.CENTER));
    bdSlider.setLabelTable(bdSlider.getLabelTable());
    bdSlider.getLabelTable().put(new Integer(40),
        new JLabel("0.4", SwingConstants.CENTER));
    bdSlider.setLabelTable(bdSlider.getLabelTable());
    bdSlider.getLabelTable().put(new Integer(60),
        new JLabel("0.6", SwingConstants.CENTER));
    bdSlider.setLabelTable(bdSlider.getLabelTable());
    bdSlider.getLabelTable().put(new Integer(80),
        new JLabel("0.8", SwingConstants.CENTER));
    bdSlider.setLabelTable(bdSlider.getLabelTable());
    bdSlider.getLabelTable().put(new Integer(100),
        new JLabel("1.0", SwingConstants.CENTER));
    bdSlider.setLabelTable(bdSlider.getLabelTable());

    bdSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        minBondDistance = source.getValue() / 100f;
        viewer.setMinBondDistance(minBondDistance);
        currentProperties.put("minBondDistance", "" + minBondDistance);
        viewer.rebond();
      }
    });
    bdPanel.add(bdSlider);

    c.weightx = 0.0;
    gridbag.setConstraints(bdPanel, c);
    bondPanel.add(bdPanel);

    return bondPanel;
  }

  public JPanel buildColorsPanel() {

    JPanel colorPanel = new JPanel();
    colorPanel.setLayout(new GridLayout(0, 2));

    JPanel backgroundPanel = new JPanel();
    backgroundPanel.setLayout(new BorderLayout());
    backgroundPanel.setBorder(new TitledBorder(GT._("Background")));
    bButton = new JButton();
    bButton.setBackground(colorBackground);
    bButton.setToolTipText(GT._("Set the Background Color"));
    ActionListener startBackgroundChooser = new ActionListener() {

      public void actionPerformed(ActionEvent e) {

        Color color = JColorChooser.showDialog(
            bButton, GT._("Background Color"), colorBackground);
        if (color == null) {
          return;
        }
        colorBackground = color;
        bButton.setBackground(colorBackground);
        viewer.setColorBackground(colorBackground);
        currentProperties.put("colorBackground",
            Integer.toString(colorBackground.getRGB()));
      }
    };
    bButton.addActionListener(startBackgroundChooser);
    backgroundPanel.add(bButton, BorderLayout.CENTER);
    colorPanel.add(backgroundPanel);

    JPanel pickedPanel = new JPanel();
    pickedPanel.setLayout(new BorderLayout());
    pickedPanel.setBorder(new TitledBorder(GT._("Picked Atoms")));
    pButton = new JButton();
    pButton.setBackground(colorSelection);
    pButton.setToolTipText(GT._("Set the Color for Picked Atoms"));
    ActionListener startPickedChooser = new ActionListener() {

      public void actionPerformed(ActionEvent e) {

        Color color = JColorChooser.showDialog(
            pButton, GT._("Picked Atom Color"), colorSelection);
        if (color == null) {
          return;
        }
        colorSelection = color;
        pButton.setBackground(colorSelection);
        viewer.setColorSelection(colorSelection);
        currentProperties.put("colorSelection",
            Integer.toString(colorSelection.getRGB()));
      }
    };
    pButton.addActionListener(startPickedChooser);
    pickedPanel.add(pButton, BorderLayout.CENTER);
    colorPanel.add(pickedPanel);

    // text color panel
    JPanel textPanel = new JPanel();
    textPanel.setLayout(new BorderLayout());
    textPanel.setBorder(new TitledBorder(GT._("Text")));

    isLabelAtomColor = viewer.getColorLabel() == null;
    cbIsLabelAtomColor = guimap.newJCheckBox("Prefs.isLabelAtomColor",
                                             isLabelAtomColor);
    cbIsLabelAtomColor.addItemListener(checkBoxListener);
    textPanel.add(cbIsLabelAtomColor, BorderLayout.NORTH);

    tButton = new JButton();
    tButton.setBackground(colorText);
    tButton.setToolTipText(GT._("Set the Color for Text"));
    tButton.setEnabled(!isLabelAtomColor);
    ActionListener startTextChooser = new ActionListener() {

      public void actionPerformed(ActionEvent e) {

        Color color = JColorChooser.showDialog(
            tButton, GT._("Text Color"), colorText);
        if (color == null) {
          return;
        }
        colorText = color;
        tButton.setBackground(colorText);
        viewer.setColorLabel(colorText);
        currentProperties.put("colorText",
            Integer.toString(colorText.getRGB()));
      }
    };
    tButton.addActionListener(startTextChooser);
    textPanel.add(tButton, BorderLayout.CENTER);
    colorPanel.add(textPanel);

    // bond color panel
    JPanel bondPanel = new JPanel();
    bondPanel.setLayout(new BorderLayout());
    bondPanel.setBorder(new TitledBorder(GT._("Bonds")));

    isBondAtomColor = viewer.getColorBond() == null;
    cbIsBondAtomColor = guimap.newJCheckBox("Prefs.isBondAtomColor",
                                            isBondAtomColor);
    cbIsBondAtomColor.addItemListener(checkBoxListener);
    bondPanel.add(cbIsBondAtomColor, BorderLayout.NORTH);

    eButton = new JButton();
    eButton.setBackground(colorBond);
    eButton.setToolTipText(GT._("Set the Color for Bonds"));
    eButton.setEnabled(!isBondAtomColor);
    ActionListener startBondChooser = new ActionListener() {

      public void actionPerformed(ActionEvent e) {

        Color color = JColorChooser.showDialog(
            eButton, GT._("Bond Color"), colorBond);
        if (color == null) {
          return;
        }
        colorBond = color;
        eButton.setBackground(colorBond);
        viewer.setColorBond(colorBond);
        // should this be Integer.toString(colorBind.getRGB()) ?
        currentProperties.put("colorBond", "" + colorBond.getRGB());
      }
    };
    eButton.addActionListener(startBondChooser);
    bondPanel.add(eButton, BorderLayout.CENTER);
    colorPanel.add(bondPanel);

    // vector color panel
    JPanel vectorPanel = new JPanel();
    vectorPanel.setLayout(new BorderLayout());
    vectorPanel.setBorder(new TitledBorder(GT._("Vectors")));
    vButton = new JButton();
    vButton.setBackground(colorVector);
    vButton.setToolTipText(GT._("Set the Vector Color"));
    ActionListener startVectorChooser = new ActionListener() {

      public void actionPerformed(ActionEvent e) {

        Color color = JColorChooser.showDialog(
            vButton, GT._("Vector Color"), colorVector);
        if (color == null) {
          return;
        }
        colorVector = color;
        vButton.setBackground(colorVector);
        viewer.setColorVector(colorVector);
        currentProperties.put("colorVector",
            Integer.toString(colorVector.getRGB()));
        viewer.refresh();
      }
    };
    vButton.addActionListener(startVectorChooser);
    vectorPanel.add(vButton, BorderLayout.CENTER);
    colorPanel.add(vectorPanel);

    // measurement color panel
    JPanel measurementColorPanel = new JPanel();
    measurementColorPanel.setLayout(new BorderLayout());
    measurementColorPanel.setBorder(new TitledBorder(GT._("Measurements")));
    measurementColorButton = new JButton();
    measurementColorButton.setBackground(colorVector);
    measurementColorButton
      .setToolTipText(GT._("Color for distance, angle, & torsion measurements"));
    ActionListener startMeasurementColorChooser = new ActionListener() {

      public void actionPerformed(ActionEvent e) {

        Color color =JColorChooser.showDialog(
            measurementColorButton, GT._("Measurements Color"), colorMeasurement);
        if (color == null) {
          return;
        }
        colorMeasurement = color;
        measurementColorButton.setBackground(colorMeasurement);
        viewer.setColorMeasurement(colorMeasurement);
        currentProperties.put("colorMeasurement",
            Integer.toString(colorMeasurement.getRGB()));
        viewer.refresh();
      }
    };
    measurementColorButton.addActionListener(startMeasurementColorChooser);
    measurementColorPanel.add(measurementColorButton, BorderLayout.CENTER);
    colorPanel.add(measurementColorPanel);

    return colorPanel;
  }

  /*
  public JPanel buildVibratePanel() {

    JPanel vibratePanel = new JPanel();
    vibratePanel.setLayout(new GridLayout(0, 1));

    JPanel notePanel = new JPanel();
    notePanel.setLayout(new BorderLayout());
    notePanel.setBorder(new EtchedBorder());
    JLabel noteLabel = new JLabel(
      GT._("Note: settings will only be seen after selecting a new vibration."));
    notePanel.add(noteLabel, BorderLayout.CENTER);
    vibratePanel.add(notePanel);

    JPanel vasPanel = new JPanel();
    vasPanel.setLayout(new BorderLayout());
    vasPanel.setBorder(new TitledBorder(GT._("Amplitude Scale")));
    vasSlider = new JSlider(JSlider.HORIZONTAL, 0, 200,
        (int) (100.0 * Vibrate.getAmplitudeScale()));
    vasSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    vasSlider.setPaintTicks(true);
    vasSlider.setMajorTickSpacing(40);
    vasSlider.setPaintLabels(true);
    vasSlider.getLabelTable().put(new Integer(0),
        new JLabel("0.0", JLabel.CENTER));
    vasSlider.setLabelTable(vasSlider.getLabelTable());
    vasSlider.getLabelTable().put(new Integer(40),
        new JLabel("0.4", JLabel.CENTER));
    vasSlider.setLabelTable(vasSlider.getLabelTable());
    vasSlider.getLabelTable().put(new Integer(80),
        new JLabel("0.8", JLabel.CENTER));
    vasSlider.setLabelTable(vasSlider.getLabelTable());
    vasSlider.getLabelTable().put(new Integer(120),
        new JLabel("1.2", JLabel.CENTER));
    vasSlider.setLabelTable(vasSlider.getLabelTable());
    vasSlider.getLabelTable().put(new Integer(160),
        new JLabel("1.6", JLabel.CENTER));
    vasSlider.setLabelTable(vasSlider.getLabelTable());
    vasSlider.getLabelTable().put(new Integer(200),
        new JLabel("2.0", JLabel.CENTER));
    vasSlider.setLabelTable(vasSlider.getLabelTable());

    vasSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        VibrateAmplitudeScale = source.getValue() / 100.0;
        Vibrate.setAmplitudeScale(VibrateAmplitudeScale);
        currentProperties.put("VibrateAmplitudeScale",
            Double.toString(VibrateAmplitudeScale));
      }
    });
    vasPanel.add(vasSlider, BorderLayout.SOUTH);
    vibratePanel.add(vasPanel);

    JPanel vvsPanel = new JPanel();
    vvsPanel.setLayout(new BorderLayout());
    vvsPanel.setBorder(new TitledBorder(GT._("Vector Scale")));
    vvsSlider = new JSlider(JSlider.HORIZONTAL, 0, 200,
        (int) (100.0 * Vibrate.getVectorScale()));
    vvsSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    vvsSlider.setPaintTicks(true);
    vvsSlider.setMajorTickSpacing(40);
    vvsSlider.setPaintLabels(true);
    vvsSlider.getLabelTable().put(new Integer(0),
        new JLabel("0.0", JLabel.CENTER));
    vvsSlider.setLabelTable(vvsSlider.getLabelTable());
    vvsSlider.getLabelTable().put(new Integer(40),
        new JLabel("0.4", JLabel.CENTER));
    vvsSlider.setLabelTable(vvsSlider.getLabelTable());
    vvsSlider.getLabelTable().put(new Integer(80),
        new JLabel("0.8", JLabel.CENTER));
    vvsSlider.setLabelTable(vvsSlider.getLabelTable());
    vvsSlider.getLabelTable().put(new Integer(120),
        new JLabel("1.2", JLabel.CENTER));
    vvsSlider.setLabelTable(vvsSlider.getLabelTable());
    vvsSlider.getLabelTable().put(new Integer(160),
        new JLabel("1.6", JLabel.CENTER));
    vvsSlider.setLabelTable(vvsSlider.getLabelTable());
    vvsSlider.getLabelTable().put(new Integer(200),
        new JLabel("2.0", JLabel.CENTER));
    vvsSlider.setLabelTable(vvsSlider.getLabelTable());

    vvsSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        VibrateVectorScale = source.getValue() / 100.0;
        Vibrate.setVectorScale(VibrateVectorScale);
        currentProperties.put("VibrateVectorScale",
            Double.toString(VibrateVectorScale));
      }
    });
    vvsPanel.add(vvsSlider, BorderLayout.SOUTH);
    vibratePanel.add(vvsPanel);

    JPanel vfPanel = new JPanel();
    vfPanel.setLayout(new BorderLayout());
    vfPanel.setBorder(new TitledBorder(GT._("Number of Frames")));

    vfSlider = new JSlider(JSlider.HORIZONTAL, 0, 50,
        Vibrate.getNumberFrames());
    vfSlider.putClientProperty("JSlider.isFilled", Boolean.TRUE);
    vfSlider.setPaintTicks(true);
    vfSlider.setMajorTickSpacing(5);
    vfSlider.setPaintLabels(true);
    vfSlider.addChangeListener(new ChangeListener() {

      public void stateChanged(ChangeEvent e) {

        JSlider source = (JSlider) e.getSource();
        VibrationFrames = source.getValue();
        Vibrate.setNumberFrames(VibrationFrames);
        currentProperties.put("VibrationFrames",
            Integer.toString(VibrationFrames));
      }
    });

    vfPanel.add(vfSlider, BorderLayout.SOUTH);
    vibratePanel.add(vfPanel);

    return vibratePanel;
  }
  */

  protected void centerDialog() {

    Dimension screenSize = this.getToolkit().getScreenSize();
    Dimension size = this.getSize();
    screenSize.height = screenSize.height / 2;
    screenSize.width = screenSize.width / 2;
    size.height = size.height / 2;
    size.width = size.width / 2;
    int y = screenSize.height - size.height;
    int x = screenSize.width - size.width;
    this.setLocation(x, y);
  }

  public void ok() {
    save();
    dispose();
  }

  public void cancel() {
    updateComponents();
    dispose();
  }

  private void updateComponents() {
    // Display panel
    cH.setSelected(viewer.getShowHydrogens());
    cM.setSelected(viewer.getShowMeasurements());

    cbWireframeRotation.setSelected(viewer.getWireframeRotation());

    cbPerspectiveDepth.setSelected(viewer.getPerspectiveDepth());
    cbShowAxes.setSelected(viewer.getShowAxes());
    cbShowBoundingBox.setSelected(viewer.getShowBbcage());

    cbAxesOrientationRasmol.setSelected(viewer.getAxesOrientationRasmol());
    
    cbOpenFilePreview.setSelected(openFilePreview);

    // Atom panel controls: 
    vdwPercentSlider.setValue(viewer.getPercentVdwAtom());

    // Bond panel controls:
    abYes.setSelected(viewer.getAutoBond());
    bwSlider.setValue(viewer.getMadBond()/2);
    bdSlider.setValue((int) (100 * viewer.getMinBondDistance()));
    btSlider.setValue((int) (100 * viewer.getBondTolerance()));

    // Color panel controls:
    bButton.setBackground(colorBackground);
    pButton.setBackground(colorSelection);
    cbIsLabelAtomColor.setSelected(isLabelAtomColor);
    tButton.setBackground(colorText);
    tButton.setEnabled(!isLabelAtomColor);
    cbIsBondAtomColor.setSelected(isBondAtomColor);
    eButton.setBackground(colorBond);
    eButton.setEnabled(!isBondAtomColor);
    vButton.setBackground(colorVector);
    measurementColorButton.setBackground(colorMeasurement);

    /*
    // Vibrate panel controls
    vasSlider.setValue((int) (100.0 * Vibrate.getAmplitudeScale()));
    vvsSlider.setValue((int) (100.0 * Vibrate.getVectorScale()));
    vfSlider.setValue(Vibrate.getNumberFrames());
    */

  }

  private void save() {
    try {
      FileOutputStream fileOutputStream =
        new FileOutputStream(Jmol.UserPropsFile);
      currentProperties.store(fileOutputStream, "Jmol");
      fileOutputStream.close();
    } catch (Exception e) {
      System.out.println("Error saving preferences" + e);
    }
    viewer.refresh();
  }

  void initializeProperties() {
    originalSystemProperties = System.getProperties();
    jmolDefaultProperties = new Properties(originalSystemProperties);
    for (int i = jmolDefaults.length; (i -= 2) >= 0; )
      jmolDefaultProperties.put(jmolDefaults[i], jmolDefaults[i+1]);
    currentProperties = new Properties(jmolDefaultProperties);
    try {
      FileInputStream fis2 = new FileInputStream(Jmol.UserPropsFile);
      currentProperties.load(new BufferedInputStream(fis2, 1024));
      fis2.close();
    } catch (Exception e2) {
    }
    System.setProperties(currentProperties);
  }

  void resetDefaults(String[] overrides) {
    currentProperties = new Properties(jmolDefaultProperties);
    System.setProperties(currentProperties);
    if (overrides != null) {
      for (int i = overrides.length; (i -= 2) >= 0; )
        currentProperties.put(overrides[i], overrides[i+1]);
    }
    initVariables();
    viewer.refresh();
    updateComponents();
  }

  void initVariables() {

    autoBond = Boolean.getBoolean("autoBond");
    showHydrogens = Boolean.getBoolean("showHydrogens");
    //showVectors = Boolean.getBoolean("showVectors");
    showMeasurements = Boolean.getBoolean("showMeasurements");
    wireframeRotation = Boolean.getBoolean("wireframeRotation");
    perspectiveDepth = Boolean.getBoolean("perspectiveDepth");
    showAxes = Boolean.getBoolean("showAxes");
    showBoundingBox = Boolean.getBoolean("showBoundingBox");
    axesOrientationRasmol = Boolean.getBoolean("axesOrientationRasmol");
    openFilePreview = Boolean.valueOf(System.getProperty("openFilePreview", "true")).booleanValue();
    colorBackground = Color.getColor("colorBackground");
    colorSelection = Color.getColor("colorSelection");
    isLabelAtomColor = Boolean.getBoolean("isLabelAtomColor");
    colorText = Color.getColor("colorText");
    isBondAtomColor = Boolean.getBoolean("isBondAtomColor");
    colorBond = Color.getColor("colorBond");
    colorVector = Color.getColor("colorVector");
    colorMeasurement = Color.getColor("colorMeasurement");
    /*
    VibrationFrames = Integer.getInteger("VibrationFrames").intValue();
    */

    minBondDistance =
      new Float(currentProperties.getProperty("minBondDistance")).floatValue();
    bondTolerance =
      new Float(currentProperties.getProperty("bondTolerance")).floatValue();
    marBond = Short.parseShort(currentProperties.getProperty("marBond"));
    percentVdwAtom =
      Integer.parseInt(currentProperties.getProperty("percentVdwAtom"));
    /*
    VibrateAmplitudeScale =
        new Double(currentProperties.getProperty("VibrateAmplitudeScale")).doubleValue();
    VibrateVectorScale =
        new Double(currentProperties.getProperty("VibrateVectorScale")).doubleValue();
    */
    //    viewer.setColorOutline(colorOutline);
    viewer.setColorSelection(colorSelection);
    viewer.setColorLabel(isLabelAtomColor ? null : colorText);
    viewer.setColorBond(isBondAtomColor ? null : colorBond);
    viewer.setPercentVdwAtom(percentVdwAtom);
    //viewer.setPropertyStyleString(AtomPropsMode);
    viewer.setMarBond(marBond);
    viewer.setColorVector(colorVector);
    viewer.setColorMeasurement(colorMeasurement);
    viewer.setColorBackground(colorBackground);
    viewer.setMinBondDistance(minBondDistance);
    viewer.setBondTolerance(bondTolerance);
    viewer.setAutoBond(autoBond);
    viewer.setShowHydrogens(showHydrogens);
    viewer.setShowMeasurements(showMeasurements);
    viewer.setWireframeRotation(wireframeRotation);
    viewer.setPerspectiveDepth(perspectiveDepth);
    viewer.setShowAxes(showAxes);
    viewer.setShowBbcage(showBoundingBox);
    viewer.setAxesOrientationRasmol(axesOrientationRasmol);
    /*
    Vibrate.setAmplitudeScale(VibrateAmplitudeScale);
    Vibrate.setVectorScale(VibrateVectorScale);
    Vibrate.setNumberFrames(VibrationFrames);
    */
  }

  class PrefsAction extends AbstractAction {

    public PrefsAction() {
      super("prefs");
      this.setEnabled(true);
    }

    public void actionPerformed(ActionEvent e) {
      show();
    }
  }

  public Action[] getActions() {
    Action[] defaultActions = {
      prefsAction
    };
    return defaultActions;
  }

  protected Action getAction(String cmd) {
    return (Action) commands.get(cmd);
  }

  ItemListener checkBoxListener = new ItemListener() {

    //Component c;
    //AbstractButton b;

    public void itemStateChanged(ItemEvent e) {

      JCheckBox cb = (JCheckBox) e.getSource();
      String key = guimap.getKey(cb);
      boolean isSelected = cb.isSelected();
      String strSelected = isSelected ? "true" : "false";
      if (key.equals("Prefs.showHydrogens")) {
        showHydrogens = isSelected;
        viewer.setShowHydrogens(showHydrogens);
        currentProperties.put("showHydrogens", strSelected);
      } else if (key.equals("Prefs.showMeasurements")) {
        showMeasurements = isSelected;
        viewer.setShowMeasurements(showMeasurements);
        currentProperties.put("showMeasurements", strSelected);
      } else if (key.equals("Prefs.isLabelAtomColor")) {
        isLabelAtomColor = isSelected;
        viewer.setColorLabel(isLabelAtomColor ? null : colorText);
        currentProperties.put("isLabelAtomColor", strSelected);
        tButton.setEnabled(!isLabelAtomColor);
      } else if (key.equals("Prefs.isBondAtomColor")) {
        isBondAtomColor = isSelected;
        viewer.setColorBond(isBondAtomColor ? null : colorBond);
        currentProperties.put("isBondAtomColor", strSelected);
        eButton.setEnabled(!isBondAtomColor);
      } else if (key.equals("Prefs.wireframeRotation")) {
        wireframeRotation = isSelected;
        viewer.setWireframeRotation(wireframeRotation);
        currentProperties.put("wireframeRotation", strSelected);
      } else if (key.equals("Prefs.perspectiveDepth")) {
        perspectiveDepth = isSelected;
        viewer.setPerspectiveDepth(perspectiveDepth);
        currentProperties.put("perspectiveDepth", strSelected);
      } else if (key.equals("Prefs.showAxes")) {
        showAxes = isSelected;
        viewer.setShowAxes(isSelected);
        currentProperties.put("showAxes", strSelected);
      } else if (key.equals("Prefs.showBoundingBox")) {
        showBoundingBox = isSelected;
        viewer.setShowBbcage(isSelected);
        currentProperties.put("showBoundingBox", strSelected);
      } else if (key.equals("Prefs.axesOrientationRasmol")) {
        axesOrientationRasmol = isSelected;
        viewer.setAxesOrientationRasmol(isSelected);
        currentProperties.put("axesOrientationRasmol", strSelected);
      } else if (key.equals("Prefs.openFilePreview")) {
      	openFilePreview = isSelected;
      	currentProperties.put("openFilePreview", strSelected);
      }
    }
  };

  private JButton applyButton;
  private JButton jmolDefaultsButton;
  private JButton rasmolDefaultsButton;
  private JButton cancelButton;
  private JButton okButton;
  
  public void actionPerformed(ActionEvent event) {
    if (event.getSource() == applyButton) {
      save();
    } else if (event.getSource() == jmolDefaultsButton) {
      resetDefaults(null);
    } else if (event.getSource() == rasmolDefaultsButton) {
      resetDefaults(rasmolOverrides);
    } else if (event.getSource() == cancelButton) {
      cancel();
    } else if (event.getSource() == okButton) {
      ok();
    }
  }

}
