package pdbParser;

import java.awt.*;

import javax.swing.*;

/**
 * A class that calls functions to generate a membrane model as a
 * .pdb-File. The current process of generating the membrane model will be
 * described by a progressbar.
 *
 */
public class PDBProgressBar extends JWindow {

	private PDBwriter writer;

	private JProgressBar bar;

	private boolean status = true;

	private int max;
	
	final JFrame frame;

	
	/**
	 * By initialising this constructor the membrane model will be generated as a pdb.-File. 
	 * The current status of the process is
	 * described by a ProgrssBar.
	 * 
	 * @param Frame frame 
	 */
	public PDBProgressBar(final JFrame frame) {
		
		this.frame = frame;
		
		writer = new PDBwriter();		
		max = (int) writer.getProgressPDB() / 1000;

		int width = 250;
		int height = 50;
		int progBarHeight = 20;
		int labelHeight = 20;

		JPanel contentPane = new JPanel();
		contentPane.setLayout(null);
		contentPane.setBorder(BorderFactory.createLineBorder(Color.BLACK, 1));
		contentPane.setBackground(Color.GRAY);

		JLabel label = new JLabel();
		label.setFont(new Font("", Font.BOLD, 12));
		label.setBounds(3,5,width-6,labelHeight);
		label.setText("                    Generating PDB-file ...");
		label.setForeground(Color.WHITE);

		bar = new JProgressBar();
		bar.setStringPainted(true);
		bar.setBackground(Color.WHITE);
		bar.setBounds(3,28,width-6,progBarHeight);
		
		bar.setBorderPainted(true);
		bar.setMaximum(max);

		contentPane.add(label);
		contentPane.add(bar);

		setSize(width, height);
		
		//neccassary to position the Progessbar
		Point point =frame.getLocation();
		
		setLocation((50+point.x+(frame.getWidth()/2)-(width/2)),(point.y+(frame.getHeight()/2)-(height/2)));
		setContentPane(contentPane);

		setVisible(true);
		
		
		
		/*
		 * This thread calls the function to generate a PDB
		 * 
		 * */
		Runnable pdbstart = new Runnable() {
			public void run() {
				frame.setEnabled(false);
				writer.writePDB();
				status = false;
				frame.setEnabled(true);
			}
		};

		
		/*
		 * This thread checks the current status of the PDB-generating-function.
		 * 
		 * */
		Runnable update = new Runnable() {
			public void run() {
				while (status) {
					bar.setValue((int) writer.getCurrentthreadCount() / 1000);
				}

				bar.setValue(max);
				try {
					Thread.sleep(200);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				setVisible(false);
			}
		};

		/*
		 * Threads will be executed
		 * */
		
		new Thread(pdbstart).start();
		new Thread(update).start();

	}
}
