package pdbParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.Writer;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import protein.Protein;

/**
 * This class makes functions available
 * to construct a Protein object by given PDBfile. 
 * 
 *
 */
public class PDBreader implements IPDBreader {

	private Hashtable hash = null;

	private StringBuffer uff = new StringBuffer();
	
	
	/**
	 * initialises the constructor
	 *
	 */	
	public PDBreader() {
	}

	/** 
	 * This function creates for each PDBfile in the
	 * given directory a Protein object. 
	 * The Protein objects will be stored in a hashtable. The keys are the PDBnames
	 * and the values are the corresponding Protein objects. 
	 * 
	 * @param PDBnames Vector
	 * @param Directory String
	 * @return Proteins Hashtable 
	 * @throws IOException 
	 */
	public Hashtable getProteins(Vector vector, String DBpath) throws IOException {

		hash = new Hashtable();
		Iterator it = vector.iterator();
		while (it.hasNext()) {

			String PDBcode = (String) it.next();
			hash.put(PDBcode, readFile(PDBcode, DBpath));
		}
		return hash;
	}

	/**
	 * 
	 * This function creates a Protein object 
	 * by given PDBname and directory.
	 * 
	 * @param String PDBname
	 * @param String Path	
	 * @return Protein Protein
	 * @throws IOException 
	 */
	public Protein getProtein(String PDBcode, String DBpath) throws IOException {
		return readFile(PDBcode, DBpath);
	}

	/**
	 * 
	 * This function creates for each PDBfile in the
	 * given directory a Protein object. 
	 * The Protein objects will be stored in a hashtable.
	 * The keys are the PDBnames
	 * and the values are the correspnding Protein objects. 
	 * 
	 * @param DBpath String
	 * @return ProteinObjects Hashtable
	 * @throws IOException 
	 */
	public Hashtable getAllProteins(String DBpath) throws IOException {

		hash = new Hashtable();
		File[] pdbs = null;
		File dir = new File(DBpath);

		if (dir.exists()) {
			if (dir.isDirectory()) {
				pdbs = dir.listFiles(new FilenameFilter() {
					public boolean accept(File dir, String name) {
						return name.endsWith(".pdb");
					}
				});
			}
		}

		for (int i = 0; i < pdbs.length; i++) {
			Protein protein = readFile(null, pdbs[i].getAbsolutePath());
			hash.put(protein.getPDBcode(), protein);
		}

		return hash;
	}
	/*
	 *this function extracts 
	 *information about the protein
	 *from given pdb-file 
	 *like atom coordinates, 
	 *secondary structures and more 
	 *
	 * 
	 * 
	 * */
	private Protein readFile(String PDBcode, String DBpath) throws IOException {

	    int residueSequenceCount = 0;
		Protein protein = null;
		BufferedReader buffreader;
		boolean ter_symbol = false;
		boolean SEQRES_stop = false;
		boolean HET_stop = false;
		boolean HELIX_stop = false;
		boolean SHEET_stop = false;
		boolean TURN_stop = false;
		boolean SSBOND_stop = false;
		boolean LINK_stop = false;
		boolean HYDBOND_stop = false;
		boolean SLTBRG_stop = false;
		boolean CISPEP_stop = false;
		boolean ATOM_stop = false;

		String line, name, date, filepath;
		StringBuffer COMPD = new StringBuffer();
		StringBuffer SOURCE = new StringBuffer();
		StringBuffer DBREF = new StringBuffer();
		StringBuffer SEQRES = new StringBuffer();
		StringBuffer HET = new StringBuffer();
		StringBuffer HETNAM = new StringBuffer();
		StringBuffer FORMUL = new StringBuffer();
		StringBuffer HELIX = new StringBuffer();
		StringBuffer SHEET = new StringBuffer();
		StringBuffer TURN = new StringBuffer();
		StringBuffer SSBOND = new StringBuffer();
		StringBuffer LINK = new StringBuffer();
		StringBuffer HYDBND = new StringBuffer();
		StringBuffer SLTBRG = new StringBuffer();
		StringBuffer CISPEP = new StringBuffer();
		StringBuffer superAtom = new StringBuffer();
		Vector description = new Vector();
		name = null;
		Vector[] coordinates = new Vector[4];
		Vector atomPosX = new Vector();
		Vector atomPosY = new Vector();
		Vector atomPosZ = new Vector();
		Vector atom = new Vector();
		int model_count = 0;
		int atom_count = 0;
		int atom_quantity =0;
		String substring = null;
		char stop_char = '@';

		if (PDBcode == null) {
			filepath = DBpath;
		} else {
			filepath = DBpath + "\\" + PDBcode + ".pdb";
		}
		
			
			buffreader = new BufferedReader(new FileReader(filepath));

			// liest die erste Zeile

			StringTokenizer st = new StringTokenizer(buffreader.readLine()
					.substring(7));
			int i = 0;
			while (i < st.countTokens() - 2) {
				if (name == null) {
					name = st.nextToken();
				} else {
					name = name + " " + st.nextToken();
				}
			}

			date = st.nextToken();
			if (PDBcode == null) {
				PDBcode = st.nextToken();				
			} else {
				name = PDBcode;
				DBpath = DBpath + PDBcode + ".pdb";
			}

			while ((line = buffreader.readLine()) != null
					&& ter_symbol == false) {

				StringBuffer current = new StringBuffer(line);

				// primre Struktur

				if (line.startsWith("MODEL ") && !ATOM_stop) {
					superAtom.append(line + "\n");
				} else if (line.startsWith("TITLE ")) {
					description.add(line.substring(10, line.length() - 1));
					
				}

				else if (line.startsWith("ENDMDL") && !ATOM_stop) {
					superAtom.append(line + "\n");
					ter_symbol = true;
				}

				else if (line.startsWith("ATOM") && !ATOM_stop) {
					if (line.charAt(21) != ' ') {
						if (line.charAt(21) != stop_char) {
							current.setCharAt(21, 'A');
							atom_count++;
							superAtom.append(current + "\n");
							atomPosX
									.add(Double.valueOf(line.substring(30, 38)));
							atomPosY
									.add(Double.valueOf(line.substring(38, 46)));
							atomPosZ
									.add(Double.valueOf(line.substring(46, 54)));
							atom.add(line.substring(76, 78));
							residueSequenceCount = (Double.valueOf(line
									.substring(22, 26))).intValue();
						} else {
							ATOM_stop = true;
						}
					} else {
						current.setCharAt(21, 'A');
						atom_count++;
						superAtom.append(current + "\n");
						atomPosX.add(Double.valueOf(line.substring(30, 38)));
						atomPosY.add(Double.valueOf(line.substring(38, 46)));
						atomPosZ.add(Double.valueOf(line.substring(46, 54)));
						atom.add(line.substring(76, 78));
						residueSequenceCount = (Double.valueOf(line.substring(
								22, 26))).intValue();

					}
				} else if (line.startsWith("HETATM") && !ATOM_stop) {
					current.setCharAt(21, 'A');
					atom_count++;
					superAtom.append(current + "\n");
					atomPosX.add(Double.valueOf(line.substring(30, 38)));
					atomPosY.add(Double.valueOf(line.substring(38, 46)));
					atomPosZ.add(Double.valueOf(line.substring(46, 54)));
					residueSequenceCount = (Double.valueOf(line.substring(22,
							26))).intValue();
					if (line.length() >= 77) {

						atom.add(line.substring(76, 78));
					} else {
						atom.add("  ");
					}
				}

				else if (line.startsWith("SIGATM") && !ATOM_stop) {
					atom_count++;
					current.setCharAt(21, 'A');
					superAtom.append(current + "\n");
					atomPosX.add(Double.valueOf(line.substring(30, 38)));
					atomPosY.add(Double.valueOf(line.substring(38, 46)));
					atomPosZ.add(Double.valueOf(line.substring(46, 54)));
					atom.add(line.substring(76, 78));
					residueSequenceCount = (Double.valueOf(line.substring(22,
							26))).intValue();
				} else if (line.startsWith("ANISOU") && !ATOM_stop) {
					atom_count++;
					current.setCharAt(21, 'A');
					superAtom.append(current + "\n");

				} else if (line.startsWith("SIGUIJ") && !ATOM_stop) {
					atom_count++;
					current.setCharAt(21, 'A');
					superAtom.append(current + "\n");
				} else if (line.startsWith("TER") && !ATOM_stop) {
					atom_count++;
					model_count++;
					current.setCharAt(21, 'A');
					superAtom.append(current + "\n");
					residueSequenceCount = (Double.valueOf(line.substring(22,
							26))).intValue();
				}

				else if (line.startsWith("SOURCE")) {
					SOURCE.append(line + "\n");
					atom_quantity++;
				}

				else if (line.startsWith("SEQRES") && !SEQRES_stop) {

					if (line.charAt(11) != ' ') {
						if (line.charAt(11) != stop_char) {
							current.setCharAt(11, 'A');
							SEQRES.append(current + "\n");
							atom_quantity++;
						} else {
							SEQRES_stop = true;
						}
					} else {
						current.setCharAt(11, 'A');
						atom_quantity++;
						SEQRES.append(current + "\n");
					}

				}

				else if (line.startsWith("DBREF")) {

					if (stop_char == '@') {
						if (substring != null) {
							if (substring.equals(line.substring(14, line
									.length() - 1))) {
								stop_char = line.charAt(12);
							} else {
								substring = line.substring(14,
										line.length() - 1);
							}

						} else {
							atom_quantity++;
							substring = line.substring(14, line.length() - 1);
							DBREF.append(current + "\n");
						}
					}

				} else if (line.startsWith("HET ") && !HET_stop) {

					if (line.charAt(12) != ' ') {
						if (line.charAt(12) != stop_char) {
							current.setCharAt(12, 'A');
							atom_quantity++;
							HET.append(current + "\n");
						} else {
							HET_stop = true;
						}
					} else {
						atom_quantity++;
						current.setCharAt(12, 'A');
						HET.append(current + "\n");
					}

				} else if (line.startsWith("HETNAM")) {
					HETNAM.append(line + "\n");
					atom_quantity++;
				} else if (line.startsWith("SSBOND") && !SSBOND_stop) {

					if (line.charAt(16) != ' ') {
						if (line.charAt(16) != stop_char) {

							current.setCharAt(15, 'A');
							current.setCharAt(29, 'A');
							atom_quantity++;
							SSBOND.append(current + "\n");
						} else {
							SSBOND_stop = true;
						}
					} else {

						current.setCharAt(15, 'A');
						current.setCharAt(29, 'A');
						atom_quantity++;
						SSBOND.append(current + "\n");
					}

				} else if (line.startsWith("LINK") && !LINK_stop) {

					if (line.charAt(21) != ' ') {
						if (line.charAt(21) != stop_char) {

							current.setCharAt(21, 'A');
							current.setCharAt(51, 'A');
							atom_quantity++;
							LINK.append(current + "\n");
						} else {
							LINK_stop = true;
						}
					} else {
						atom_quantity++;
						current.setCharAt(21, 'A');
						current.setCharAt(51, 'A');
						LINK.append(current + "\n");
					}

				} else if (line.startsWith("HYDBND") && !HYDBOND_stop) {

					if (line.charAt(21) != ' ') {
						if (line.charAt(21) != stop_char) {
							current.setCharAt(21, 'A');
							current.setCharAt(52, 'A');
							atom_quantity++;
							HYDBND.append(current + "\n");
						} else {
							HYDBOND_stop = true;
						}
					} else {
						current.setCharAt(21, 'A');
						current.setCharAt(52, 'A');
						atom_quantity++;
						HYDBND.append(current + "\n");
					}

				} else if (line.startsWith("SLTBRG") && !SLTBRG_stop) {

					if (line.charAt(21) != ' ') {
						if (line.charAt(21) != stop_char) {
							current.setCharAt(21, 'A');
							current.setCharAt(51, 'A');
							SLTBRG.append(current + "\n");
							atom_quantity++;
						} else {
							SLTBRG_stop = true;
						}
					} else {
						current.setCharAt(21, 'A');
						current.setCharAt(51, 'A');
						SLTBRG.append(current + "\n");
						atom_quantity++;
					}
				} else if (line.startsWith("CISPEP") && !CISPEP_stop) {

					if (line.charAt(15) != ' ') {
						if (line.charAt(15) != stop_char) {
							current.setCharAt(15, 'A');
							current.setCharAt(29, 'A');
							CISPEP.append(current + "\n");
							atom_quantity++;
						} else {
							CISPEP_stop = true;
						}
					} else {
						current.setCharAt(15, 'A');
						current.setCharAt(29, 'A');
						CISPEP.append(current + "\n");
						atom_quantity++;
					}

				} else if (line.startsWith("COMPND")) {
					COMPD.append(line + "\n");
					atom_quantity++;
				} else if (line.startsWith("FORMUL")) {
					FORMUL.append(line + "\n");
					atom_quantity++;
				}

				else if (line.startsWith("HELIX") && !HELIX_stop) {

					if (line.charAt(19) != ' ') {
						if (line.charAt(19) != stop_char) {

							current.setCharAt(19, 'A');
							current.setCharAt(31, 'A');
							atom_quantity++;
							HELIX.append(current + "\n");
						} else {
							HELIX_stop = true;
						}
					} else {

						current.setCharAt(19, 'A');
						current.setCharAt(31, 'A');
						atom_quantity++;
						HELIX.append(current + "\n");
					}

				} else if (line.startsWith("SHEET") && !SHEET_stop) {
					if (line.charAt(21) != ' ') {
						if (line.charAt(21) != stop_char) {

							current.setCharAt(21, 'A');
							current.setCharAt(32, 'A');

							if (current.charAt(64) != ' ') {

								current.setCharAt(49, 'A');
								current.setCharAt(64, 'A');
							}
							atom_quantity++;
							SHEET.append(current + "\n");
						} else {
							SHEET_stop = true;
						}
					} else {

						current.setCharAt(21, 'A');
						current.setCharAt(32, 'A');

						if (current.charAt(64) != ' ') {

							current.setCharAt(49, 'A');
							current.setCharAt(64, 'A');
						}
						atom_quantity++;
						SHEET.append(current + "\n");
					}
				}

				else if (line.startsWith("TURN") && !TURN_stop) {
					if (line.charAt(19) != ' ') {
						if (line.charAt(19) != stop_char) {
							atom_quantity++;
							current.setCharAt(19, 'A');
							current.setCharAt(30, 'A');
							TURN.append(current + "\n");
						} else {
							TURN_stop = true;
						}
					} else {
						atom_quantity++;
						current.setCharAt(19, 'A');
						current.setCharAt(30, 'A');
						TURN.append(current + "\n");
					}
				}
			}

			coordinates[1] = atomPosX;
			coordinates[2] = atomPosY;
			coordinates[3] = atomPosZ;
			coordinates[0] = atom;
			buffreader.close();
			protein = new Protein(PDBcode);
			protein.setSuperAtom(superAtom.toString());
			protein.setPDBcode(PDBcode);
			protein.set_Line_quantity((atom_quantity+atom_count));
			protein.setModel_count(1);
			protein.setCOMPD(COMPD.toString());
			protein.setHET(HET.toString());
			protein.setHETNAM(HETNAM.toString());
			protein.setSOURCE(SOURCE.toString());

			if (DBREF.toString().length() != 0) {
				DBREF.setCharAt(12, 'A');
				DBREF.replace(14, 18, spaceGenerater(1, 4));
				DBREF.replace(20, 24, spaceGenerater(residueSequenceCount, 4));
			}

			protein.setDBREF(DBREF.toString());
			StringBuffer modifiedSEQRES = new StringBuffer();
			if (SEQRES.toString().length() != 0) {

				StringTokenizer tok = new StringTokenizer(SEQRES.toString(),
						"\n", false);

				while (tok.hasMoreTokens()) {

					StringBuffer current2 = new StringBuffer(tok.nextToken());
					current2.replace(13, 17, spaceGenerater(
							residueSequenceCount, 4));
					modifiedSEQRES.append(current2 + "\n");
				}
			}

			protein.setSEQRES(modifiedSEQRES.toString());
			protein.setFORMUL(FORMUL.toString());
			protein.setHELIX(HELIX.toString());
			protein.setSHEET(SHEET.toString());
			protein.setTURN(TURN.toString());
			protein.setDate(date);
			protein.setName(name);
			protein.setAtom_count(atom_count);
			protein.setCoordinates(coordinates);
			protein.setOriginalCoordinates(coordinates);
			protein.setSSBOND(SSBOND.toString());
			protein.setLINK(LINK.toString());
			protein.setHYDBND(HYDBND.toString());
			protein.setSLTBRG(SLTBRG.toString());
			protein.setCISPEP(CISPEP.toString());
			protein.setResidueSequenceCount(residueSequenceCount);
			protein.setFilepath(DBpath);
			protein.setDescription(description);


		return protein;
	}
	
	/*
	 * 
	 * this function casts an integer
	 * to a string with given length   
	 * 
	 * @param integer value
	 * @param integer length
	 * @return String
	 */	
	private String spaceGenerater(int value, int length) {

		String space = "";
		String line_str = String.valueOf(value);

		for (int h = 0; h < (length - line_str.length()); h++) {
			space = space + " ";
		}
		
		line_str = space + line_str;
		return line_str;
	}
}