package pdbParser;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.vecmath.Matrix3f;

import protein.Protein;

import membranModel.MembranModel;
import membranModel.MembranModelSingleton;

public class PDBwriter implements IPDBwriter {

	MembranModel membran = MembranModelSingleton.getInstance();

//	File file;
	
	File fi ;

	StringBuffer pdb = new StringBuffer();

	int remarkRecord = 8;

	int numHETRecord = 0;

	int numHELIXRecord = 0;

	int numSHEETRecord = 0;

	int numTURNRecord = 0;

	int numSITERecord = 0;

	int numXformRecord = 6;

	int nummCoordRecord = 0;

	int numTerRecord = 0;

	int numConectRecord = 0;

	int numSEQRESRecord = 0;

	Vector vek = membran.getAllProteins();

	private String filename;
	
	private float currentthreadCount=0;

	private Protein lipid = membran.getBiLayerType();

	String date;

	double lipid_X;;

	double lipid_Z;

	public PDBwriter() {
//		this.filename = filename;
		lipid_X = lipid.getSize()[0];
		lipid_Z = lipid.getSize()[2];
		
		JFileChooser filechooser = new JFileChooser();
		filechooser.setFileFilter(new PDBFileFilter());
		filechooser.showSaveDialog(null);
		
		
		fi = filechooser.getSelectedFile();
		
		if(fi!=null){
			if(!fi.getAbsolutePath().endsWith(".pdb")){
				fi = new File(fi.toString()+".pdb");
			}
		}else{
			return;
		}
			
		this.filename = fi.toString();	
	}

	public float getProgressPDB(){
		float progressCount=0;
		
		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			
			Protein prot = (Protein) vek.elementAt(vekCount);		
			progressCount = progressCount +(prot.getProteinPositions().size()*prot.get_Line_quantity());	
	
		}		
		progressCount = progressCount +(2*(lipid.getProteinPositions().size()*lipid.get_Line_quantity()));		
		return progressCount;
	}
	
	
	
	public void writePDB() {
//		JFileChooser filechooser = new JFileChooser();
//		filechooser.showSaveDialog(null);
//		File fi = filechooser.getSelectedFile();


		//von Jobo muss hier rein fr die komplette membranView /!\
//		this.filename = fi.toString();	
		
		
		if (fi != null) {

			FileWriter out = null;
			try {
				out = new FileWriter(fi);
				writeHeader(filename, out);
				writeTitle(out);
				writeCOMPND(out);
				writeSOURCE(out);
				writeKEYWDS(out);
				writeEXPDATA(out);
				writeAUTHOR(out);
				writeREVDAT(out);
				writeREMARK2(out);
				writeREMARK3(out);
				writeBREF(out);
				writeSEQRES(out);
				writeHET(out);
//				writeHETNAM(out);
				writeHELIX(out);
				writeSHEET(out);
				writeTURN(out);

				writeSSBOND(out);
				writeLINK(out);
				writeHYDBND(out);
				writeSLTBRG(out);
				writeCISPEP(out);

				writeCRYST1(out);
				writeORIGX(out);
				writeSCALE(out);
				writeMODEL(out);
				writeMASTER(out);
				writeEND(out);

			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			
			try {
				out.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

	}

	private void writeHeader(String filename, FileWriter out) {

		SimpleDateFormat fmt;
		fmt = new SimpleDateFormat();
		fmt.applyPattern("dd'-'MMM'-'yy");

		Calendar cal;
		cal = new GregorianCalendar();
		date = fmt.format(cal.getTime());
		try {
			out.write("HEADER \t" + membran.getMembraneTitle() + "  " + date
					+ "  " + filename + "\n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void writeTitle(FileWriter out) {

		String title = "";
		StringTokenizer tok = new StringTokenizer(membran.getDescription(),
				"\n", false);
		while (tok.hasMoreTokens()) {
			title = title + "TITLE \t" + tok.nextToken() + "\n";
			
			currentthreadCount++;
		}
		try {
			out.write(title);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeCOMPND(FileWriter out) {

		StringBuffer COMPD = new StringBuffer();

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			COMPD.append(prot.getCOMPD());
			currentthreadCount++;

		}
		try {
			out.write(COMPD.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String writeOBSLTE(FileWriter out) {
		return null;
	}

	private String writeCAVEAT(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private String writeSPRSDE(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private String writeJRNL(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeSOURCE(FileWriter out) {

		StringBuffer SOURCE = new StringBuffer();

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);
			SOURCE.append(prot.getSOURCE());
			
			currentthreadCount++;

		}
		try {
			out.write(SOURCE.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void writeKEYWDS(FileWriter out) {

		try {
			out.write("KEYWDS\t Model described by title\n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void writeEXPDATA(FileWriter out) {
		try {
			out.write("EXPDATA\t Theoretical Model\n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void writeAUTHOR(FileWriter out) {

		try {
			out.write("AUTHOR\t MEMBRANe-EDITOR v1.0\n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeREVDAT(FileWriter out) {

		try {
			out.write("REVDAT\t 1 " + date + " 0 \n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private String writeREMARK1(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeREMARK2(FileWriter out) {
		// TODO Auto-generated method stub

		try {
			out.write("REMARK   2\nREMARK   2 RESOLUTION. NOT APPLICABLE.\n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void writeREMARK3(FileWriter out) {
		// TODO Auto-generated method stub

		try {
			out
					.write("REMARK   3\nREMARK   3 REFINEMENT.\nREMARK   3   PROGRAM     :Membrane-Editor\n"
							+ "REMARK   3   AUTHORS     :Membrane-Editor\nREMARK   3\nREMARK   3 FREE TEXT;\n");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private String writeREMARKN(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeBREF(FileWriter out) {
		int pos = 12;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			a++;

			if (prot.getDBREF().length() != 0) {
				b = a;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}
					currentthreadCount++;
					StringBuffer current = new StringBuffer(prot.getDBREF());
					current.setCharAt(pos, b);
					current.replace(20, 24, spaceGenerater(
							(Double.valueOf(current.substring(20, 24))
									.intValue() * prot_count), 4));
					LINK.append(current);
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private String writeSEQADV(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeSEQRES(FileWriter out) {

		int pos = 11;

		StringBuffer LINK = new StringBuffer();

		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);
			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getSEQRES().length() != 0) {

				b = a;
				int line = 1;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						StringTokenizer tok = new StringTokenizer(prot
								.getSEQRES(), "\n", false);

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							numSEQRESRecord++;
							current.setCharAt(pos, b);
							current.replace(13, 17, spaceGenerater((Double
									.valueOf(current.substring(13, 17))
									.intValue() * prot_count), 4));
							// current.replace(8, 10, spaceGenerater(line, 2));
							LINK.append(current + "\n");
							line++;
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String writeMODRES(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeHET(FileWriter out) {

		int pos = 12;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getHET().length() != 0) {

				b = a;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						StringTokenizer tok = new StringTokenizer(
								prot.getHET(), "\n", false);

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							numHETRecord++;

							current.setCharAt(pos, b);
							current.replace(13, 17, spaceGenerater((Double
									.valueOf(current.substring(13, 17))
									.intValue() + (prot
									.getResidueSequenceCount() * h)), 4));
							LINK.append(current + "\n");
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeHETNAM(FileWriter out) {

		StringBuffer HETNAM = new StringBuffer();

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);
			if (prot.getHETNAM().length() != 0) {

				StringTokenizer tok = new StringTokenizer(prot.getHETNAM(),
						"\n", false);

				while (tok.hasMoreTokens()) {

					String token = tok.nextToken();
					if (!HETNAM.toString().matches("(?i).*" + token + ".*")) {
						HETNAM.append(prot.getHETNAM());
						currentthreadCount++;

					}
				}
			}
		}
		try {
			out.write(HETNAM.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String writeHETSYN(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private String writeFORMUL(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeHELIX(FileWriter out) {

		int pos = 19;
		int pos2 = 31;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getHELIX().length() != 0) {
				int line = 1;
				b = a;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						StringTokenizer tok = new StringTokenizer(prot
								.getHELIX(), "\n", false);

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							numHELIXRecord++;

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(21, 25)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(33, 37)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(21, 25,
											spaceGenerater(residue1, 4));

							current
									.replace(33, 37,
											spaceGenerater(residue2, 4));

							current.replace(7, 10, spaceGenerater(line, 3));
							LINK.append(current + "\n");
							line++;
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeSHEET(FileWriter out) {

		int pos = 21;
		int pos2 = 32;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getSHEET().length() != 0) {
				b = a;
				int line = 1;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						StringTokenizer tok = new StringTokenizer(prot
								.getSHEET(), "\n", false);
						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							numSHEETRecord++;
							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(22, 26)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(33, 37)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(22, 26,
											spaceGenerater(residue1, 4));

							current
									.replace(33, 37,
											spaceGenerater(residue2, 4));

							if (current.length() > 67
									&& current.charAt(68) != ' ') {

								residue1 = (Double.valueOf(
										current.substring(50, 54)).intValue() + (prot
										.getResidueSequenceCount() * h));
								residue2 = (Double.valueOf(
										current.substring(65, 69)).intValue() + (prot
										.getResidueSequenceCount() * h));

								current.replace(50, 54, spaceGenerater(
										residue1, 4));

								current.replace(65, 69, spaceGenerater(
										residue2, 4));
								
								current.setCharAt(49, b);
								current.setCharAt(64, b);

							}
							current.replace(7, 10, spaceGenerater(line, 3));
							LINK.append(current + "\n");
							line++;
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeTURN(FileWriter out) {

		int pos = 19;
		int pos2 = 30;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;
			int line = 1;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getTURN().length() != 0) {
				b = a;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						StringTokenizer tok = new StringTokenizer(prot
								.getTURN(), "\n", false);

						while (tok.hasMoreTokens()) {
							line++;
							StringBuffer current = new StringBuffer(tok
									.nextToken());

							numTURNRecord++;

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);

							int residue1 = (Double.valueOf(
									current.substring(20, 24)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(31, 35)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(20, 24,
											spaceGenerater(residue1, 4));
							current
									.replace(31, 35,
											spaceGenerater(residue2, 4));

							current.replace(7, 10, spaceGenerater(line, 3));
							LINK.append(current + "\n");
							line++;
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeSSBOND(FileWriter out) {

		int pos = 15;
		int pos2 = 29;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;
			int line = 1;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getSSBOND().length() != 0) {
				b = a;
				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						StringTokenizer tok = new StringTokenizer(prot
								.getSSBOND(), "\n", false);

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(17, 21)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(31, 35)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(17, 21,
											spaceGenerater(residue1, 4));
							current
									.replace(31, 35,
											spaceGenerater(residue2, 4));
							current.replace(7, 10, spaceGenerater(line, 3));
							LINK.append(current + "\n");
							line++;
							currentthreadCount++;
						}
					}
					b++;
				}
				for (int j = 1; j < entries; j++) {
					a++;
				}
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeLINK(FileWriter out) {
		int pos = 21;
		int pos2 = 51;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getLINK().length() != 0) {
				b = a;
				StringTokenizer tok = new StringTokenizer(prot.getLINK(), "\n",
						false);
				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(22, 26)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(52, 56)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(22, 26,
											spaceGenerater(residue1, 4));
							current
									.replace(52, 56,
											spaceGenerater(residue2, 4));

							LINK.append(current + "\n");
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeHYDBND(FileWriter out) {
		int pos = 21;
		int pos2 = 52;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getHYDBND().length() != 0) {
				b = a;
				StringTokenizer tok = new StringTokenizer(prot.getHYDBND(),
						"\n", false);
				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}
					for (int h = 0; h < prot_count; h++) {

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(22, 27)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(53, 58)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(22, 27,
											spaceGenerater(residue1, 4));
							current
									.replace(53, 58,
											spaceGenerater(residue2, 4));

							LINK.append(current + "\n");
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeSLTBRG(FileWriter out) {
		int pos = 21;
		int pos2 = 51;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getSLTBRG().length() != 0) {

				b = a;
				StringTokenizer tok = new StringTokenizer(prot.getSLTBRG(),
						"\n", false);
				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}
					for (int h = 0; h < prot_count; h++) {

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(22, 26)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(52, 56)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(22, 26,
											spaceGenerater(residue1, 4));
							current
									.replace(52, 56,
											spaceGenerater(residue2, 4));

							LINK.append(current + "\n");
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeCISPEP(FileWriter out) {
		int pos = 15;
		int pos2 = 29;

		StringBuffer LINK = new StringBuffer();
		char a = '@';
		char b = '@';

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {
			Protein prot = (Protein) vek.elementAt(vekCount);

			a++;
			int line = 1;

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			if (prot.getCISPEP().length() != 0) {
				b = a;
				StringTokenizer tok = new StringTokenizer(prot.getCISPEP(),
						"\n", false);

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				for (int i = 0; i < entries; i++) {

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}
					for (int h = 0; h < prot_count; h++) {

						while (tok.hasMoreTokens()) {
							StringBuffer current = new StringBuffer(tok
									.nextToken());

							current.setCharAt(pos, b);
							current.setCharAt(pos2, b);
							int residue1 = (Double.valueOf(
									current.substring(17, 21)).intValue() + (prot
									.getResidueSequenceCount() * h));
							int residue2 = (Double.valueOf(
									current.substring(31, 35)).intValue() + (prot
									.getResidueSequenceCount() * h));

							current
									.replace(17, 21,
											spaceGenerater(residue1, 4));
							current
									.replace(31, 35,
											spaceGenerater(residue2, 4));

							current.replace(7, 10, spaceGenerater(line, 3));
							LINK.append(current + "\n");
							line++;
							currentthreadCount++;
						}
					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String writeSITE(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeCRYST1(FileWriter out) {

		try {
			out
					.write("CRYST1    1.000    1.000    1.000  90.00  90.00  90.00 P 1           1\n");
			currentthreadCount++;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeORIGX(FileWriter out) {

		try {
			out
					.write("ORIGX1      1.000000  0.000000  0.000000        0.00000\n"
							+ "ORIGX2      0.000000  1.000000  0.000000        0.00000\n"
							+ "ORIGX3      0.000000  0.000000  1.000000        0.00000\n");
			currentthreadCount++;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void writeSCALE(FileWriter out) {

		try {
			out
					.write("SCALE1      1.000000  0.000000  0.000000        0.00000\n"
							+ "SCALE2      0.000000  1.000000  0.000000        0.00000\n"
							+ "SCALE3      0.000000  0.000000  1.000000        0.00000\n");
			currentthreadCount++;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String writeMTRIX(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private String writeTVECT(FileWriter out) {
		// TODO Auto-generated method stub
		return null;
	}

	private void writeMODEL(FileWriter out) {

		membran.complete_scaling_proteins();

		int pos = 21;
		int line = 1;
		int modelCount = 0;

		StringBuffer LINK;

		char a = '@';
		char b = '@';

		LINK = new StringBuffer();

		for (int vekCount = 0; vekCount < vek.size(); vekCount++) {

			Protein prot = (Protein) vek.elementAt(vekCount);
		

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			double X = 0;
			double Y = 0;
			double Z = 0;

			boolean firstLine = true;
			boolean wasIn = false;
			a++;

			if (prot.getSuperAtom().length() != 0) {

				Vector proteinPosition = prot.getProteinPositions();
				int prot_Count = -1;

				int entries_per_loop = 0;
				int protein_positions = prot.getProteinPositions().size();
				int prot_count = 0;

				int protein_position_iterator = 0;
				b = a;

				for (int i = 0; i < entries; i++) {

					line = 1;

					entries_per_loop = entries_per_loop + loops;

					if (entries_per_loop <= protein_positions) {
						prot_count = loops;
					} else {
						prot_count = protein_positions
								- (entries_per_loop - loops);
					}

					for (int h = 0; h < prot_count; h++) {

						int prot_coordinates_position = -1;
						prot_Count++;
						int[] intArray = (int[]) proteinPosition
								.elementAt(protein_position_iterator++);

						double arrayShift_X = intArray[0] * lipid_X;
						double arrayShift_Z = intArray[1] * lipid_Z;

						Vector[] prot_coordinates;

						if (intArray[2] != 0) {
							// System.out.println("Degreeangabe,
							// prot.getRotatedCoordinatesforWriter wird
							// aufgerufen: "+ intArray[2]);
							prot_coordinates = prot.getRotatedCoordinatesforWriter(intArray[2]);
							// System.out.println("v1[] lngen: " +
							// prot_coordinates[0].size() +", " +
							// prot_coordinates[1].size());
						} else {
							// System.out.println("keine Degreeangabe!!!");
							prot_coordinates = prot.getCoordinates();
							// System.out.println("Atomzahl: " +
							// prot_coordinates[1].size());
						}

						nummCoordRecord = 0;
						numTerRecord = 0;
						numConectRecord = 0;

						StringTokenizer tok = new StringTokenizer(prot
								.getSuperAtom(), "\n", false);
						char test = '@';

						while (tok.hasMoreTokens()) {

							StringBuffer current = new StringBuffer(tok
									.nextToken());

							if (firstLine) {
								modelCount++;
								firstLine = false;

								if (current.toString().startsWith("MODEL")) {
									// LINK.append("MODEL " + modelCount +
									// "\n");
								} else if (current.toString()
										.startsWith("ATOM")) {

									line++;
									// LINK.append("MODEL " + modelCount +
									// "\n");
									current.setCharAt(pos, b);

									prot_coordinates_position++;

									X = ((Double) prot_coordinates[1]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_X;
									Y = ((Double) prot_coordinates[2]
											.get(prot_coordinates_position))
											.doubleValue()
											+ 500
											+ prot.getYTranslation()
											+ (lipid.getSize()[1] * 2 - prot
													.getSize()[1]) / 2;
									Z = ((Double) prot_coordinates[3]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_Z;

									current.replace(30, 38, spaceGeneratorAtom(
											X, 8));
									current.replace(38, 46, spaceGeneratorAtom(
											Y, 8));
									current.replace(46, 54, spaceGeneratorAtom(
											Z, 8));

									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,
											5));
									LINK.append(current + "\n");
									currentthreadCount++;
									wasIn = true;
									nummCoordRecord++;
								} else if (current.toString().startsWith(
										"HETATM")) {
									line++;

									// LINK.append("MODEL " + modelCount +
									// "\n");
									current.setCharAt(pos, b);

									prot_coordinates_position++;
									
									X = ((Double) prot_coordinates[1]
											.get(prot_coordinates_position))
											.doubleValue() + arrayShift_X;
									// +500, weil es keine negativen zahlen geben
									// darf.
									// deswegen wird es statisch skaliert
									X = ((Double) prot_coordinates[1]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_X;
									Y = ((Double) prot_coordinates[2]
											.get(prot_coordinates_position))
											.doubleValue()
											+ 500
											+ prot.getYTranslation()
											+ (lipid.getSize()[1] * 2 - prot
													.getSize()[1]) / 2;
									/**if (prot_coordinates_position < 5) {
										System.out.println("Y ist nachher " +Y);
									}*/
									Z = ((Double) prot_coordinates[3]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_Z;
									
									current.replace(30, 38, spaceGeneratorAtom(
											X, 8));
									current.replace(38, 46, spaceGeneratorAtom(
											Y, 8));
									current.replace(46, 54, spaceGeneratorAtom(
											Z, 8));

									/**this creates buggy results in amira and 
									 * i do not know the intention of this (maybe Sebastian does): 
									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,5));*/
									
									LINK.append(current + "\n");
									wasIn = true;
									nummCoordRecord++;
									currentthreadCount++;

								}

							// if not first line
							} else {

								if (current.toString().startsWith("MODEL")) {
									modelCount++;
									// LINK.append("MODEL " + modelCount +
									// "\n");

								} else if (current.toString().startsWith(
										"ENDMDL")) {
									// LINK.append("ENDMDL\n");
								} else if (current.toString()
										.startsWith("ATOM")) {
									nummCoordRecord++;

									line++;

									current.setCharAt(pos, b);
									prot_coordinates_position++;

									X = ((Double) prot_coordinates[1]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_X;
									Y = ((Double) prot_coordinates[2]
											.get(prot_coordinates_position))
											.doubleValue()
											+ 500
											+ prot.getYTranslation()
											+ (lipid.getSize()[1] * 2 - prot
													.getSize()[1]) / 2;
									Z = ((Double) prot_coordinates[3]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_Z;

									current.replace(30, 38, spaceGeneratorAtom(
											X, 8));
									current.replace(38, 46, spaceGeneratorAtom(
											Y, 8));
									current.replace(46, 54, spaceGeneratorAtom(
											Z, 8));
									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,
											5));
									LINK.append(current + "\n");
									currentthreadCount++;
								} else if (current.toString().startsWith("TER")) {
									numTerRecord++;
									current.setCharAt(pos, b);

									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,
											5));
									LINK.append(current + "\n");
									currentthreadCount++;
									line++;
								} else if (current.toString().startsWith(
										"HETATM")) {
									nummCoordRecord++;

									current.setCharAt(pos, b);
									prot_coordinates_position++;

									X = ((Double) prot_coordinates[1]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_X;
									Y = ((Double) prot_coordinates[2]
											.get(prot_coordinates_position))
											.doubleValue()
											+ 500
											+ prot.getYTranslation()
											+ (lipid.getSize()[1] * 2 - prot
													.getSize()[1]) / 2;
									Z = ((Double) prot_coordinates[3]
											.get(prot_coordinates_position))
											.doubleValue()
											+ arrayShift_Z;

									current.replace(30, 38, spaceGeneratorAtom(
											X, 8));
									current.replace(38, 46, spaceGeneratorAtom(
											Y, 8));
									current.replace(46, 54, spaceGeneratorAtom(
											Z, 8));
									
									/**this creates buggy results in amira and 
									 * i do not know the intention of this (maybe Sebastian does): 
									 current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,5));*/
									
									LINK.append(current + "\n");
									currentthreadCount++;
									line++;
								} else if (current.toString().startsWith(
										"SIGATM")) {
									current.setCharAt(pos, b);

									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,
											5));
									LINK.append(current + "\n");
									line++;
									currentthreadCount++;
								} else if (current.toString().startsWith(
										"CONECT")) {
									currentthreadCount++;
									LINK.append(current + "\n");
								}

								else if (current.toString()
										.startsWith("ANISOU")) {
									current.setCharAt(pos, b);

									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,
											5));
									LINK.append(current + "\n");
									currentthreadCount++;
									line++;
								} else if (current.toString().startsWith(
										"SIGUIJ")) {
									current.setCharAt(pos, b);

									current
											.replace(
													22,
													26,
													spaceGenerater(
															(Double
																	.valueOf(
																			current
																					.substring(
																							22,
																							26))
																	.intValue() + (prot
																	.getResidueSequenceCount() * h)),
															4));
									current.replace(6, 11, spaceGenerater(line,
											5));
									LINK.append(current + "\n");
									currentthreadCount++;
									line++;
								}
							}
						}

						// v.1.0: if (LINK.length() > 5000000) {
						if (LINK.length() > 500000) {
							try {
								out.write(LINK.toString());
								LINK = new StringBuffer();
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}

						}

					}
					b++;
				}
			}
			for (int j = 1; j < entries; j++) {
				a++;
			}
		}
		try {
			out.write(LINK.toString());

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		writeLipid(a, out);
	}

	private void writeLipid(char x, FileWriter out) {

		double lipid_shift_y = lipid.getSize()[1];
//		System.out.println("Lipid_positions: " + lipid.getProteinPositions());
//		System.out.println("lipid.matrix: " + lipid.getRotationMatrix());
		
		char a = ++x;
/*//----------------------------------------
		Matrix3f bim0 = new Matrix3f();
		bim0.rotZ((float)(Math.PI*1.5));
		lipid.lipidScaling(bim0);
//-----------------------------------------		
*/		int r = 2;
		for (int i = 1; i >= 0; i--) {

			r--;
			int pos = 21;
			int line = 1;

			StringBuffer LIPID = new StringBuffer();
			
			Protein prot = lipid;
			//lipid.completeScaling();
			
			if(i==0){
				Matrix3f bim = new Matrix3f();
				bim.rotZ((float)Math.PI);
				lipid.lipidScaling(bim);
			}

			line = 1;

			double X = 0;
			double Y = 0;
			double Z = 0;

			Vector[] prot_coordinates = prot.getCoordinates();			

			int loops = 100000 / prot.getAtom_count();
			int entries = (int) Math.ceil((double) prot.getProteinPositions()
					.size()
					/ loops);

			Vector proteinPosition = prot.getProteinPositions();
			int prot_Count = -1;

			int entries_per_loop = 0;
			int protein_positions = proteinPosition.size();
			//System.out.println("protein_positions anzahl: " + protein_positions);
			int prot_count = 0;

			int protein_position_iterator = 0;

			for (int k = 0; k < entries; k++) {

				line = 1;

				entries_per_loop = entries_per_loop + loops;

				if (entries_per_loop <= protein_positions) {
					prot_count = loops;
				} else {
					prot_count = protein_positions - (entries_per_loop - loops);
				}

				for (int h = 0; h < prot_count; h++) {

					int prot_coordinates_position = -1;
					prot_Count++;
					int[] intArray = (int[]) proteinPosition
							.elementAt(protein_position_iterator++);

					double arrayShift_X = intArray[0] * lipid_X;
					double arrayShift_Z = intArray[1] * lipid_Z;

					nummCoordRecord = 0;
					numTerRecord = 0;
					numConectRecord = 0;

					StringTokenizer tok = new StringTokenizer(prot
							.getSuperAtom(), "\n", false);

					while (tok.hasMoreTokens()) {

						StringBuffer current = new StringBuffer(tok.nextToken());
						{

							if (current.toString().startsWith("HETATM")) {

								current.setCharAt(pos, a);
								prot_coordinates_position++;
								X = ((Double) prot_coordinates[1]
										.get(prot_coordinates_position))
										.doubleValue() + arrayShift_X;
								// +500, weil es keine negativen zahlen geben
								// darf.
								// deswegen wird es statisch skaliert
								Y = ((Double) prot_coordinates[2]
										.get(prot_coordinates_position))
										.doubleValue() + 500 + (i*(lipid_shift_y + membran.getBiLayerSpace()));
								Z = ((Double) prot_coordinates[3]
										.get(prot_coordinates_position))
										.doubleValue() + arrayShift_Z;

								current.replace(30, 38,
										spaceGeneratorAtom(X, 8));
								current.replace(38, 46,
										spaceGeneratorAtom(Y, 8));
								current.replace(46, 54,
										spaceGeneratorAtom(Z, 8));
								// current.replace(6, 11, spaceGenerater(line,
								// 5));
								LIPID.append(current + "\n");
								currentthreadCount++;
								line++;
							}
						}

						if (prot_Count % 100 == 0) {
							try {
								out.write(LIPID.toString());
								LIPID = new StringBuffer();
							} catch (IOException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}

					}
				}
				try {
					out.write(LIPID.toString());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				a++;
			}
//			backrotating of the coordinates
			if(i==0){
				Matrix3f bim = new Matrix3f();
				bim.rotZ((float)-Math.PI);
				lipid.lipidScaling(bim);
			}
		}
	}

	private String writeCONECT(FileWriter out) {

		return " ";
	}

	private void writeMASTER(FileWriter out) {

		StringBuffer master = new StringBuffer();
		master.append("MASTER");
		master.append(spaceGenerater(remarkRecord, 5));
		master.append(spaceGenerater(0, 5));
		master.append(spaceGenerater(numHETRecord, 5));
		master.append(spaceGenerater(numHELIXRecord, 5));
		master.append(spaceGenerater(numSHEETRecord, 5));
		master.append(spaceGenerater(numTURNRecord, 5));
		master.append(spaceGenerater(numSITERecord, 5));
		master.append(spaceGenerater(numXformRecord, 5));
		master.append(spaceGenerater(nummCoordRecord, 5));
		master.append(spaceGenerater(numTerRecord, 5));
		master.append(spaceGenerater(numConectRecord, 5));
		master.append(spaceGenerater(numSEQRESRecord, 5));
		master.append("\n");
		currentthreadCount++;
		
		try {
			out.write(master.toString());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeEND(FileWriter out) {

		try {
			out.write("END");
			currentthreadCount++;
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String spaceGenerater(int value, int length) {
		String space = "";

		String line_str = String.valueOf(value);

		for (int h = 0; h < (length - line_str.length()); h++) {
			space = space + " ";
		}
		line_str = space + line_str;
		return line_str;
	}

	private String spaceGeneratorAtom(double value, int length) {

		value = Math.round(value * 1000.) / 1000.;
		String line_str = String.valueOf(value);

		if (line_str.length() == 9) {
			value = Math.round(value * 100.) / 100.;
			line_str = String.valueOf(value);
		} else if (line_str.length() == 10) {
			value = Math.round(value * 10.) / 10.;
			line_str = String.valueOf(value);
		} else if (line_str.length() == 11) {
			value = Math.round(value * 1.) / 1.;
			line_str = String.valueOf(value);
		}

		String space = "";

		for (int h = 0; h < (length - line_str.length()); h++) {
			space = space + " ";
		}
		line_str = space + line_str;

		return line_str;

	}

//	Methode drinn lassen!
	/**
	 * @return Returns the filename.
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 * @return Returns the currentthreadCount.
	 */
	public float getCurrentthreadCount() {
		return currentthreadCount;
	}
}
